@@ -1,8 +1,9 @@
-{-# LANGUAGE FlexibleInstances, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.FlexibleManipulate
+-- Description :  Move and resize floating windows without warping the mouse.
 -- Copyright   :  (c) Michael Sloan
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -23,8 +24,9 @@ module XMonad.Actions.FlexibleManipulate (
 ) where
 
 import XMonad
+import XMonad.Prelude ((<&>))
 import qualified Prelude as P
-import Prelude (($), (.), fst, snd, uncurry, const, id, Ord(..), Monad(..), fromIntegral, Double, Integer, map, round, otherwise)
+import Prelude (Double, Integer, Ord (..), const, fromIntegral, fst, id, map, otherwise, round, snd, uncurry, ($), (.))
 
 -- $usage
 -- First, add this import to your @~\/.xmonad\/xmonad.hs@:
@@ -79,24 +81,23 @@ position = const 0.5
 --   manipulation action.
 mouseWindow :: (Double -> Double) -> Window -> X ()
 mouseWindow f w = whenX (isClient w) $ withDisplay $ \d -> do
-    io $ raiseWindow d w
-    [wpos, wsize] <- io $ getWindowAttributes d w >>= return . winAttrs
+    [wpos, wsize] <- io $ getWindowAttributes d w <&> winAttrs
     sh <- io $ getWMNormalHints d w
-    pointer <- io $ queryPointer d w >>= return . pointerPos
+    pointer <- io $ queryPointer d w <&> pointerPos
 
     let uv = (pointer - wpos) / wsize
         fc = mapP f uv
         mul = mapP (\x -> 2 P.- 2 P.* P.abs(x P.- 0.5)) fc --Fudge factors: interpolation between 1 when on edge, 2 in middle
         atl = ((1, 1) - fc) * mul
         abr = fc * mul
-    mouseDrag (\ex ey -> io $ do
+    mouseDrag (\ex ey -> do
         let offset = (fromIntegral ex, fromIntegral ey) - pointer
             npos = wpos + offset * atl
             nbr = (wpos + wsize) + offset * abr
             ntl = minP (nbr - (32, 32)) npos    --minimum size
             nwidth = applySizeHintsContents sh $ mapP (round :: Double -> Integer) (nbr - ntl)
-        moveResizeWindow d w (round $ fst ntl) (round $ snd ntl) `uncurry` nwidth
-        return ())
+        io $ moveResizeWindow d w (round $ fst ntl) (round $ snd ntl) `uncurry` nwidth
+        float w)
         (float w)
 
     float w
@@ -113,7 +114,7 @@ type Pnt = (Double, Double)
 pairUp :: [a] -> [(a,a)]
 pairUp [] = []
 pairUp [_] = []
-pairUp (x:y:xs) = (x, y) : (pairUp xs)
+pairUp (x:y:xs) = (x, y) : pairUp xs
 
 mapP :: (a -> b) -> (a, a) -> (b, b)
 mapP f (x, y) = (f x, f y)
@@ -132,4 +133,3 @@ infixl 7  *, /
 (*) = zipP (P.*)
 (/) :: (P.Fractional a) => (a,a) -> (a,a) -> (a,a)
 (/) = zipP (P./)
-
