@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.FlexibleResize
+-- Description :  Resize floating windows from any corner.
 -- Copyright   :  (c) Lukas Mai
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -20,7 +21,7 @@ module XMonad.Actions.FlexibleResize (
 ) where
 
 import XMonad
-import XMonad.Util.XUtils (fi)
+import XMonad.Prelude (fi)
 import Foreign.C.Types
 
 -- $usage
@@ -50,7 +51,6 @@ mouseResizeEdgeWindow
   -> Window   -- ^ The window to resize.
   -> X ()
 mouseResizeEdgeWindow edge w = whenX (isClient w) $ withDisplay $ \d -> do
-    io $ raiseWindow d w
     wa <- io $ getWindowAttributes d w
     sh <- io $ getWMNormalHints d w
     (_, _, _, _, _, ix, iy, _) <- io $ queryPointer d w
@@ -62,16 +62,17 @@ mouseResizeEdgeWindow edge w = whenX (isClient w) $ withDisplay $ \d -> do
         (cy, fy, gy) = mkSel north height pos_y
     io $ warpPointer d none w 0 0 0 0 cx cy
     mouseDrag (\ex ey -> do let (nw,nh) = applySizeHintsContents sh (gx ex, gy ey)
-                            io $ moveResizeWindow d w (fx nw) (fy nh) nw nh)
+                            io $ moveResizeWindow d w (fx nw) (fy nh) nw nh
+                            float w)
               (float w)
     where
     findPos :: CInt -> Position -> Maybe Bool
-    findPos m s = if p < 0.5 - edge/2
-                  then Just True
-                  else if p < 0.5 + edge/2
-                       then Nothing
-                       else Just False
-                  where p = fi m / fi s
+    findPos m s
+      | p < 0.5 - edge/2 = Just True
+      | p < 0.5 + edge/2 = Nothing
+      | otherwise = Just False
+      where
+          p = fi m / fi s
     mkSel :: Maybe Bool -> Position -> Position -> (Position, Dimension -> Position, Position -> Dimension)
     mkSel b k p = case b of
                       Just True ->  (0, (fi k + fi p -).fi, (fi k + fi p -).fi)
