@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Actions.FloatKeys
+-- Description  :  Move and resize floating windows.
 -- Copyright    : (c) Karsten Schoelzel <kuser@gmx.de>
 -- License      : BSD
 --
@@ -22,6 +23,7 @@ module XMonad.Actions.FloatKeys (
                 ) where
 
 import XMonad
+import XMonad.Prelude (fi)
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -43,10 +45,9 @@ import XMonad
 --   right and @dy@ pixels down.
 keysMoveWindow :: D -> Window -> X ()
 keysMoveWindow (dx,dy) w = whenX (isClient w) $ withDisplay $ \d -> do
-    io $ raiseWindow d w
     wa <- io $ getWindowAttributes d w
-    io $ moveWindow d w (fromIntegral (fromIntegral (wa_x wa) + dx))
-                        (fromIntegral (fromIntegral (wa_y wa) + dy))
+    io $ moveWindow d w (fi (fi (wa_x wa) + dx))
+                        (fi (fi (wa_y wa) + dy))
     float w
 
 -- | @keysMoveWindowTo (x, y) (gx, gy)@ moves the window relative
@@ -61,14 +62,14 @@ keysMoveWindow (dx,dy) w = whenX (isClient w) $ withDisplay $ \d -> do
 -- > keysMoveWindowTo (1024,0) (1, 0)      -- put window in the top right corner
 keysMoveWindowTo :: P -> G -> Window -> X ()
 keysMoveWindowTo (x,y) (gx, gy) w = whenX (isClient w) $ withDisplay $ \d -> do
-    io $ raiseWindow d w
     wa <- io $ getWindowAttributes d w
-    io $ moveWindow d w (x - round (gx * fromIntegral (wa_width wa)))
-                        (y - round (gy * fromIntegral (wa_height wa)))
+    io $ moveWindow d w (x - round (gx * fi (wa_width wa)))
+                        (y - round (gy * fi (wa_height wa)))
     float w
 
 type G = (Rational, Rational)
 type P = (Position, Position)
+type ChangeDim = (Int, Int)
 
 -- | @keysResizeWindow (dx, dy) (gx, gy)@ changes the width by @dx@
 --   and the height by @dy@, leaving the window-relative point @(gx,
@@ -80,7 +81,7 @@ type P = (Position, Position)
 -- > keysResizeWindow (10, 0) (0, 1%2)    -- does the same, unless sizeHints are applied
 -- > keysResizeWindow (10, 10) (1%2, 1%2) -- add 5 pixels on each side
 -- > keysResizeWindow (-10, -10) (0, 1)   -- shrink the window in direction of the bottom-left corner
-keysResizeWindow :: D -> G -> Window -> X ()
+keysResizeWindow :: ChangeDim -> G -> Window -> X ()
 keysResizeWindow = keysMoveResize keysResizeWindow'
 
 -- | @keysAbsResizeWindow (dx, dy) (ax, ay)@ changes the width by @dx@
@@ -90,34 +91,34 @@ keysResizeWindow = keysMoveResize keysResizeWindow'
 --   For example:
 --
 -- > keysAbsResizeWindow (10, 10) (0, 0)   -- enlarge the window; if it is not in the top-left corner it will also be moved down and to the right.
-keysAbsResizeWindow :: D -> D -> Window -> X ()
+keysAbsResizeWindow :: ChangeDim -> D -> Window -> X ()
 keysAbsResizeWindow = keysMoveResize keysAbsResizeWindow'
 
-keysAbsResizeWindow' :: SizeHints -> P -> D -> D -> D -> (P,D)
+keysAbsResizeWindow' :: SizeHints -> P -> D -> ChangeDim -> D -> (P,D)
 keysAbsResizeWindow' sh (x,y) (w,h) (dx,dy) (ax, ay) = ((round nx, round ny), (nw, nh))
     where
-        (nw, nh) = applySizeHintsContents sh (w + dx, h + dy)
+        -- The width and height of a window are positive and thus
+        -- converting to 'Dimension' should be safe.
+        (nw, nh) = applySizeHintsContents sh (fi w + dx, fi h + dy)
         nx :: Rational
-        nx = fromIntegral (ax * w + nw * (fromIntegral x - ax)) / fromIntegral w
+        nx = fi (ax * w + nw * (fi x - ax)) / fi w
         ny :: Rational
-        ny = fromIntegral (ay * h + nh * (fromIntegral y - ay)) / fromIntegral h
+        ny = fi (ay * h + nh * (fi y - ay)) / fi h
 
-keysResizeWindow' :: SizeHints -> P -> D -> D -> G -> (P,D)
+keysResizeWindow' :: SizeHints -> P -> D -> ChangeDim -> G -> (P,D)
 keysResizeWindow' sh (x,y) (w,h) (dx,dy) (gx, gy) = ((nx, ny), (nw, nh))
     where
-        (nw, nh) = applySizeHintsContents sh (w + dx, h + dy)
-        nx = round $ fromIntegral x + gx * fromIntegral w - gx * fromIntegral nw
-        ny = round $ fromIntegral y + gy * fromIntegral h - gy * fromIntegral nh
+        (nw, nh) = applySizeHintsContents sh (fi w + dx, fi h + dy)
+        nx = round $ fi x + gx * fi w - gx * fi nw
+        ny = round $ fi y + gy * fi h - gy * fi nh
 
 keysMoveResize :: (SizeHints -> P -> D -> a -> b -> (P,D)) -> a -> b -> Window -> X ()
 keysMoveResize f move resize w = whenX (isClient w) $ withDisplay $ \d -> do
-    io $ raiseWindow d w
     wa <- io $ getWindowAttributes d w
     sh <- io $ getWMNormalHints d w
-    let wa_dim = (fromIntegral $ wa_width wa, fromIntegral $ wa_height wa)
-        wa_pos = (fromIntegral $ wa_x wa, fromIntegral $ wa_y wa)
+    let wa_dim = (fi $ wa_width wa, fi $ wa_height wa)
+        wa_pos = (fi $ wa_x wa, fi $ wa_y wa)
         (wn_pos, wn_dim) = f sh wa_pos wa_dim move resize
     io $ resizeWindow d w `uncurry` wn_dim
     io $ moveWindow d w `uncurry` wn_pos
     float w
-
