@@ -1,7 +1,8 @@
-{-# LANGUAGE ScopedTypeVariables, GeneralizedNewtypeDeriving, TypeSynonymInstances, FlexibleInstances, OverlappingInstances #-}
+{-# LANGUAGE ScopedTypeVariables, GeneralizedNewtypeDeriving, FlexibleInstances, TupleSections #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.GridSelect
+-- Description :  Display items in a 2D grid and select from it with the keyboard or the mouse.
 -- Copyright   :  Clemens Fruhwirth <clemens@endorphin.org>
 -- License     :  BSD-style (see LICENSE)
 --
@@ -28,7 +29,6 @@ module XMonad.Actions.GridSelect (
     -- * Configuration
     GSConfig(..),
     def,
-    defaultGSConfig,
     TwoDPosition,
     buildDefaultGSConfig,
 
@@ -48,6 +48,7 @@ module XMonad.Actions.GridSelect (
     fromClassName,
     stringColorizer,
     colorRangeFromClassName,
+    stringToRatio,
 
     -- * Navigation Mode assembly
     TwoD,
@@ -79,16 +80,14 @@ module XMonad.Actions.GridSelect (
     -- * Types
     TwoDState,
     ) where
-import Data.Maybe
+import Control.Arrow ((***))
 import Data.Bits
-import Data.Char
 import Data.Ord (comparing)
-import Control.Applicative
 import Control.Monad.State
-import Control.Arrow
 import Data.List as L
 import qualified Data.Map as M
 import XMonad hiding (liftX)
+import XMonad.Prelude
 import XMonad.Util.Font
 import XMonad.Prompt (mkUnmanagedWindow)
 import XMonad.StackSet as W
@@ -96,7 +95,7 @@ import XMonad.Layout.Decoration
 import XMonad.Util.NamedWindows
 import XMonad.Actions.WindowBringer (bringWindow)
 import Text.Printf
-import System.Random (mkStdGen, genRange, next)
+import System.Random (mkStdGen, randomR)
 import Data.Word (Word8)
 
 -- $usage
@@ -107,13 +106,13 @@ import Data.Word (Word8)
 --
 -- Then add a keybinding, e.g.
 --
--- >    , ((modm, xK_g), goToSelected defaultGSConfig)
+-- >    , ((modm, xK_g), goToSelected def)
 --
 -- This module also supports displaying arbitrary information in a grid and letting
 -- the user select from it. E.g. to spawn an application from a given list, you
 -- can use the following:
 --
--- >   , ((modm, xK_s), spawnSelected defaultGSConfig ["xterm","gmplayer","gvim"])
+-- >   , ((modm, xK_s), spawnSelected def ["xterm","gmplayer","gvim"])
 
 -- $commonGSConfig
 --
@@ -123,7 +122,7 @@ import Data.Word (Word8)
 -- > {-# LANGUAGE NoMonomorphismRestriction #-}
 -- > import XMonad
 -- > ...
--- > gsconfig1 = defaultGSConfig { gs_cellheight = 30, gs_cellwidth = 100 }
+-- > gsconfig1 = def { gs_cellheight = 30, gs_cellwidth = 100 }
 --
 -- An example where 'buildDefaultGSConfig' is used instead of 'defaultGSConfig'
 -- in order to specify a custom colorizer is @gsconfig2@ (found in
@@ -222,18 +221,14 @@ instance HasColorizer Window where
 instance HasColorizer String where
     defaultColorizer = stringColorizer
 
-instance HasColorizer a where
+instance {-# OVERLAPPABLE #-} HasColorizer a where
     defaultColorizer _ isFg =
         let getColor = if isFg then focusedBorderColor else normalBorderColor
-        in asks $ flip (,) "black" . getColor . config
+        in asks $ (, "black") . getColor . config
 
 instance HasColorizer a => Default (GSConfig a) where
     def = buildDefaultGSConfig defaultColorizer
 
-{-# DEPRECATED defaultGSConfig "Use def (from Data.Default, and re-exported from XMonad.Actions.GridSelect) instead." #-}
-defaultGSConfig :: HasColorizer a => GSConfig a
-defaultGSConfig = def
-
 type TwoDPosition = (Integer, Integer)
 
 type TwoDElementMap a = [(TwoDPosition,(String,a))]
@@ -264,7 +259,7 @@ generateElementmap s = do
     -- Sorts the elementmap
     sortedElements = orderElementmap searchString filteredElements
     -- Case Insensitive version of isInfixOf
-    needle `isInfixOfI` haystack = (upper needle) `isInfixOf` (upper haystack)
+    needle `isInfixOfI` haystack = upper needle `isInfixOf` upper haystack
     upper = map toUpper
 
 
@@ -308,8 +303,8 @@ diamondLayer n =
   -- tr = top right
   --  r = ur ++ 90 degree clock-wise rotation of ur
   let tr = [ (x,n-x) | x <- [0..n-1] ]
-      r  = tr ++ (map (\(x,y) -> (y,-x)) tr)
-  in r ++ (map (negate *** negate) r)
+      r  = tr ++ map (\(x,y) -> (y,-x)) tr
+  in r ++ map (negate *** negate) r
 
 diamond :: (Enum a, Num a, Eq a) => [(a, a)]
 diamond = concatMap diamondLayer [0..]
@@ -339,7 +334,7 @@ drawWinBox win font (fg,bg) bc ch cw text x y cp =
     drawRectangle dpy win bordergc (fromInteger x) (fromInteger y) (fromInteger cw) (fromInteger ch)
   stext <- shrinkWhile (shrinkIt shrinkText)
            (\n -> do size <- liftIO $ textWidthXMF dpy font n
-                     return $ size > (fromInteger (cw-(2*cp))))
+                     return $ size > fromInteger (cw-(2*cp)))
            text
   -- calculate the offset to vertically centre the text based on the ascender and descender
   (asc,desc) <- liftIO $ textExtentsXMF font stext
@@ -392,9 +387,9 @@ updateElementsWithColorizer colorizer elementmap = do
     mapM_ updateElement elementmap
 
 stdHandle :: Event -> TwoD a (Maybe a) -> TwoD a (Maybe a)
-stdHandle (ButtonEvent { ev_event_type = t, ev_x = x, ev_y = y }) contEventloop
+stdHandle ButtonEvent{ ev_event_type = t, ev_x = x, ev_y = y } contEventloop
     | t == buttonRelease = do
-        s @  TwoDState { td_paneX = px, td_paneY = py,
+        s@TwoDState { td_paneX = px, td_paneY = py,
                          td_gsconfig = (GSConfig ch cw _ _ _ _ _ _ _ _) } <- get
         let gridX = (fi x - (px - cw) `div` 2) `div` cw
             gridY = (fi y - (py - ch) `div` 2) `div` ch
@@ -403,7 +398,7 @@ stdHandle (ButtonEvent { ev_event_type = t, ev_x = x, ev_y = y }) contEventloop
              Nothing -> contEventloop
     | otherwise = contEventloop
 
-stdHandle (ExposeEvent { }) contEventloop = updateAllElements >> contEventloop
+stdHandle ExposeEvent{} contEventloop = updateAllElements >> contEventloop
 
 stdHandle _ contEventloop = contEventloop
 
@@ -435,7 +430,7 @@ shadowWithKeymap keymap dflt keyEvent@(ks,_,m') = fromMaybe (dflt keyEvent) (M.l
 select :: TwoD a (Maybe a)
 select = do
   s <- get
-  return $ fmap (snd . snd) $ findInElementMap (td_curpos s) (td_elementmap s)
+  return $ snd . snd <$> findInElementMap (td_curpos s) (td_elementmap s)
 
 -- | Closes gridselect returning no element.
 cancel :: TwoD a (Maybe a)
@@ -450,7 +445,7 @@ setPos newPos = do
       oldPos = td_curpos s
   when (isJust newSelectedEl && newPos /= oldPos) $ do
     put s { td_curpos = newPos }
-    updateElements (catMaybes [(findInElementMap oldPos elmap), newSelectedEl])
+    updateElements (catMaybes [findInElementMap oldPos elmap, newSelectedEl])
 
 -- | Moves the cursor by the offsets specified
 move :: (Integer, Integer) -> TwoD a ()
@@ -550,7 +545,7 @@ navNSearch = makeXEventhandler $ shadowWithKeymap navNSearchKeyMap navNSearchDef
           ,((0,xK_Up)         , move (0,-1) >> navNSearch)
           ,((0,xK_Tab)        , moveNext >> navNSearch)
           ,((shiftMask,xK_Tab), movePrev >> navNSearch)
-          ,((0,xK_BackSpace), transformSearchString (\s -> if (s == "") then "" else init s) >> navNSearch)
+          ,((0,xK_BackSpace), transformSearchString (\s -> if s == "" then "" else init s) >> navNSearch)
           ]
         -- The navigation handler ignores unknown key symbols, therefore we const
         navNSearchDefaultHandler (_,s,_) = do
@@ -564,7 +559,7 @@ substringSearch returnNavigation = fix $ \me ->
   let searchKeyMap = M.fromList [
            ((0,xK_Escape)   , transformSearchString (const "") >> returnNavigation)
           ,((0,xK_Return)   , returnNavigation)
-          ,((0,xK_BackSpace), transformSearchString (\s -> if (s == "") then "" else init s) >> me)
+          ,((0,xK_BackSpace), transformSearchString (\s -> if s == "" then "" else init s) >> me)
           ]
       searchDefaultHandler (_,s,_) = do
           transformSearchString (++ s)
@@ -576,8 +571,8 @@ substringSearch returnNavigation = fix $ \me ->
 -- Conversion scheme as in http://en.wikipedia.org/wiki/HSV_color_space
 hsv2rgb :: Fractional a => (Integer,a,a) -> (a,a,a)
 hsv2rgb (h,s,v) =
-    let hi = (div h 60) `mod` 6 :: Integer
-        f = (((fromInteger h)/60) - (fromInteger hi)) :: Fractional a => a
+    let hi = div h 60 `mod` 6 :: Integer
+        f = ((fromInteger h/60) - fromInteger hi) :: Fractional a => a
         q = v * (1-f)
         p = v * (1-s)
         t = v * (1-(1-f)*s)
@@ -594,19 +589,19 @@ hsv2rgb (h,s,v) =
 stringColorizer :: String -> Bool -> X (String, String)
 stringColorizer s active =
     let seed x = toInteger (sum $ map ((*x).fromEnum) s) :: Integer
-        (r,g,b) = hsv2rgb ((seed 83) `mod` 360,
-                           (fromInteger ((seed 191) `mod` 1000))/2500+0.4,
-                           (fromInteger ((seed 121) `mod` 1000))/2500+0.4)
+        (r,g,b) = hsv2rgb (seed 83 `mod` 360,
+                           fromInteger (seed 191 `mod` 1000)/2500+0.4,
+                           fromInteger (seed 121 `mod` 1000)/2500+0.4)
     in if active
          then return ("#faff69", "black")
-         else return ("#" ++ concat (map (twodigitHex.(round :: Double -> Word8).(*256)) [r, g, b] ), "white")
+         else return ("#" ++ concatMap (twodigitHex.(round :: Double -> Word8).(*256)) [r, g, b], "white")
 
 -- | Colorize a window depending on it's className.
 fromClassName :: Window -> Bool -> X (String, String)
 fromClassName w active = runQuery className w >>= flip defaultColorizer active
 
 twodigitHex :: Word8 -> String
-twodigitHex a = printf "%02x" a
+twodigitHex = printf "%02x"
 
 -- | A colorizer that picks a color inside a range,
 -- and depending on the window's class.
@@ -635,15 +630,12 @@ mix (r1, g1, b1) (r2, g2, b2) r = (mix' r1 r2, mix' g1 g2, mix' b1 b2)
 
 -- | Generates a Double from a string, trying to
 -- achieve a random distribution.
--- We create a random seed from the sum of all characters
+-- We create a random seed from the hash of all characters
 -- in the string, and use it to generate a ratio between 0 and 1
 stringToRatio :: String -> Double
 stringToRatio "" = 0
-stringToRatio s = let gen = mkStdGen $ sum $ map fromEnum s
-                      range = (\(a, b) -> b - a) $ genRange gen
-                      randomInt = foldr1 combine $ replicate 20 next
-                      combine f1 f2 g = let (_, g') = f1 g in f2 g'
-                  in fi (fst $ randomInt gen) / fi range
+stringToRatio s = let gen = mkStdGen $ foldl' (\t c -> t * 31 + fromEnum c) 0 s
+                  in fst $ randomR (0, 1) gen
 
 -- | Brings up a 2D grid of elements in the center of the screen, and one can
 -- select an element with cursors keys. The selected element is returned.
@@ -662,14 +654,14 @@ gridselect gsconfig elements =
     font <- initXMF (gs_font gsconfig)
     let screenWidth = toInteger $ rect_width scr
         screenHeight = toInteger $ rect_height scr
-    selectedElement <- if (status == grabSuccess) then do
+    selectedElement <- if status == grabSuccess then do
                             let restriction ss cs = (fromInteger ss/fromInteger (cs gsconfig)-1)/2 :: Double
                                 restrictX = floor $ restriction screenWidth gs_cellwidth
                                 restrictY = floor $ restriction screenHeight gs_cellheight
-                                originPosX = floor $ ((gs_originFractX gsconfig) - (1/2)) * 2 * fromIntegral restrictX
-                                originPosY = floor $ ((gs_originFractY gsconfig) - (1/2)) * 2 * fromIntegral restrictY
+                                originPosX = floor $ (gs_originFractX gsconfig - (1/2)) * 2 * fromIntegral restrictX
+                                originPosY = floor $ (gs_originFractY gsconfig - (1/2)) * 2 * fromIntegral restrictY
                                 coords = diamondRestrict restrictX restrictY originPosX originPosY
-                                s = TwoDState { td_curpos = (head coords),
+                                s = TwoDState { td_curpos = head coords,
                                                 td_availSlots = coords,
                                                 td_elements = elements,
                                                 td_gsconfig = gsconfig,
@@ -680,7 +672,7 @@ gridselect gsconfig elements =
                                                 td_searchString = "",
                                                 td_elementmap = [] }
                             m <- generateElementmap s
-                            evalTwoD (updateAllElements >> (gs_navigate gsconfig))
+                            evalTwoD (updateAllElements >> gs_navigate gsconfig)
                                      (s { td_elementmap = m })
                       else
                           return Nothing
@@ -702,20 +694,17 @@ gridselectWindow gsconf = windowMap >>= gridselect gsconf
 withSelectedWindow :: (Window -> X ()) -> GSConfig Window -> X ()
 withSelectedWindow callback conf = do
     mbWindow <- gridselectWindow conf
-    case mbWindow of
-        Just w -> callback w
-        Nothing -> return ()
+    for_ mbWindow callback
 
 windowMap :: X [(String,Window)]
 windowMap = do
     ws <- gets windowset
-    wins <- mapM keyValuePair (W.allWindows ws)
-    return wins
- where keyValuePair w = flip (,) w `fmap` decorateName' w
+    mapM keyValuePair (W.allWindows ws)
+ where keyValuePair w = (, w) <$> decorateName' w
 
 decorateName' :: Window -> X String
 decorateName' w = do
-  fmap show $ getName w
+  show <$> getName w
 
 -- | Builds a default gs config from a colorizer function.
 buildDefaultGSConfig :: (a -> Bool -> X (String,String)) -> GSConfig a
@@ -770,7 +759,7 @@ gridselectWorkspace' conf func = withWindowSet $ \ws -> do
 --
 -- > import XMonad.Actions.DynamicWorkspaces (addWorkspace)
 -- >
--- > gridselectWorkspace' defaultGSConfig
+-- > gridselectWorkspace' def
 -- >                          { gs_navigate   = navNSearch
 -- >                          , gs_rearranger = searchStringRearrangerGenerator id
 -- >                          }
@@ -789,7 +778,7 @@ noRearranger _ = return
 -- already present).
 searchStringRearrangerGenerator :: (String -> a) -> Rearranger a
 searchStringRearrangerGenerator f =
-    let r "" xs                       = return $ xs
-        r s  xs | s `elem` map fst xs = return $ xs
+    let r "" xs                       = return xs
+        r s  xs | s `elem` map fst xs = return xs
                 | otherwise           = return $ xs ++ [(s, f s)]
     in r
