@@ -1,4 +1,6 @@
-{- | Module      :  XMonad.Actions.Search
+{- |
+   Module      :  XMonad.Actions.Search
+   Description :  Easily run Internet searches on web sites through xmonad.
    Copyright   :  (C) 2007 Gwern Branwen
    License     :  None; public domain
 
@@ -18,6 +20,7 @@ module XMonad.Actions.Search (   -- * Usage
                                  searchEngineF,
                                  promptSearch,
                                  promptSearchBrowser,
+                                 promptSearchBrowser',
                                  selectSearch,
                                  selectSearchBrowser,
                                  isPrefixOf,
@@ -35,12 +38,13 @@ module XMonad.Actions.Search (   -- * Usage
                                  debbts,
                                  debpts,
                                  dictionary,
+                                 ebay,
+                                 github,
                                  google,
                                  hackage,
                                  hoogle,
                                  images,
                                  imdb,
-                                 isohunt,
                                  lucky,
                                  maps,
                                  mathworld,
@@ -63,13 +67,12 @@ module XMonad.Actions.Search (   -- * Usage
                           ) where
 
 import           Codec.Binary.UTF8.String (encode)
-import           Data.Char                (isAlphaNum, isAscii)
-import           Data.List                (isPrefixOf)
 import           Text.Printf
 import           XMonad                   (X (), liftIO)
 import           XMonad.Prompt            (XPConfig (), XPrompt (showXPrompt, nextCompletion, commandToComplete),
                                            getNextCompletion,
                                            historyCompletionP, mkXPrompt)
+import           XMonad.Prelude           (isAlphaNum, isAscii, isPrefixOf)
 import           XMonad.Prompt.Shell      (getBrowser)
 import           XMonad.Util.Run          (safeSpawn)
 import           XMonad.Util.XSelection   (getSelection)
@@ -113,6 +116,10 @@ import           XMonad.Util.XSelection   (getSelection)
 
 * 'dictionary' -- dictionary.reference.com search.
 
+* 'ebay' -- Ebay keyword search.
+
+* 'github' -- GitHub keyword search.
+
 * 'google' -- basic Google search.
 
 * 'hackage' -- Hackage, the Haskell package database.
@@ -125,8 +132,6 @@ import           XMonad.Util.XSelection   (getSelection)
 
 * 'imdb'   -- the Internet Movie Database.
 
-* 'isohunt' -- isoHunt search.
-
 * 'lucky' -- Google "I'm feeling lucky" search.
 
 * 'maps'   -- Google maps.
@@ -137,7 +142,7 @@ import           XMonad.Util.XSelection   (getSelection)
 
 * 'scholar' -- Google scholar academic search.
 
-* 'thesaurus' -- thesaurus.reference.com search.
+* 'thesaurus' -- thesaurus.com search.
 
 * 'wayback' -- the Wayback Machine.
 
@@ -191,7 +196,7 @@ Or in combination with XMonad.Util.EZConfig:
 >
 > searchList :: [(String, S.SearchEngine)]
 > searchList = [ ("g", S.google)
->              , ("h", S.hoohle)
+>              , ("h", S.hoogle)
 >              , ("w", S.wikipedia)
 >              ]
 
@@ -210,7 +215,7 @@ engine.
 Happy searching! -}
 
 -- | A customized prompt indicating we are searching, and the name of the site.
-data Search = Search Name
+newtype Search = Search Name
 instance XPrompt Search where
     showXPrompt (Search name)= "Search [" ++ name ++ "]: "
     nextCompletion _ = getNextCompletion
@@ -248,7 +253,7 @@ search browser site query = safeSpawn browser [site query]
    appends it to the base. You can easily define a new engine locally using
    exported functions without needing to modify "XMonad.Actions.Search":
 
-> myNewEngine = searchEngine "site" "http://site.com/search="
+> myNewEngine = searchEngine "site" "https://site.com/search="
 
    The important thing is that the site has a interface which accepts the escaped query
    string as part of the URL. Alas, the exact URL to feed searchEngine varies
@@ -257,21 +262,21 @@ search browser site query = safeSpawn browser [site query]
    Generally, examining the resultant URL of a search will allow you to reverse-engineer
    it if you can't find the necessary URL already described in other projects such as Surfraw. -}
 searchEngine :: Name -> String -> SearchEngine
-searchEngine name site = searchEngineF name (\s -> site ++ (escape s))
+searchEngine name site = searchEngineF name (\s -> site ++ escape s)
 
 {- | If your search engine is more complex than this (you may want to identify
    the kind of input and make the search URL dependent on the input or put the query
    inside of a URL instead of in the end) you can use the alternative 'searchEngineF' function.
 
 > searchFunc :: String -> String
-> searchFunc s | "wiki:"   `isPrefixOf` s = "http://en.wikipedia.org/wiki/" ++ (escape $ tail $ snd $ break (==':') s)
->              | "http://" `isPrefixOf` s = s
->              | otherwise               = (use google) s
+> searchFunc s | "wiki:"    `isPrefixOf` s = "https://en.wikipedia.org/wiki/" ++ (escape $ tail $ snd $ break (==':') s)
+>              | "https://" `isPrefixOf` s = s
+>              | otherwise                 = (use google) s
 > myNewEngine = searchEngineF "mymulti" searchFunc
 
    @searchFunc@ here searches for a word in wikipedia if it has a prefix
    of \"wiki:\" (you can use the 'escape' function to escape any forbidden characters), opens an address
-   directly if it starts with \"http:\/\/\" and otherwise uses the provided google search engine.
+   directly if it starts with \"https:\/\/\" and otherwise uses the provided google search engine.
    You can use other engines inside of your own through the 'use' function as shown above to make
    complex searches.
 
@@ -281,38 +286,39 @@ searchEngineF :: Name -> Site -> SearchEngine
 searchEngineF = SearchEngine
 
 -- The engines.
-amazon, alpha, codesearch, deb, debbts, debpts, dictionary, google, hackage, hoogle,
-  images, imdb, isohunt, lucky, maps, mathworld, openstreetmap, scholar, stackage, thesaurus, vocabulary, wayback, wikipedia, wiktionary,
+amazon, alpha, codesearch, deb, debbts, debpts, dictionary, ebay, github, google, hackage, hoogle,
+  images, imdb, lucky, maps, mathworld, openstreetmap, scholar, stackage, thesaurus, vocabulary, wayback, wikipedia, wiktionary,
   youtube, duckduckgo :: SearchEngine
-amazon        = searchEngine "amazon"        "http://www.amazon.com/s/ref=nb_sb_noss_2?url=search-alias%3Daps&field-keywords="
-alpha         = searchEngine "alpha"         "http://www.wolframalpha.com/input/?i="
-codesearch    = searchEngine "codesearch"    "http://www.google.com/codesearch?q="
-deb           = searchEngine "deb"           "http://packages.debian.org/"
-debbts        = searchEngine "debbts"        "http://bugs.debian.org/"
-debpts        = searchEngine "debpts"        "http://packages.qa.debian.org/"
-dictionary    = searchEngine "dict"          "http://dictionary.reference.com/browse/"
-google        = searchEngine "google"        "http://www.google.com/search?num=100&q="
-hackage       = searchEngine "hackage"       "http://hackage.haskell.org/package/"
-hoogle        = searchEngine "hoogle"        "http://www.haskell.org/hoogle/?q="
-images        = searchEngine "images"        "http://images.google.fr/images?q="
-imdb          = searchEngine "imdb"          "http://www.imdb.com/find?s=all&q="
-isohunt       = searchEngine "isohunt"       "http://isohunt.com/torrents/?ihq="
-lucky         = searchEngine "lucky"         "http://www.google.com/search?btnI&q="
-maps          = searchEngine "maps"          "http://maps.google.com/maps?q="
-mathworld     = searchEngine "mathworld"     "http://mathworld.wolfram.com/search/?query="
-openstreetmap = searchEngine "openstreetmap" "http://gazetteer.openstreetmap.org/namefinder/?find="
-scholar       = searchEngine "scholar"       "http://scholar.google.com/scholar?q="
-stackage      = searchEngine "stackage"      "www.stackage.org/lts/hoogle?q="
-thesaurus     = searchEngine "thesaurus"     "http://thesaurus.reference.com/search?q="
-wikipedia     = searchEngine "wiki"          "http://en.wikipedia.org/wiki/Special:Search?go=Go&search="
-wiktionary    = searchEngine "wikt"          "http://en.wiktionary.org/wiki/Special:Search?go=Go&search="
-youtube       = searchEngine "youtube"       "http://www.youtube.com/results?search_type=search_videos&search_query="
-wayback       = searchEngineF "wayback"      ("http://web.archive.org/web/*/"++)
-vocabulary    = searchEngine "vocabulary"    "http://www.vocabulary.com/search?q="
+amazon        = searchEngine "amazon"        "https://www.amazon.com/s/ref=nb_sb_noss_2?url=search-alias%3Daps&field-keywords="
+alpha         = searchEngine "alpha"         "https://www.wolframalpha.com/input/?i="
+codesearch    = searchEngine "codesearch"    "https://developers.google.com/s/results/code-search?q="
+deb           = searchEngine "deb"           "https://packages.debian.org/"
+debbts        = searchEngine "debbts"        "https://bugs.debian.org/"
+debpts        = searchEngine "debpts"        "https://packages.qa.debian.org/"
+dictionary    = searchEngine "dict"          "https://dictionary.reference.com/browse/"
+ebay          = searchEngine "ebay"          "https://www.ebay.com/sch/i.html?_nkw="
+github        = searchEngine "github"        "https://github.com/search?q="
+google        = searchEngine "google"        "https://www.google.com/search?num=100&q="
+hackage       = searchEngine "hackage"       "https://hackage.haskell.org/package/"
+hoogle        = searchEngine "hoogle"        "https://hoogle.haskell.org/?hoogle="
+images        = searchEngine "images"        "https://images.google.fr/images?q="
+imdb          = searchEngine "imdb"          "https://www.imdb.com/find?s=all&q="
+lucky         = searchEngine "lucky"         "https://www.google.com/search?btnI&q="
+maps          = searchEngine "maps"          "https://maps.google.com/maps?q="
+mathworld     = searchEngine "mathworld"     "https://mathworld.wolfram.com/search/?query="
+openstreetmap = searchEngine "openstreetmap" "https://www.openstreetmap.org/search?query="
+scholar       = searchEngine "scholar"       "https://scholar.google.com/scholar?q="
+stackage      = searchEngine "stackage"      "https://www.stackage.org/lts/hoogle?q="
+thesaurus     = searchEngine "thesaurus"     "https://thesaurus.com/browse/"
+wikipedia     = searchEngine "wiki"          "https://en.wikipedia.org/wiki/Special:Search?go=Go&search="
+wiktionary    = searchEngine "wikt"          "https://en.wiktionary.org/wiki/Special:Search?go=Go&search="
+youtube       = searchEngine "youtube"       "https://www.youtube.com/results?search_type=search_videos&search_query="
+wayback       = searchEngineF "wayback"      ("https://web.archive.org/web/*/"++)
+vocabulary    = searchEngine "vocabulary"    "https://www.vocabulary.com/search?q="
 duckduckgo    = searchEngine "duckduckgo"    "https://duckduckgo.com/?t=lm&q="
 
 multi :: SearchEngine
-multi = namedEngine "multi" $ foldr1 (!>) [amazon, alpha, codesearch, deb, debbts, debpts, dictionary, google, hackage, hoogle, images, imdb, isohunt, lucky, maps, mathworld, openstreetmap, scholar, thesaurus, wayback, wikipedia, wiktionary, duckduckgo, (prefixAware google)]
+multi = namedEngine "multi" $ foldr1 (!>) [amazon, alpha, codesearch, deb, debbts, debpts, dictionary, ebay, github, google, hackage, hoogle, images, imdb, lucky, maps, mathworld, openstreetmap, scholar, thesaurus, wayback, wikipedia, wiktionary, duckduckgo, prefixAware google]
 
 {- | This function wraps up a search engine and creates a new one, which works
    like the argument, but goes directly to a URL if one is given rather than
@@ -320,9 +326,9 @@ multi = namedEngine "multi" $ foldr1 (!>) [amazon, alpha, codesearch, deb, debbt
 
 > myIntelligentGoogleEngine = intelligent google
 
-   Now if you search for http:\/\/xmonad.org it will directly open in your browser-}
+   Now if you search for https:\/\/xmonad.org it will directly open in your browser-}
 intelligent :: SearchEngine -> SearchEngine
-intelligent (SearchEngine name site) = searchEngineF name (\s -> if (fst $ break (==':') s) `elem` ["http", "https", "ftp"] then s else (site s))
+intelligent (SearchEngine name site) = searchEngineF name (\s -> if takeWhile (/= ':') s `elem` ["http", "https", "ftp"] then s else site s)
 
 -- | > removeColonPrefix "foo://bar" ~> "//bar"
 -- > removeColonPrefix "foo//bar" ~> "foo//bar"
@@ -342,6 +348,7 @@ removeColonPrefix s = if ':' `elem` s then drop 1 $ dropWhile (':' /=) s else s
   google. The use of intelligent will make sure that URLs are opened directly. -}
 (!>) :: SearchEngine -> SearchEngine -> SearchEngine
 (SearchEngine name1 site1) !> (SearchEngine name2 site2) = searchEngineF (name1 ++ "/" ++ name2) (\s -> if (name1++":") `isPrefixOf` s then site1 (removeColonPrefix s) else site2 s)
+infixr 6 !>
 
 {- | Makes a search engine prefix-aware. Especially useful together with '!>'.
    It will automatically remove the prefix from a query so that you don\'t end
@@ -358,8 +365,18 @@ namedEngine name (SearchEngine _ site) = searchEngineF name site
    Prompt's result, passes it to a given searchEngine and opens it in a given
    browser. -}
 promptSearchBrowser :: XPConfig -> Browser -> SearchEngine -> X ()
-promptSearchBrowser config browser (SearchEngine name site) =
-    mkXPrompt (Search name) config (historyCompletionP ("Search [" `isPrefixOf`)) $ search browser site
+promptSearchBrowser config browser (SearchEngine name site) = do
+    hc <- historyCompletionP ("Search [" `isPrefixOf`)
+    mkXPrompt (Search name) config hc $ search browser site
+
+{- | Like 'promptSearchBrowser', but only suggest previous searches for the
+   given 'SearchEngine' in the prompt. -}
+promptSearchBrowser' :: XPConfig -> Browser -> SearchEngine -> X ()
+promptSearchBrowser' config browser (SearchEngine name site) = do
+    hc <- historyCompletionP (searchName `isPrefixOf`)
+    mkXPrompt (Search name) config hc $ search browser site
+  where
+    searchName = showXPrompt (Search name)
 
 {- | Like 'search', but in this case, the string is not specified but grabbed
  from the user's response to a prompt. Example:
