@@ -1,7 +1,8 @@
-{-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.ShowText
+-- Description :  Display text on the screen.
 -- Copyright   :  (c) Mario Pastorelli (2012)
 -- License     :  BSD-style (see xmonad/LICENSE)
 --
@@ -17,17 +18,15 @@ module XMonad.Actions.ShowText
     ( -- * Usage
       -- $usage
       def
-    , defaultSTConfig
     , handleTimerEvent
     , flashText
     , ShowTextConfig(..)
     ) where
 
-import Control.Monad (when)
 import Data.Map (Map,empty,insert,lookup)
-import Data.Monoid (mempty, All)
 import Prelude hiding (lookup)
 import XMonad
+import XMonad.Prelude (All, fi, when)
 import XMonad.StackSet (current,screen)
 import XMonad.Util.Font (Align(AlignCenter)
                        , initXMF
@@ -37,7 +36,6 @@ import XMonad.Util.Font (Align(AlignCenter)
 import XMonad.Util.Timer (startTimer)
 import XMonad.Util.XUtils (createNewWindow
                          , deleteWindow
-                         , fi
                          , showWindow
                          , paintAndWrite)
 import qualified XMonad.Util.ExtensibleState as ES
@@ -58,7 +56,7 @@ import qualified XMonad.Util.ExtensibleState as ES
 
 -- | ShowText contains the map with timers as keys and created windows as values
 newtype ShowText = ShowText (Map Atom Window)
-    deriving (Read,Show,Typeable)
+    deriving (Read,Show)
 
 instance ExtensionClass ShowText where
     initialValue = ShowText empty
@@ -75,22 +73,22 @@ data ShowTextConfig =
 
 instance Default ShowTextConfig where
   def =
+#ifdef XFT
+    STC { st_font = "xft:monospace-20"
+#else
     STC { st_font = "-misc-fixed-*-*-*-*-20-*-*-*-*-*-*-*"
+#endif
         , st_bg   = "black"
         , st_fg   = "white"
     }
 
-{-# DEPRECATED defaultSTConfig "Use def (from Data.Default, and re-exported by XMonad.Actions.ShowText) instead." #-}
-defaultSTConfig :: ShowTextConfig
-defaultSTConfig = def
-
 -- | Handles timer events that notify when a window should be removed
 handleTimerEvent :: Event -> X All
 handleTimerEvent (ClientMessageEvent _ _ _ dis _ mtyp d) = do
     (ShowText m) <- ES.get :: X ShowText
     a <- io $ internAtom dis "XMONAD_TIMER" False
-    when (mtyp == a && length d >= 1)
-         (whenJust (lookup (fromIntegral $ d !! 0) m) deleteWindow)
+    when (mtyp == a && not (null d))
+         (whenJust (lookup (fromIntegral $ head d) m) deleteWindow)
     mempty
 handleTimerEvent _ = mempty
 
