@@ -1,9 +1,12 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE CPP #-}
 {-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE MultiWayIf #-}
+{-# LANGUAGE LambdaCase #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.TreeSelect
+-- Description :  Display workspaces or actions in a tree-like format.
 -- Copyright   :  (c) Tom Smeets <tom.tsmeets@gmail.com>
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -39,6 +42,7 @@ module XMonad.Actions.TreeSelect
 
     , TSConfig(..)
     , tsDefaultConfig
+    , def
 
       -- * Navigation
       -- $navigation
@@ -60,16 +64,13 @@ module XMonad.Actions.TreeSelect
     , treeselectAt
     ) where
 
-import Control.Applicative
 import Control.Monad.Reader
 import Control.Monad.State
-import Data.List (find)
-import Data.Maybe
 import Data.Tree
-import Foreign
+import Foreign (shiftL, shiftR, (.&.))
 import System.IO
-import System.Posix.Process (forkProcess, executeFile)
 import XMonad hiding (liftX)
+import XMonad.Prelude
 import XMonad.StackSet as W
 import XMonad.Util.Font
 import XMonad.Util.NamedWindows
@@ -113,10 +114,10 @@ import Graphics.X11.Xrender
 --
 -- Optionally, if you add 'workspaceHistoryHook' to your 'logHook' you can use the \'o\' and \'i\' keys to select from previously-visited workspaces
 --
--- > xmonad $ defaultConfig { ...
--- >                        , workspaces = toWorkspaces myWorkspaces
--- >                        , logHook = workspaceHistoryHook
--- >                        }
+-- > xmonad $ def { ...
+-- >              , workspaces = toWorkspaces myWorkspaces
+-- >              , logHook = workspaceHistoryHook
+-- >              }
 --
 -- After that you still need to bind buttons to 'treeselectWorkspace' to start selecting a workspaces and moving windows
 --
@@ -131,22 +132,22 @@ import Graphics.X11.Xrender
 -- $config
 -- The selection menu is very configurable, you can change the font, all colors and the sizes of the boxes.
 --
--- The default config defined as 'tsDefaultConfig'
+-- The default config defined as 'def'
 --
--- > tsDefaultConfig = TSConfig { ts_hidechildren = True
--- >                            , ts_background   = 0xc0c0c0c0
--- >                            , ts_font         = "xft:Sans-16"
--- >                            , ts_node         = (0xff000000, 0xff50d0db)
--- >                            , ts_nodealt      = (0xff000000, 0xff10b8d6)
--- >                            , ts_highlight    = (0xffffffff, 0xffff0000)
--- >                            , ts_extra        = 0xff000000
--- >                            , ts_node_width   = 200
--- >                            , ts_node_height  = 30
--- >                            , ts_originX      = 0
--- >                            , ts_originY      = 0
--- >                            , ts_indent       = 80
--- >                            , ts_navigate     = defaultNavigation
--- >                            }
+-- > def = TSConfig { ts_hidechildren = True
+-- >                , ts_background   = 0xc0c0c0c0
+-- >                , ts_font         = "xft:Sans-16"
+-- >                , ts_node         = (0xff000000, 0xff50d0db)
+-- >                , ts_nodealt      = (0xff000000, 0xff10b8d6)
+-- >                , ts_highlight    = (0xffffffff, 0xffff0000)
+-- >                , ts_extra        = 0xff000000
+-- >                , ts_node_width   = 200
+-- >                , ts_node_height  = 30
+-- >                , ts_originX      = 0
+-- >                , ts_originY      = 0
+-- >                , ts_indent       = 80
+-- >                , ts_navigate     = defaultNavigation
+-- >                }
 
 -- $pixel
 --
@@ -160,8 +161,8 @@ import Graphics.X11.Xrender
 -- white       = 0xffffffff
 -- black       = 0xff000000
 -- red         = 0xffff0000
--- blue        = 0xff00ff00
--- green       = 0xff0000ff
+-- green       = 0xff00ff00
+-- blue        = 0xff0000ff
 -- transparent = 0x00000000
 -- @
 
@@ -258,6 +259,7 @@ defaultNavigation = M.fromList
 -- Using nice alternating blue nodes
 tsDefaultConfig :: TSConfig a
 tsDefaultConfig = def
+{-# DEPRECATED tsDefaultConfig "Use def (from Data.Default, and re-exported by XMonad.Actions.TreeSelect) instead." #-}
 
 -- | Tree Node With a name and extra text
 data TSNode a = TSNode { tsn_name  :: String
@@ -316,7 +318,9 @@ treeselectAt conf@TSConfig{..} zipper hist = withDisplay $ \display -> do
         set_colormap attributes colormap
         set_background_pixel attributes ts_background
         set_border_pixel attributes 0
-        createWindow display rootw rect_x rect_y rect_width rect_height 0 (visualInfo_depth vinfo) inputOutput (visualInfo_visual vinfo) (cWColormap .|. cWBorderPixel .|. cWBackPixel) attributes
+        w <- createWindow display rootw rect_x rect_y rect_width rect_height 0 (visualInfo_depth vinfo) inputOutput (visualInfo_visual vinfo) (cWColormap .|. cWBorderPixel .|. cWBackPixel) attributes
+        setClassHint display w (ClassHint "xmonad-tree_select" "xmonad")
+        pure w
 
     liftIO $ do
         -- TODO: move below?
@@ -405,7 +409,7 @@ treeselectWorkspace c xs f = do
                             , "XConfig.workspaces: "
                             ] ++ map tag ws
         hPutStrLn stderr msg
-        _ <- forkProcess $ executeFile "xmessage" True [msg] Nothing
+        xmessage msg
         return ()
   where
     mkNode n w = do
@@ -448,8 +452,8 @@ splitPath i = case break (== '.') i of
 -- >        ]
 -- >    ]
 treeselectAction :: TSConfig (X a) -> Forest (TSNode (X a)) -> X ()
-treeselectAction c xs = treeselect c xs >>= \x -> case x of
-    Just a  -> a >> return ()
+treeselectAction c xs = treeselect c xs >>= \case
+    Just a  -> void a
     Nothing -> return ()
 
 forMForest :: (Functor m, Applicative m, Monad m) => [Tree a] -> (a -> m b) -> m [Tree b]
@@ -461,7 +465,7 @@ mapMTree f (Node x xs) = Node <$> f x <*>  mapM (mapMTree f) xs
 
 -- | Quit returning the currently selected node
 select :: TreeSelect a (Maybe a)
-select = Just <$> gets (tsn_value . cursor . tss_tree)
+select = gets (Just . (tsn_value . cursor . tss_tree))
 
 -- | Quit without returning anything
 cancel :: TreeSelect a (Maybe a)
@@ -523,18 +527,21 @@ moveWith f = do
 -- | wait for keys and run navigation
 navigate :: TreeSelect a (Maybe a)
 navigate = gets tss_display >>= \d -> join . liftIO . allocaXEvent $ \e -> do
-    maskEvent d (exposureMask .|. keyPressMask .|. buttonReleaseMask) e
+    maskEvent d (exposureMask .|. keyPressMask .|. buttonReleaseMask .|. buttonPressMask) e
 
     ev <- getEvent e
 
-    if ev_event_type ev == keyPress
-      then do
-        (ks, _) <- lookupString $ asKeyEvent e
-        return $ do
-            mask <- liftX $ cleanMask (ev_state ev)
-            f <- asks ts_navigate
-            fromMaybe navigate $ M.lookup (mask, fromMaybe xK_VoidSymbol ks) f
-      else return navigate
+    if | ev_event_type ev == keyPress -> do
+           (ks, _) <- lookupString $ asKeyEvent e
+           return $ do
+               mask <- liftX $ cleanMask (ev_state ev)
+               f <- asks ts_navigate
+               fromMaybe navigate $ M.lookup (mask, fromMaybe xK_VoidSymbol ks) f
+       | ev_event_type ev == buttonPress -> do
+           -- See XMonad.Prompt Note [Allow ButtonEvents]
+           allowEvents d replayPointer currentTime
+           return navigate
+       | otherwise -> return navigate
 
 -- | Request a full redraw
 redraw :: TreeSelect a ()
@@ -599,7 +606,7 @@ drawNode ix iy TSNode{..} col = do
     colormap <- gets tss_colormap
     visual   <- gets tss_visual
     liftIO $ drawWinBox window display visual colormap gc font col tsn_name ts_extra tsn_extra
-        (ix * ts_indent) (iy * ts_node_height)
+        (ix * ts_indent + ts_originX) (iy * ts_node_height + ts_originY)
         ts_node_width ts_node_height
 
     -- TODO: draw extra text (transparent background? or ts_background)
@@ -650,8 +657,16 @@ drawStringXMF display window visual colormap gc font col x y text = case font of
 --
 -- Note that it uses short to represent its components
 fromARGB :: Pixel -> XRenderColor
-fromARGB x = XRenderColor (fromIntegral $ 0xff00 .&. shiftR x 8)  -- red
-                          (fromIntegral $ 0xff00 .&. x)           -- green
-                          (fromIntegral $ 0xff00 .&. shiftL x 8)  -- blue
-                          (fromIntegral $ 0xff00 .&. shiftR x 16) -- alpha
+fromARGB x =
+#if MIN_VERSION_X11_xft(0, 3, 3)
+    XRenderColor r g b a
+#else
+    -- swapped green/blue as a workaround for the faulty Storable instance in X11-xft < 0.3.3
+    XRenderColor r b g a
+#endif
+  where
+    r = fromIntegral $ 0xff00 .&. shiftR x 8
+    g = fromIntegral $ 0xff00 .&. x
+    b = fromIntegral $ 0xff00 .&. shiftL x 8
+    a = fromIntegral $ 0xff00 .&. shiftR x 16
 #endif
