@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.Warp
+-- Description :  Warp the pointer to a given window or screen.
 -- Copyright   :  (c) daniel@wagner-home.com
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -22,7 +23,7 @@ module XMonad.Actions.Warp (
                            warpToWindow
                           ) where
 
-import Data.List
+import XMonad.Prelude
 import XMonad
 import XMonad.StackSet as W
 
@@ -101,7 +102,7 @@ warpToWindow h v =
 warpToScreen :: ScreenId -> Rational -> Rational -> X ()
 warpToScreen n h v = do
     root <- asks theRoot
-    (StackSet {current = x, visible = xs}) <- gets windowset
+    StackSet{current = x, visible = xs} <- gets windowset
     whenJust (fmap (screenRect . W.screenDetail) . find ((n==) . W.screen) $ x : xs)
         $ \r ->
             warp root (rect_x r + fraction h (rect_width  r))
