@@ -1,5 +1,6 @@
 {- |
 Module      :  XMonad.Actions.WindowGo
+Description :  Operations for raising (traveling to) windows.
 License     :  Public domain
 
 Maintainer  :  <gwern0@gmail.com>
@@ -36,10 +37,8 @@ module XMonad.Actions.WindowGo (
                  module XMonad.ManageHook
                 ) where
 
-import Control.Monad
-import Data.Char (toLower)
 import qualified Data.List as L (nub,sortBy)
-import Data.Monoid
+import XMonad.Prelude
 import XMonad (Query(), X(), ManageHook, WindowSet, withWindowSet, runQuery, liftIO, ask)
 import Graphics.X11 (Window)
 import XMonad.ManageHook
