@@ -1,6 +1,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.WindowMenu
+-- Description :  Display window management actions in the center of the focused window.
 -- Copyright   :  (c) Jan Vornberger 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -29,7 +30,7 @@ import qualified XMonad.StackSet as W
 import XMonad.Actions.GridSelect
 import XMonad.Layout.Maximize
 import XMonad.Actions.Minimize
-import XMonad.Util.XUtils (fi)
+import XMonad.Prelude (fi)
 
 -- $usage
 --
@@ -68,7 +69,7 @@ windowMenu = withFocused $ \w -> do
                     | tag <- tags ]
     runSelectedAction gsConfig actions
 
-getSize :: Window -> X (Rectangle)
+getSize :: Window -> X Rectangle
 getSize w = do
   d  <- asks display
   wa <- io $ getWindowAttributes d w
