@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Actions.WindowNavigation
+-- Description :  Experimental rewrite of "XMonad.Layout.WindowNavigation".
 -- Copyright   :  (c) 2007  David Roundy <droundy@darcs.net>,
 --                          Devin Mullins <me@twifkak.com>
 -- Maintainer  :  Devin Mullins <me@twifkak.com>
@@ -40,17 +41,14 @@ module XMonad.Actions.WindowNavigation (
                                        ) where
 
 import XMonad
+import XMonad.Prelude (catMaybes, fromMaybe, listToMaybe, sortOn)
 import XMonad.Util.Types (Direction2D(..))
 import qualified XMonad.StackSet as W
 
-import Control.Applicative ((<$>))
 import Control.Arrow (second)
 import Data.IORef
-import Data.List (sortBy)
 import Data.Map (Map())
 import qualified Data.Map as M
-import Data.Maybe (catMaybes, fromMaybe, listToMaybe)
-import Data.Ord (comparing)
 import qualified Data.Set as S
 
 -- $usage
@@ -65,6 +63,11 @@ import qualified Data.Set as S
 -- >             $ def { ... }
 -- >     xmonad config
 --
+-- Or, for the brave souls:
+--
+-- > main = xmonad =<< withWindowNavigation (xK_w, xK_a, xK_s, xK_d)
+-- >             $ def { ... }
+--
 -- Here, we pass in the keys for navigation in counter-clockwise order from up.
 -- It creates keybindings for @modMask@ to move to window, and @modMask .|. shiftMask@
 -- to swap windows.
@@ -125,9 +128,12 @@ swap = withTargetWindow swapWithFocused
                                    mapWindows (swapWin currentWin targetWin) winSet
                 Nothing -> winSet
         mapWindows f ss = W.mapWorkspace (mapWindows' f) ss
-        mapWindows' f ws@(W.Workspace { W.stack = s }) = ws { W.stack = mapWindows'' f <$> s }
+        mapWindows' f ws@W.Workspace{ W.stack = s } = ws { W.stack = mapWindows'' f <$> s }
         mapWindows'' f (W.Stack focused up down) = W.Stack (f focused) (map f up) (map f down)
-        swapWin win1 win2 win = if win == win1 then win2 else if win == win2 then win1 else win
+        swapWin win1 win2 win
+          | win == win1 = win2
+          | win == win2 = win1
+          | otherwise = win
 
 withTargetWindow :: (Window -> WindowSet -> WindowSet) -> IORef WNState -> Direction2D -> X ()
 withTargetWindow adj posRef dir = fromCurrentPoint posRef $ \win pos -> do
@@ -137,11 +143,11 @@ withTargetWindow adj posRef dir = fromCurrentPoint posRef $ \win pos -> do
       setPosition posRef pos targetRect
 
 trackMovement :: IORef WNState -> X ()
-trackMovement posRef = fromCurrentPoint posRef $ \win pos -> do
+trackMovement posRef = fromCurrentPoint posRef $ \win pos ->
                            windowRect win >>= flip whenJust (setPosition posRef pos . snd)
 
 fromCurrentPoint :: IORef WNState -> (Window -> Point -> X ()) -> X ()
-fromCurrentPoint posRef f = withFocused $ \win -> do
+fromCurrentPoint posRef f = withFocused $ \win ->
                                 currentPosition posRef >>= f win
 
 -- Gets the current position from the IORef passed in, or if nothing (say, from
@@ -193,7 +199,7 @@ windowRects = fmap catMaybes . mapM windowRect . S.toList =<< gets mapped
 windowRect :: Window -> X (Maybe (Window, Rectangle))
 windowRect win = withDisplay $ \dpy -> do
     (_, x, y, w, h, bw, _) <- io $ getGeometry dpy win
-    return $ Just $ (win, Rectangle x y (w + 2 * bw) (h + 2 * bw))
+    return $ Just (win, Rectangle x y (w + 2 * bw) (h + 2 * bw))
     `catchX` return Nothing
 
 -- Modified from droundy's implementation of WindowNavigation:
@@ -209,7 +215,7 @@ inr L (Point px py) (Rectangle rx ry w h) =             px > rx + fromIntegral w
                                             py >= ry && py < ry + fromIntegral h
 
 sortby :: Direction2D -> [(a,Rectangle)] -> [(a,Rectangle)]
-sortby D = sortBy $ comparing (rect_y . snd)
-sortby R = sortBy $ comparing (rect_x . snd)
+sortby D = sortOn (rect_y . snd)
+sortby R = sortOn (rect_x . snd)
 sortby U = reverse . sortby D
 sortby L = reverse . sortby R
