@@ -1,7 +1,8 @@
-{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Actions.WorkspaceCursors
+-- Description  : Like "XMonad.Actions.Plane" for an arbitrary number of dimensions.
 -- Copyright    : (c) 2009 Adam Vogt <vogt.adam@gmail.com>
 -- License      : BSD
 --
@@ -46,14 +47,10 @@ import qualified XMonad.StackSet as W
 import XMonad.Actions.FocusNth(focusNth')
 import XMonad.Layout.LayoutModifier(ModifiedLayout(..),
                                     LayoutModifier(handleMess, redoLayout))
-import XMonad(Typeable, Message, WorkspaceId, X, XState(windowset),
+import XMonad(Message, WorkspaceId, X, XState(windowset),
               fromMessage, sendMessage, windows, gets)
-import Control.Monad((<=<), guard, liftM, liftM2, when)
-import Control.Applicative((<$>))
-import Data.Foldable(Foldable(foldMap), toList)
-import Data.Maybe(fromJust, listToMaybe)
-import Data.Monoid(Monoid(mappend, mconcat))
-import Data.Traversable(sequenceA)
+import XMonad.Util.Stack (reverseS)
+import XMonad.Prelude (find, fromJust, guard, liftA2, toList, when, (<=<))
 
 -- $usage
 --
@@ -61,13 +58,10 @@ import Data.Traversable(sequenceA)
 --
 -- > import XMonad
 -- > import XMonad.Actions.WorkspaceCursors
--- > import XMonad.Hooks.DynamicLog
 -- > import XMonad.Util.EZConfig
 -- > import qualified XMonad.StackSet as W
 -- >
--- > main = do
--- >     x <- xmobar conf
--- >     xmonad x
+-- > main = xmonad conf
 -- >
 -- > conf = additionalKeysP def
 -- >        { layoutHook = workspaceCursors myCursors $ layoutHook def
@@ -92,7 +86,8 @@ import Data.Traversable(sequenceA)
 --   workspaces. Or change it such that workspaces are created when you try to
 --   view it.
 --
--- * Function for pretty printing for DynamicLog that groups workspaces by
+-- * Function for pretty printing for "XMonad.Hooks.StatusBar.PP" that groups
+--   workspaces by
 -- common prefixes
 --
 -- * Examples of adding workspaces to the cursors, having them appear multiple
@@ -118,7 +113,7 @@ end = Cons . fromJust . W.differentiate . map End
 
 data Cursors a
     = Cons (W.Stack (Cursors a))
-    | End a deriving (Eq,Show,Read,Typeable)
+    | End a deriving (Eq,Show,Read)
 
 instance Foldable Cursors where
     foldMap f (End x) = f x
@@ -144,7 +139,7 @@ getFocus (End x) = x
 
 -- This could be made more efficient, if the fact that the suffixes are grouped
 focusTo ::  (Eq t) => t -> Cursors t -> Maybe (Cursors t)
-focusTo x = listToMaybe . filter ((x==) . getFocus) . changeFocus (const True)
+focusTo x = find ((x==) . getFocus) . changeFocus (const True)
 
 -- | non-wrapping version of 'W.focusUp''
 noWrapUp ::  W.Stack t -> W.Stack t
@@ -153,20 +148,19 @@ noWrapUp x@(W.Stack _ []   _ ) = x
 
 -- | non-wrapping version of 'W.focusDown''
 noWrapDown ::  W.Stack t -> W.Stack t
-noWrapDown = reverseStack . noWrapUp . reverseStack
-    where reverseStack (W.Stack t ls rs) = W.Stack t rs ls
+noWrapDown = reverseS . noWrapUp . reverseS
 
 focusDepth ::  Cursors t -> Int
 focusDepth (Cons x) = 1 + focusDepth (W.focus x)
 focusDepth (End  _) = 0
 
 descend :: Monad m =>(W.Stack (Cursors a) -> m (W.Stack (Cursors a)))-> Int-> Cursors a-> m (Cursors a)
-descend f 1 (Cons x) = Cons `liftM` f x
-descend f n (Cons x) | n > 1 = liftM Cons $ descend f (pred n) `onFocus` x
+descend f 1 (Cons x) = Cons <$> f x
+descend f n (Cons x) | n > 1 = fmap Cons $ descend f (pred n) `onFocus` x
 descend _ _ x = return x
 
 onFocus :: (Monad m) => (a1 -> m a1) -> W.Stack a1 -> m (W.Stack a1)
-onFocus f st = (\x -> st { W.focus = x}) `liftM` f (W.focus st)
+onFocus f st = (\x -> st { W.focus = x}) <$> f (W.focus st)
 
 -- | @modifyLayer@ is used to change the focus at a given depth
 modifyLayer :: (W.Stack (Cursors String) -> W.Stack (Cursors String)) -> Int -> X ()
@@ -192,10 +186,10 @@ modifyLayer' :: (W.Stack (Cursors String) -> X (W.Stack (Cursors String))) -> In
 modifyLayer' f depth = modifyCursors (descend f depth)
 
 modifyCursors ::  (Cursors String -> X (Cursors String)) -> X ()
-modifyCursors = sendMessage . ChangeCursors . (liftM2 (>>) updateXMD return <=<)
+modifyCursors = sendMessage . ChangeCursors . (liftA2 (>>) updateXMD return <=<)
 
-data WorkspaceCursors a = WorkspaceCursors (Cursors String)
-    deriving (Typeable,Read,Show)
+newtype WorkspaceCursors a = WorkspaceCursors (Cursors String)
+    deriving (Read,Show)
 
 -- | The state is stored in the 'WorkspaceCursors' layout modifier. Put this as
 -- your outermost modifier, unless you want different cursors at different
@@ -203,8 +197,7 @@ data WorkspaceCursors a = WorkspaceCursors (Cursors String)
 workspaceCursors :: Cursors String -> l a -> ModifiedLayout WorkspaceCursors l a
 workspaceCursors = ModifiedLayout . WorkspaceCursors
 
-data ChangeCursors = ChangeCursors { unWrap :: Cursors String -> X (Cursors String) }
-    deriving (Typeable)
+newtype ChangeCursors = ChangeCursors { unWrap :: Cursors String -> X (Cursors String) }
 
 instance Message ChangeCursors
 
