@@ -3,6 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Config.Azerty
+-- Description  : Fix some keybindings for users of French keyboard layouts.
 -- Copyright    : (c) Devin Mullins <me@twifkak.com>
 -- License      : BSD
 --
@@ -46,7 +47,7 @@ azertyKeys = azertyKeysTop [0x26,0xe9,0x22,0x27,0x28,0x2d,0xe8,0x5f,0xe7,0xe0]
 
 belgianKeys = azertyKeysTop [0x26,0xe9,0x22,0x27,0x28,0xa7,0xe8,0x21,0xe7,0xe0]
 
-azertyKeysTop topRow conf@(XConfig {modMask = modm}) = M.fromList $
+azertyKeysTop topRow conf@XConfig{modMask = modm} = M.fromList $
     [((modm, xK_semicolon), sendMessage (IncMasterN (-1)))]
     ++
     [((m .|. modm, k), windows $ f i)
