@@ -3,6 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Config.Desktop
+-- Description  : Core settings for interfacing with desktop environments.
 -- Copyright    : (c) Spencer Janssen <spencerjanssen@gmail.com>
 -- License      : BSD
 --
@@ -57,6 +58,7 @@ module XMonad.Config.Desktop (
 import XMonad
 import XMonad.Hooks.ManageDocks
 import XMonad.Hooks.EwmhDesktops
+import XMonad.Layout.LayoutModifier (ModifiedLayout)
 import XMonad.Util.Cursor
 
 import qualified Data.Map as M
@@ -164,13 +166,16 @@ import qualified Data.Map as M
 -- >        adjustEventInput
 --
 
+desktopConfig :: XConfig (ModifiedLayout AvoidStruts
+                             (Choose Tall (Choose (Mirror Tall) Full)))
 desktopConfig = docks $ ewmh def
     { startupHook     = setDefaultCursor xC_left_ptr <+> startupHook def
     , layoutHook      = desktopLayoutModifiers $ layoutHook def
     , keys            = desktopKeys <+> keys def }
 
-desktopKeys (XConfig {modMask = modm}) = M.fromList $
+desktopKeys :: XConfig l -> M.Map (KeyMask, KeySym) (X ())
+desktopKeys XConfig{modMask = modm} = M.fromList
     [ ((modm, xK_b), sendMessage ToggleStruts) ]
 
-desktopLayoutModifiers layout = avoidStruts layout
-
+desktopLayoutModifiers :: LayoutClass l a => l a -> ModifiedLayout AvoidStruts l a
+desktopLayoutModifiers = avoidStruts
