@@ -1,14 +1,16 @@
 -- boilerplate {{{
 {-# LANGUAGE ExistentialQuantification, NoMonomorphismRestriction, TypeSynonymInstances #-}
 {-# OPTIONS_GHC -fno-warn-missing-signatures -fno-warn-type-defaults #-}
+-----------------------------------------------------------------------------
+-- |
+-- Module      :  XMonad.Config.Dmwit
+-- Description :  Daniel Wagner's xmonad configuration.
+--
+------------------------------------------------------------------------
 module XMonad.Config.Dmwit where
 
 -- system imports
-import Control.Applicative
-import Control.Monad
 import Control.Monad.Trans
-import Data.Char
-import Data.List
 import Data.Map (Map, fromList)
 import Data.Ratio
 import Data.Word
@@ -29,9 +31,10 @@ import XMonad.Hooks.DynamicLog
 import XMonad.Hooks.ManageDocks
 import XMonad.Hooks.ManageHelpers
 import XMonad.Layout.Grid
-import XMonad.Layout.IndependentScreens
+import XMonad.Layout.IndependentScreens hiding (withScreen)
 import XMonad.Layout.Magnifier
 import XMonad.Layout.NoBorders
+import XMonad.Prelude
 import XMonad.Util.Dzen hiding (x, y)
 import XMonad.Util.SpawnOnce
 -- }}}
@@ -112,7 +115,6 @@ fullscreenMPlayer = className =? "MPlayer" --> do
         Just (16 :% 9)  -> viewFullOn 1 "5" win
         _               -> doFloat
     where
-    fi               = fromIntegral :: Dimension -> Double
     approx (n, d)    = approxRational (fi n / fi d) (1/100)
 
 operationOn f s n w = do
@@ -236,7 +238,7 @@ keyBindings conf = let m = modMask conf in fromList . anyMask $ [
     ((m .|. shiftMask  , xK_p          ), spawnHere termLauncher),
     ((m .|. shiftMask  , xK_c          ), kill),
     ((m                , xK_q          ), restart "xmonad" True),
-    ((m .|. shiftMask  , xK_q          ), io (exitWith ExitSuccess)),
+    ((m .|. shiftMask  , xK_q          ), io exitSuccess),
     ((m                , xK_grave      ), sendMessage NextLayout),
     ((m .|. shiftMask  , xK_grave      ), setLayout $ layoutHook conf),
     ((m                , xK_o          ), sendMessage Toggle),
