@@ -3,6 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Config.Gnome
+-- Description  : Config for integrating xmonad with GNOME.
 -- Copyright    : (c) Spencer Janssen <spencerjanssen@gmail.com>
 -- License      : BSD
 --
@@ -45,7 +46,7 @@ gnomeConfig = desktopConfig
     , keys     = gnomeKeys <+> keys desktopConfig
     , startupHook = gnomeRegister >> startupHook desktopConfig }
 
-gnomeKeys (XConfig {modMask = modm}) = M.fromList $
+gnomeKeys XConfig{modMask = modm} = M.fromList
     [ ((modm, xK_p), gnomeRun)
     , ((modm .|. shiftMask, xK_q), spawn "gnome-session-quit --logout") ]
 
@@ -72,7 +73,7 @@ gnomeRun = withDisplay $ \dpy -> do
 -- > gconftool-2 -s /desktop/gnome/session/required_components/windowmanager xmonad --type string
 gnomeRegister :: MonadIO m => m ()
 gnomeRegister = io $ do
-    x <- lookup "DESKTOP_AUTOSTART_ID" `fmap` getEnvironment
+    x <- lookup "DESKTOP_AUTOSTART_ID" <$> getEnvironment
     whenJust x $ \sessionId -> safeSpawn "dbus-send"
             ["--session"
             ,"--print-reply=literal"
