@@ -1,8 +1,9 @@
-{-# LANGUAGE FlexibleContexts, FlexibleInstances, FunctionalDependencies, KindSignatures, MultiParamTypeClasses, UndecidableInstances #-}
+{-# LANGUAGE FlexibleContexts, FlexibleInstances, FunctionalDependencies, KindSignatures, UndecidableInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Config.Prime
+-- Description :  Draft of a brand new config syntax for xmonad.
 -- Copyright   :  Devin Mullins <devin.mullins@gmail.com>
 -- License     :  BSD-style (see LICENSE)
 --
@@ -115,7 +116,7 @@ ifThenElse,
 import Prelude hiding ((>>), mod)
 import qualified Prelude as P ((>>=), (>>))
 
-import Data.Monoid (All)
+import XMonad.Prelude (All)
 
 import XMonad hiding (xmonad, XConfig(..))
 import XMonad (XConfig(XConfig))
@@ -478,7 +479,7 @@ wsActions = Summable wsActions_ (\x c -> c { wsActions_ = x }) (++)
 -- >     wsSetName 1 "mail"
 -- >     wsSetName 2 "web"
 wsSetName :: Int -> String -> Arr WorkspaceConfig WorkspaceConfig
-wsSetName index newName = wsNames =. (map maybeSet . zip [0..])
+wsSetName index newName = wsNames =. zipWith (curry maybeSet) [0..]
   where maybeSet (i, oldName) | i == (index - 1) = newName
                               | otherwise = oldName
 
@@ -497,8 +498,8 @@ withScreens :: Arr ScreenConfig ScreenConfig -> Prime l l
 withScreens sarr xconf = (P.>>=) (sarr def) $ \sconf -> sprime sconf xconf
   where sprime :: ScreenConfig -> Prime l l
         sprime sconf =
-          (keys =+ [(mod ++ key, action sid) | (sid, key) <- zip [0..] (sKeys_ sconf),
-                                               (mod, action) <- sActions_ sconf])
+          keys =+ [(mod ++ key, action sid) | (sid, key) <- zip [0..] (sKeys_ sconf),
+                                              (mod, action) <- sActions_ sconf]
 
 data ScreenConfig = ScreenConfig {
   sKeys_ :: [String],
