@@ -3,6 +3,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Config.Xfce
+-- Description  : Config for integrating xmonad with Xfce.
 -- Copyright    : (c) Ivan Miljenovic <Ivan.Miljenovic@gmail.com>
 -- License      : BSD
 --
@@ -36,10 +37,10 @@ import qualified Data.Map as M
 -- For examples of how to further customize @xfceConfig@ see "XMonad.Config.Desktop".
 
 xfceConfig = desktopConfig
-    { terminal = "Terminal"
+    { terminal = "xfce4-terminal"
     , keys     = xfceKeys <+> keys desktopConfig }
 
-xfceKeys (XConfig {modMask = modm}) = M.fromList $
+xfceKeys XConfig{modMask = modm} = M.fromList
     [ ((modm,               xK_p), spawn "xfrun4")
     , ((modm .|. shiftMask, xK_p), spawn "xfce4-appfinder")
     , ((modm .|. shiftMask, xK_q), spawn "xfce4-session-logout")
