@@ -2,6 +2,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.DebugEvents
+-- Description :  Dump diagnostic information about X11 events received by xmonad.
 -- Copyright   :  (c) Brandon S Allbery KF8NH, 2012
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -24,31 +25,24 @@ import           Prelude
 import           XMonad                               hiding (windowEvent
                                                              ,(-->)
                                                              )
+import           XMonad.Prelude                       hiding (fi, bool)
 
 import           XMonad.Hooks.DebugKeyEvents                 (debugKeyEvents)
 import           XMonad.Util.DebugWindow                     (debugWindow)
 
 -- import           Graphics.X11.Xlib.Extras.GetAtomName        (getAtomName)
 
-import           Control.Exception.Extensible         as E
+import           Control.Exception                    as E
+import           Control.Monad.Fail
 import           Control.Monad.State
 import           Control.Monad.Reader
-import           Data.Char                                   (isDigit)
-import           Data.Maybe                                  (fromJust)
-import           Data.List                                   (genericIndex
-                                                             ,genericLength
-                                                             ,unfoldr
-                                                             )
 import           Codec.Binary.UTF8.String
-import           Data.Maybe                                  (fromMaybe)
-import           Data.Monoid
 import           Foreign
 import           Foreign.C.Types
 import           Numeric                                     (showHex)
 import           System.Exit
 import           System.IO
 import           System.Process
-import           Control.Applicative
 
 -- | Event hook to dump all received events.  You should probably not use this
 --   unconditionally; it will produce massive amounts of output.
@@ -58,17 +52,17 @@ debugEventsHook e =  debugEventsHook' e >> return (All True)
 -- | Dump an X11 event.  Can't be used directly as a 'handleEventHook'.
 debugEventsHook' :: Event -> X ()
 
-debugEventsHook' (ConfigureRequestEvent {ev_window       = w
-                                        ,ev_parent       = p
-                                        ,ev_x            = x
-                                        ,ev_y            = y
-                                        ,ev_width        = wid
-                                        ,ev_height       = ht
-                                        ,ev_border_width = bw
-                                        ,ev_above        = above
-                                        ,ev_detail       = place
-                                        ,ev_value_mask   = msk
-                                        }) = do
+debugEventsHook' ConfigureRequestEvent{ev_window       = w
+                                      ,ev_parent       = p
+                                      ,ev_x            = x
+                                      ,ev_y            = y
+                                      ,ev_width        = wid
+                                      ,ev_height       = ht
+                                      ,ev_border_width = bw
+                                      ,ev_above        = above
+                                      ,ev_detail       = place
+                                      ,ev_value_mask   = msk
+                                      } = do
   windowEvent "ConfigureRequest" w
   windowEvent "  parent"         p
 --  mask <- quickFormat msk $ dumpBits wmCRMask
@@ -91,95 +85,93 @@ debugEventsHook' (ConfigureRequestEvent {ev_window       = w
                            ]
   say "  requested" s
 
-debugEventsHook' (ConfigureEvent        {ev_window = w
-                                        ,ev_above  = above
-                                        }) = do
+debugEventsHook' ConfigureEvent        {ev_window = w
+                                       ,ev_above  = above
+                                       } = do
   windowEvent "Configure" w
   -- most of the content is covered by debugWindow
   when (above /= none) $ debugWindow above >>= say "  above"
 
-debugEventsHook' (MapRequestEvent       {ev_window     = w
-                                        ,ev_parent     = p
-                                        }) =
+debugEventsHook' MapRequestEvent       {ev_window     = w
+                                       ,ev_parent     = p
+                                       } =
   windowEvent "MapRequest" w >>
   windowEvent "  parent"   p
 
-debugEventsHook' e@(KeyEvent {ev_event_type = t})
+debugEventsHook' e@KeyEvent {ev_event_type = t}
     | t == keyPress =
   io (hPutStr stderr "KeyPress ") >>
   debugKeyEvents e >>
   return ()
 
-debugEventsHook' (ButtonEvent           {ev_window = w
-                                        ,ev_state  = s
-                                        ,ev_button = b
-                                        }) = do
+debugEventsHook' ButtonEvent           {ev_window = w
+                                       ,ev_state  = s
+                                       ,ev_button = b
+                                       } = do
   windowEvent "Button" w
   nl <- gets numberlockMask
   let msk | s == 0    = ""
           | otherwise = "modifiers " ++ vmask nl s
   say "  button" $ show b ++ msk
 
-debugEventsHook' (DestroyWindowEvent    {ev_window = w
-                                        }) =
+debugEventsHook' DestroyWindowEvent    {ev_window = w
+                                        } =
   windowEvent "DestroyWindow" w
 
-debugEventsHook' (UnmapEvent            {ev_window = w
-                                        }) =
+debugEventsHook' UnmapEvent            {ev_window = w
+                                       } =
   windowEvent "Unmap" w
 
-debugEventsHook' (MapNotifyEvent        {ev_window = w
-                                        }) =
+debugEventsHook' MapNotifyEvent        {ev_window = w
+                                       } =
   windowEvent "MapNotify" w
 
 {- way too much output; suppressed.
 
-debugEventsHook' (CrossingEvent         {ev_window    = w
-                                        ,ev_subwindow = s
-                                        }) =
+debugEventsHook' (CrossingEvent        {ev_window    = w
+                                       ,ev_subwindow = s
+                                       }) =
   windowEvent "Crossing"    w >>
   windowEvent "  subwindow" s
 -}
-debugEventsHook' (CrossingEvent         {}) =
+debugEventsHook' CrossingEvent         {} =
   return ()
 
-debugEventsHook' (SelectionRequest      {ev_requestor = rw
-                                        ,ev_owner     = ow
-                                        ,ev_selection = a
-                                        }) =
+debugEventsHook' SelectionRequest      {ev_requestor = rw
+                                       ,ev_owner     = ow
+                                       ,ev_selection = a
+                                       } =
   windowEvent "SelectionRequest" rw >>
   windowEvent "  owner"          ow >>
   atomEvent   "  atom"           a
 
-debugEventsHook' (PropertyEvent         {ev_window    = w
-                                        ,ev_atom      = a
-                                        ,ev_propstate = s
-                                        }) = do
+debugEventsHook' PropertyEvent         {ev_window    = w
+                                       ,ev_atom      = a
+                                       ,ev_propstate = s
+                                       } = do
   a' <- atomName a
   -- too many of these, and they're not real useful
-  if a' `elem` ["_NET_WM_USER_TIME"
---               ,"_NET_WM_WINDOW_OPACITY"
-               ] then return () else do
-  windowEvent "Property on" w
-  s' <- case s of
-          1 -> return "deleted"
-          0 -> dumpProperty a a' w (7 + length a')
-          _ -> error "Illegal propState; Xlib corrupted?"
-  say "  atom" $ a' ++ s'
+  if a' == "_NET_WM_USER_TIME" then return () else do
+    windowEvent "Property on" w
+    s' <- case s of
+            1 -> return "deleted"
+            0 -> dumpProperty a a' w (7 + length a')
+            _ -> error "Illegal propState; Xlib corrupted?"
+    say "  atom" $ a' ++ s'
 
-debugEventsHook' (ExposeEvent           {ev_window = w
-                                        }) =
+debugEventsHook' ExposeEvent           {ev_window = w
+                                       } =
   windowEvent "Expose" w
 
-debugEventsHook' (ClientMessageEvent    {ev_window       = w
-                                        ,ev_message_type = a
-                                        -- @@@ they did it again!  no ev_format,
-                                        --     and ev_data is [CInt]
-                                        -- @@@ and get a load of the trainwreck
-                                        --     that is setClientMessageEvent!
---                                        ,ev_format       = b
-                                        ,ev_data         = vs'
-                                        }) = do
+debugEventsHook' ClientMessageEvent    {ev_window       = w
+                                       ,ev_message_type = a
+                                       -- @@@ they did it again!  no ev_format,
+                                       --     and ev_data is [CInt]
+                                       -- @@@ and get a load of the trainwreck
+                                       --     that is setClientMessageEvent!
+--                                     ,ev_format       = b
+                                       ,ev_data         = vs'
+                                       } = do
   windowEvent "ClientMessage on" w
   n <- atomName a
   -- this is a sort of custom property
@@ -190,7 +182,7 @@ debugEventsHook' (ClientMessageEvent    {ev_window       = w
                   ta <- getAtom ta'
                   return (ta,b,l)
   let wl = bytes b
-  vs <- io $ take (l * wl) `fmap` splitCInt vs'
+  vs <- io $ take (l * wl) <$> splitCInt vs'
   s <- dumpProperty' w a n ta b vs 0 (10 + length n)
   say "  message" $ n ++ s
 
@@ -199,7 +191,7 @@ debugEventsHook' _                      = return ()
 -- | Emit information about an atom.
 atomName   :: Atom -> X String
 atomName a =  withDisplay $ \d ->
-  io $ fromMaybe ("(unknown atom " ++ show a ++ ")") `fmap` getAtomName d a
+  io $ fromMaybe ("(unknown atom " ++ show a ++ ")") <$> getAtomName d a
 
 -- | Emit an atom with respect to the current event.
 atomEvent     :: String -> Atom -> X ()
@@ -226,12 +218,6 @@ clientMessages =  [("_NET_ACTIVE_WINDOW",("_NET_ACTIVE_WINDOW",32,1))
                   ,("WM_SAVE_YOURSELF"  ,("STRING"            , 8,0))
                   ]
 
-#if __GLASGOW_HASKELL__ < 707
-finiteBitSize :: Bits a => a -> Int
-finiteBitSize x = bitSize x
-#endif
-
-
 -- | Convert a modifier mask into a useful string
 vmask                 :: KeyMask -> KeyMask -> String
 vmask numLockMask msk =  unwords $
@@ -281,13 +267,14 @@ newtype Decoder a = Decoder (ReaderT Decode (StateT DecodeState X) a)
              ,Applicative
              ,Monad
              ,MonadIO
+             ,MonadFail
              ,MonadState  DecodeState
              ,MonadReader Decode
              )
 #endif
 
 -- | Retrive, parse, and dump a window property.  As all the high-level property
---   interfaces lose information necessary to decode properties correctly, we 
+--   interfaces lose information necessary to decode properties correctly, we
 --   work at the lowest level available.
 dumpProperty          :: Atom -> String -> Window -> Int -> X String
 dumpProperty a n w i  =  do
@@ -313,9 +300,9 @@ dumpProperty a n w i  =  do
               vsp
     case rc of
       0 -> do
-        fmt <- fromIntegral `fmap` peek fmtp
+        fmt <- fromIntegral <$> peek fmtp
         vs' <-                     peek vsp
-        sz  <- fromIntegral `fmap` peek szp
+        sz  <- fromIntegral <$> peek szp
         case () of
           () | fmt == none     -> xFree vs' >> return (Left   "(property deleted)"   )
              | sz < 0          -> xFree vs' >> return (Left $ "(illegal bit size " ++
@@ -325,9 +312,9 @@ dumpProperty a n w i  =  do
                                                               show sz              ++
                                                               ")"                    )
              | otherwise       -> do
-                 len <- fromIntegral `fmap` peek lenp
+                 len <- fromIntegral <$> peek lenp
                  -- that's as in "ack! it's fugged!"
-                 ack <- fromIntegral `fmap` peek ackp
+                 ack <- fromIntegral <$> peek ackp
                  vs <- peekArray (len * bytes sz) vs'
                  _ <- xFree vs'
                  return $ Right (fmt,sz,ack,vs)
@@ -414,8 +401,8 @@ runDecode c s (Decoder p) =  runStateT (runReaderT p c) s
 bytes   :: Int -> Int
 bytes w =  w `div` 8
 
--- | The top level property decoder, for a wide variety of standard ICCCM and 
---   EWMH window properties.  We pass part of the 'ReaderT' as arguments for 
+-- | The top level property decoder, for a wide variety of standard ICCCM and
+--   EWMH window properties.  We pass part of the 'ReaderT' as arguments for
 --   pattern matching.
 dumpProp                                              :: Atom -> String -> Decoder Bool
 
@@ -527,12 +514,12 @@ dumpProp a _ | a == wM_NAME                           =  dumpString
              | a == sECONDARY                         =  dumpSelection
                -- this is gross
              | a == wM_TRANSIENT_FOR                  =  do
-                 root <- fromIntegral `fmap` inX (asks theRoot)
+                 root <- fromIntegral <$> inX (asks theRoot)
                  w <- asks window
-                 WMHints {wmh_window_group = group} <-
+                 WMHints {wmh_window_group = wgroup} <-
                    inX $ asks display >>= io . flip getWMHints w
-                 dumpExcept [(0   ,"window group " ++ show group)
-                            ,(root,"window group " ++ show group)
+                 dumpExcept [(0   ,"window group " ++ show wgroup)
+                            ,(root,"window group " ++ show wgroup)
                             ]
                             dumpWindow
              | a == rESOURCE_MANAGER                  =  dumpString
@@ -610,7 +597,7 @@ dumpArray item =  do
 dumpArray'          :: Decoder Bool -> String -> Decoder Bool
 dumpArray' item pfx =  do
   vs <- gets value
-  if vs == []
+  if null vs
     then append "]"
     else append pfx >> whenD item (dumpArray' item ",")
 
@@ -697,31 +684,30 @@ dumpList'' m ((l,p,t):ps) sep = do
 dumpString :: Decoder Bool
 dumpString =  do
   fmt <- asks pType
-  x <- inX $ mapM getAtom ["COMPOUND_TEXT","UTF8_STRING"]
-  case x of
-    [cOMPOUND_TEXT,uTF8_STRING] -> case () of
-      () | fmt == cOMPOUND_TEXT -> guardSize 16 (...)
-         | fmt == sTRING        -> guardSize  8 $ do
-                                     vs <- gets value
-                                     modify (\r -> r {value = []})
-                                     let ss = flip unfoldr (map twiddle vs) $
-                                              \s -> if null s
-                                                    then Nothing
-                                                    else let (w,s'') = break (== '\NUL') s
-                                                             s'      = if null s''
-                                                                       then s''
-                                                                       else tail s''
-                                                          in Just (w,s')
-                                     case ss of
-                                       [s] -> append $ show s
-                                       ss' -> let go (s:ss'') c = append c        >>
-                                                                  append (show s) >>
-                                                                  go ss'' ","
-                                                  go []       _ = append "]"
-                                               in append "[" >> go ss' ""
-         | fmt == uTF8_STRING   -> dumpUTF -- duplicate type test instead of code :)
-         | otherwise            -> (inX $ atomName fmt) >>=
-                                   failure . ("unrecognized string type " ++)
+  [cOMPOUND_TEXT,uTF8_STRING] <- inX $ mapM getAtom ["COMPOUND_TEXT","UTF8_STRING"]
+  case () of
+    () | fmt == cOMPOUND_TEXT -> guardSize 16 (...)
+       | fmt == sTRING        -> guardSize  8 $ do
+                                   vs <- gets value
+                                   modify (\r -> r {value = []})
+                                   let ss = flip unfoldr (map twiddle vs) $
+                                            \s -> if null s
+                                                  then Nothing
+                                                  else let (w,s'') = break (== '\NUL') s
+                                                           s'      = if null s''
+                                                                     then s''
+                                                                     else tail s''
+                                                        in Just (w,s')
+                                   case ss of
+                                     [s] -> append $ show s
+                                     ss' -> let go (s:ss'') c = append c        >>
+                                                                append (show s) >>
+                                                                go ss'' ","
+                                                go []       _ = append "]"
+                                             in append "[" >> go ss' ""
+       | fmt == uTF8_STRING   -> dumpUTF -- duplicate type test instead of code :)
+       | otherwise            -> inX (atomName fmt) >>=
+                                 failure . ("unrecognized string type " ++)
 
 -- show who owns a selection
 dumpSelection :: Decoder Bool
@@ -740,7 +726,7 @@ dumpSelection =  do
 -- for now, not querying Xkb
 dumpXKlInds :: Decoder Bool
 dumpXKlInds =  guardType iNTEGER $ do
-                 n <- fmap fromIntegral `fmap` getInt' 32
+                 n <- fmap fromIntegral <$> getInt' 32
                  case n of
                    Nothing -> propShortErr
                    Just is -> append $ "indicators " ++ unwords (dumpInds is 1 1 [])
@@ -751,7 +737,7 @@ dumpXKlInds =  guardType iNTEGER $ do
                        | n .&. bt /= 0    =  dumpInds (n .&. complement bt)
                                                       (bt `shiftL` 1)
                                                       (c + 1)
-                                                      ((show c):bs)
+                                                      (show c:bs)
                        | otherwise        =  dumpInds n
                                                       (bt `shiftL` 1)
                                                       (c + 1)
@@ -826,10 +812,10 @@ dumpUTF :: Decoder Bool
 dumpUTF =  do
   uTF8_STRING <- inX $ getAtom "UTF8_STRING"
   guardType uTF8_STRING $ guardSize 8 $ do
-  s <- gets value
-  modify (\r -> r {value = []})
-  append . show . decode . map fromIntegral $ s
-  return True
+    s <- gets value
+    modify (\r -> r {value = []})
+    append . show . decode . map fromIntegral $ s
+    return True
 
 -- dump an enumerated value using a translation table
 dumpEnum'        :: [String] -> Atom -> Decoder Bool
@@ -849,7 +835,7 @@ dumpPixmap =  guardType pIXMAP $ do
                   Just p  -> do
                     append $ "pixmap " ++ showHex p ""
                     g' <- inX $ withDisplay $ \d -> io $
-                            Just `fmap` getGeometry d (fromIntegral p)
+                            (Just <$> getGeometry d (fromIntegral p))
                             `E.catch`
                             \e -> case fromException e of
                                     Just x -> throw e `const` (x `asTypeOf` ExitSuccess)
@@ -901,7 +887,7 @@ dumpMwmInfo =  do
   guardType ta $ dumpList' [("flags" ,dumpBits mwmHints,cARDINAL)
                            ,("window",dumpWindow       ,wINDOW  )
                            ]
-             
+
 -- the most common case
 dumpEnum    :: [String] -> Decoder Bool
 dumpEnum ss =  dumpEnum' ss cARDINAL
@@ -945,7 +931,7 @@ dumpPid =  guardType cARDINAL $ do
                       case o of
                         Nothing -> append $ "pid " ++ pid
                         Just p' -> do
-                                  prc <- io $ lines `fmap` hGetContents p'
+                                  prc <- io $ lines <$> hGetContents p'
                                   -- deliberately forcing it
                                   append $ if length prc < 2
                                            then "pid " ++ pid
@@ -1001,13 +987,13 @@ dumpMDPrereg =  do
         append "total size = "
         withIndent 13 dump32
         dumpMDBlocks $ fromIntegral dsc
-    
+
 dumpMDBlocks   :: Int -> Decoder Bool
 dumpMDBlocks _ =  propSimple "(drop site info)" -- @@@ maybe later if needed
 
 dumpMotifEndian :: Decoder Bool
 dumpMotifEndian =  guardType cARDINAL $ guardSize 8 $ do
-  c <- map twiddle `fmap` eat 1
+  c <- map twiddle <$> eat 1
   case c of
     ['l'] -> append "little"
     ['B'] -> append "big"
@@ -1025,7 +1011,7 @@ dumpPercent =  guardType cARDINAL $ do
                  n <- getInt' 32
                  case n of
                    Nothing -> return False
-                   Just n' -> 
+                   Just n' ->
                        let pct = 100 * fromIntegral n' / fromIntegral (maxBound :: Word32)
                            pct :: Double
                         in append $ show (round pct :: Integer) ++ "%"
@@ -1166,7 +1152,7 @@ getInt' 64 =  guardR width 32 (\a e -> propSizeErr a e >> return Nothing) $
                 return $ Just $ lo + hi * (fromIntegral (maxBound :: Word32) + 1)
 getInt' w  =  guardR width w  (\a e -> propSizeErr a e >> return Nothing) $
               guardSize' (bytes w) (propShortErr >> return Nothing)       $
-              Just `fmap` inhale w
+              Just <$> inhale w
 
 -- parse an integral value and feed it to a show-er of some kind
 getInt     :: Int -> (Integer -> String) -> Decoder Bool
@@ -1178,28 +1164,25 @@ getInt w f =  getInt' w >>= maybe (return False) (append . f)
 -- @@@@@@@@@ evil beyond evil.  there *has* to be a better way
 inhale    :: Int -> Decoder Integer
 inhale  8 =  do
-               x <- eat 1
-               case x of
-                 [b] -> return $ fromIntegral b
+               [b] <- eat 1
+               return $ fromIntegral b
 inhale 16 =  do
-               x <- eat 2
-               case x of
-                 [b0,b1] -> io $ allocaArray 2 $ \p -> do
-                              pokeArray p [b0,b1]
-                              [v] <- peekArray 1 (castPtr p :: Ptr Word16)
-                              return $ fromIntegral v
+               [b0,b1] <- eat 2
+               io $ allocaArray 2 $ \p -> do
+                 pokeArray p [b0,b1]
+                 [v] <- peekArray 1 (castPtr p :: Ptr Word16)
+                 return $ fromIntegral v
 inhale 32 =  do
-               x <- eat 4
-               case x of
-                 [b0,b1,b2,b3] -> io $ allocaArray 4 $ \p -> do
-                                    pokeArray p [b0,b1,b2,b3]
-                                    [v] <- peekArray 1 (castPtr p :: Ptr Word32)
-                                    return $ fromIntegral v
+               [b0,b1,b2,b3] <- eat 4
+               io $ allocaArray 4 $ \p -> do
+                 pokeArray p [b0,b1,b2,b3]
+                 [v] <- peekArray 1 (castPtr p :: Ptr Word32)
+                 return $ fromIntegral v
 inhale  b =  error $ "inhale " ++ show b
 
 eat   :: Int -> Decoder Raw
 eat n =  do
-  (bs,rest) <- splitAt n `fmap` gets value
+  (bs,rest) <- gets (splitAt n . value)
   modify (\r -> r {value = rest})
   return bs
 
