@@ -1,7 +1,8 @@
-{-# LANGUAGE FlexibleInstances, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.FadeWindows
+-- Description :  A more flexible and general compositing interface than FadeInactive.
 -- Copyright   :  Brandon S Allbery KF8NH <allbery.b@gmail.com>
 -- License     :  BSD
 --
@@ -49,6 +50,7 @@ module XMonad.Hooks.FadeWindows (-- * Usage
                                 ) where
 
 import           XMonad.Core
+import           XMonad.Prelude
 import           XMonad.ManageHook                       (liftX)
 import qualified XMonad.StackSet             as W
 
@@ -56,13 +58,10 @@ import           XMonad.Hooks.FadeInactive               (setOpacity
                                                          ,isUnfocused
                                                          )
 
-import           Control.Monad                           (forM_)
 import           Control.Monad.Reader                    (ask
                                                          ,asks)
 import           Control.Monad.State                     (gets)
 import qualified Data.Map                    as M
-import           Data.Monoid                      hiding ((<>))
-import           Data.Semigroup
 
 import           Graphics.X11.Xlib.Extras                (Event(..))
 
@@ -76,17 +75,26 @@ import           Graphics.X11.Xlib.Extras                (Event(..))
 -- >     , handleEventHook = fadeWindowsEventHook
 -- >     {- ... -}
 -- >
--- > myFadeHook = composeAll [isUnfocused --> transparency 0.2
--- >                         ,                opaque
+-- > myFadeHook = composeAll [                 opaque
+-- >                         , isUnfocused --> transparency 0.2
 -- >                         ]
 --
 -- The above is like FadeInactive with a fade value of 0.2.
 --
--- FadeHooks do not accumulate; instead, they compose from right to
--- left like 'ManageHook's, so the above example @myFadeHook@ will
--- render unfocused windows at 4/5 opacity and the focused window
--- as opaque.  The 'opaque' hook above is optional, by the way, as any
--- unmatched window will be opaque by default.
+-- 'FadeHook's do not accumulate; instead, they compose from right to
+-- left like 'ManageHook's, so in the above example @myFadeHook@ will
+-- render unfocused windows at 4/5 opacity and the focused window as
+-- opaque.  This means that, in particular, the order in the above
+-- example is important.
+--
+-- The 'opaque' hook above is optional, by the way, as any unmatched
+-- window will be opaque by default.  If you want to make all windows a
+-- bit transparent by default, you can replace 'opaque' with something
+-- like
+--
+-- > transparency 0.93
+--
+-- at the top of @myFadeHook@.
 --
 -- This module is best used with "XMonad.Hooks.MoreManageHelpers", which
 -- exports a number of Queries that can be used in either @ManageHook@
@@ -164,17 +172,17 @@ opacity :: Rational -- ^ The opacity of a window as a fraction.
 opacity =  doS . Opacity . clampRatio
 
 fadeTo, translucence, fadeBy :: Rational -> FadeHook
--- ^ An alias for 'transparency'.
+-- | An alias for 'transparency'.
 fadeTo       = transparency
--- ^ An alias for 'transparency'.
+-- | An alias for 'transparency'.
 translucence = transparency
--- ^ An alias for 'transparency'.
+-- | An alias for 'opacity'.
 fadeBy       = opacity
 
 invisible, solid :: FadeHook
--- ^ An alias for 'transparent'.
+-- | An alias for 'transparent'.
 invisible    = transparent
--- ^ An alias for 'opaque'.
+-- | An alias for 'opaque'.
 solid        = opaque
 
 -- | Like 'doF', but usable with 'ManageHook'-like hooks that
@@ -205,7 +213,7 @@ fadeWindowsLogHook h =  withWindowSet $ \s -> do
   forM_ visibleWins $ \w -> do
     o <- userCodeDef (Opacity 1) (runQuery h w)
     setOpacity w $ case o of
-                     OEmpty    -> 0.93
+                     OEmpty    -> 1
                      Opacity r -> r
 
 -- | A 'handleEventHook' to handle fading and unfading of newly mapped
@@ -213,7 +221,7 @@ fadeWindowsLogHook h =  withWindowSet $ \s -> do
 --   "XMonad.Layout.Full" or "XMonad.Layout.Tabbed".  This hook may
 --   also be useful with "XMonad.Hooks.FadeInactive".
 fadeWindowsEventHook                     :: Event -> X All
-fadeWindowsEventHook (MapNotifyEvent {}) =
+fadeWindowsEventHook MapNotifyEvent{} =
   -- we need to run the fadeWindowsLogHook.  only one way...
   asks config >>= logHook >> return (All True)
 fadeWindowsEventHook _                   =  return (All True)
