@@ -1,7 +1,8 @@
-{-# LANGUAGE DeriveDataTypeable, PatternGuards, FlexibleInstances, MultiParamTypeClasses, CPP #-}
+{-# LANGUAGE PatternGuards, FlexibleInstances, MultiParamTypeClasses, CPP #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Hooks.ManageDocks
+-- Description :  Automatically manage 'dock' type programs.
 -- Copyright    : (c) Joachim Breitner <mail@joachim-breitner.de>
 -- License      : BSD
 --
@@ -15,8 +16,7 @@
 module XMonad.Hooks.ManageDocks (
     -- * Usage
     -- $usage
-    docks, manageDocks, checkDock, AvoidStruts, avoidStruts, avoidStrutsOn,
-    docksEventHook, docksStartupHook,
+    docks, manageDocks, checkDock, AvoidStruts(..), avoidStruts, avoidStrutsOn,
     ToggleStruts(..),
     SetStruts(..),
     module XMonad.Util.Types,
@@ -27,8 +27,11 @@ module XMonad.Hooks.ManageDocks (
     RectC(..),
 #endif
 
-    -- for XMonad.Actions.FloatSnap
-    calcGap
+    -- * For developers of other modules ("XMonad.Actions.FloatSnap")
+    calcGap,
+
+    -- * Standalone hooks (deprecated)
+    docksEventHook, docksStartupHook,
     ) where
 
 
@@ -38,14 +41,11 @@ import Foreign.C.Types (CLong)
 import XMonad.Layout.LayoutModifier
 import XMonad.Util.Types
 import XMonad.Util.WindowProperties (getProp32s)
-import XMonad.Util.XUtils (fi)
 import qualified XMonad.Util.ExtensibleState as XS
-import Data.Monoid (All(..), mempty)
-import Data.Functor((<$>))
+import XMonad.Prelude (All (..), fi, filterM, foldlM, void, when, (<=<))
 
 import qualified Data.Set as S
 import qualified Data.Map as M
-import Control.Monad (when, forM_, filterM)
 
 -- $usage
 -- To use this module, add the following import to @~\/.xmonad\/xmonad.hs@:
@@ -54,7 +54,7 @@ import Control.Monad (when, forM_, filterM)
 --
 -- Wrap your xmonad config with a call to 'docks', like so:
 --
--- > main = xmonad $ docks def
+-- > main = xmonad $ … . docks . … $ def{…}
 --
 -- Then add 'avoidStruts' or 'avoidStrutsOn' layout modifier to your layout
 -- to prevent windows from overlapping these windows.
@@ -91,34 +91,61 @@ docks c = c { startupHook     = docksStartupHook <+> startupHook c
             , handleEventHook = docksEventHook <+> handleEventHook c
             , manageHook      = manageDocks <+> manageHook c }
 
-newtype StrutCache = StrutCache { fromStrutCache :: M.Map Window [Strut] }
-    deriving (Eq, Typeable)
+type WindowStruts = M.Map Window [Strut]
 
-data UpdateDocks = UpdateDocks deriving Typeable
+data UpdateDocks = UpdateDocks
 instance Message UpdateDocks
 
 refreshDocks :: X ()
 refreshDocks = sendMessage UpdateDocks
 
+-- Nothing means cache hasn't been initialized yet
+newtype StrutCache = StrutCache { fromStrutCache :: Maybe WindowStruts }
+    deriving Eq
+
 instance ExtensionClass StrutCache where
-  initialValue = StrutCache M.empty
+    initialValue = StrutCache Nothing
 
-updateStrutCache :: Window -> [Strut] -> X Bool
-updateStrutCache w strut = do
-  XS.modified $ StrutCache . M.insert w strut . fromStrutCache
+modifiedStrutCache :: (Maybe WindowStruts -> X WindowStruts) -> X Bool
+modifiedStrutCache f = XS.modifiedM $ fmap (StrutCache . Just) . f . fromStrutCache
 
-deleteFromStructCache :: Window -> X Bool
-deleteFromStructCache w = do
-  XS.modified $ StrutCache . M.delete w . fromStrutCache
+getStrutCache :: X WindowStruts
+getStrutCache = do
+    cache <- maybeInitStrutCache =<< XS.gets fromStrutCache
+    cache <$ XS.put (StrutCache (Just cache))
+
+updateStrutCache :: Window -> X Bool
+updateStrutCache w = modifiedStrutCache $ updateStrut w <=< maybeInitStrutCache
+
+deleteFromStrutCache :: Window -> X Bool
+deleteFromStrutCache w = modifiedStrutCache $ fmap (M.delete w) . maybeInitStrutCache
+
+maybeInitStrutCache :: Maybe WindowStruts -> X WindowStruts
+maybeInitStrutCache = maybe (queryDocks >>= foldlM (flip updateStrut) M.empty) pure
+  where
+    queryDocks = withDisplay $ \dpy -> do
+        (_, _, wins) <- io . queryTree dpy =<< asks theRoot
+        filterM (runQuery checkDock) wins
+
+updateStrut :: Window -> WindowStruts -> X WindowStruts
+updateStrut w cache = do
+    when (w `M.notMember` cache) $ requestDockEvents w
+    strut <- getStrut w
+    pure $ M.insert w strut cache
 
 -- | Detects if the given window is of type DOCK and if so, reveals
 --   it, but does not manage it.
 manageDocks :: ManageHook
-manageDocks = checkDock --> (doIgnore <+> setDocksMask)
-    where setDocksMask = do
-            ask >>= \win -> liftX $ withDisplay $ \dpy -> do
-                io $ selectInput dpy win (propertyChangeMask .|. structureNotifyMask)
-            mempty
+manageDocks = checkDock --> (doIgnore <+> doRequestDockEvents)
+  where
+    doRequestDockEvents = ask >>= liftX . requestDockEvents >> mempty
+
+-- | Request events for a dock window.
+-- (Only if not already a client to avoid overriding 'clientMask')
+requestDockEvents :: Window -> X ()
+requestDockEvents w = whenX (not <$> isClient w) $ withDisplay $ \dpy ->
+    withWindowAttributes dpy w $ \attrs -> io $ selectInput dpy w $
+        wa_your_event_mask attrs .|. propertyChangeMask .|. structureNotifyMask
 
 -- | Checks if a window is a DOCK or DESKTOP window
 checkDock :: Query Bool
@@ -127,39 +154,32 @@ checkDock = ask >>= \w -> liftX $ do
     desk <- getAtom "_NET_WM_WINDOW_TYPE_DESKTOP"
     mbr <- getProp32s "_NET_WM_WINDOW_TYPE" w
     case mbr of
-        Just rs -> return $ any (`elem` [dock,desk]) (map fromIntegral rs)
+        Just rs -> return $ any ((`elem` [dock,desk]) . fromIntegral) rs
         _       -> return False
 
 -- | Whenever a new dock appears, refresh the layout immediately to avoid the
 -- new dock.
+{-# DEPRECATED docksEventHook "Use docks instead." #-}
 docksEventHook :: Event -> X All
-docksEventHook (MapNotifyEvent { ev_window = w }) = do
-    whenX (runQuery checkDock w <&&> (not <$> isClient w)) $ do
-        strut <- getStrut w
-        whenX (updateStrutCache w strut) refreshDocks
+docksEventHook MapNotifyEvent{ ev_window = w } = do
+    whenX (runQuery checkDock w <&&> (not <$> isClient w)) $
+        whenX (updateStrutCache w) refreshDocks
     return (All True)
-docksEventHook (PropertyEvent { ev_window = w
-                              , ev_atom = a }) = do
+docksEventHook PropertyEvent{ ev_window = w
+                            , ev_atom = a } = do
     nws <- getAtom "_NET_WM_STRUT"
     nwsp <- getAtom "_NET_WM_STRUT_PARTIAL"
-    when (a == nws || a == nwsp) $ do
-        strut <- getStrut w
-        whenX (updateStrutCache w strut) refreshDocks
+    when (a == nws || a == nwsp) $
+        whenX (updateStrutCache w) refreshDocks
     return (All True)
-docksEventHook (DestroyWindowEvent {ev_window = w}) = do
-    whenX (deleteFromStructCache w) refreshDocks
+docksEventHook DestroyWindowEvent{ ev_window = w } = do
+    whenX (deleteFromStrutCache w) refreshDocks
     return (All True)
 docksEventHook _ = return (All True)
 
+{-# DEPRECATED docksStartupHook "Use docks instead." #-}
 docksStartupHook :: X ()
-docksStartupHook = withDisplay $ \dpy -> do
-    rootw <- asks theRoot
-    (_,_,wins) <- io $ queryTree dpy rootw
-    docks <- filterM (runQuery checkDock) wins
-    forM_ docks $ \win -> do
-        strut <- getStrut win
-        updateStrutCache win strut
-    refreshDocks
+docksStartupHook = void getStrutCache
 
 -- | Gets the STRUT config, if present, in xmonad gap order
 getStrut :: Window -> X [Strut]
@@ -167,7 +187,7 @@ getStrut w = do
     msp <- getProp32s "_NET_WM_STRUT_PARTIAL" w
     case msp of
         Just sp -> return $ parseStrutPartial sp
-        Nothing -> fmap (maybe [] parseStrut) $ getProp32s "_NET_WM_STRUT" w
+        Nothing -> maybe [] parseStrut <$> getProp32s "_NET_WM_STRUT" w
  where
     parseStrut xs@[_, _, _, _] = parseStrutPartial . take 12 $ xs ++ cycle [minBound, maxBound]
     parseStrut _ = []
@@ -182,7 +202,7 @@ getStrut w = do
 calcGap :: S.Set Direction2D -> X (Rectangle -> Rectangle)
 calcGap ss = withDisplay $ \dpy -> do
     rootw <- asks theRoot
-    struts <- (filter careAbout . concat) `fmap` XS.gets (M.elems . fromStrutCache)
+    struts <- filter careAbout . concat . M.elems <$> getStrutCache
 
     -- we grab the window attributes of the root window rather than checking
     -- the width of the screen because xlib caches this info and it tends to
@@ -206,13 +226,13 @@ avoidStrutsOn :: LayoutClass l a =>
               -> ModifiedLayout AvoidStruts l a
 avoidStrutsOn ss = ModifiedLayout $ AvoidStruts (S.fromList ss)
 
-data AvoidStruts a = AvoidStruts (S.Set Direction2D) deriving ( Read, Show )
+newtype AvoidStruts a = AvoidStruts (S.Set Direction2D) deriving ( Read, Show )
 
 -- | Message type which can be sent to an 'AvoidStruts' layout
 --   modifier to alter its behavior.
 data ToggleStruts = ToggleStruts
                   | ToggleStrut Direction2D
-  deriving (Read,Show,Typeable)
+  deriving (Read,Show)
 
 instance Message ToggleStruts
 
@@ -238,7 +258,7 @@ instance Message ToggleStruts
 data SetStruts = SetStruts { addedStruts   :: [Direction2D]
                            , removedStruts :: [Direction2D] -- ^ These are removed from the currently set struts before 'addedStruts' are added.
                            }
-  deriving (Read,Show,Typeable)
+  deriving (Read,Show)
 
 instance Message SetStruts
 
