@@ -1,6 +1,8 @@
+{-# LANGUAGE LambdaCase #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Hooks.ManageHelpers
+-- Description  : Helper functions to be used in manageHook.
 -- Copyright    : (c) Lukas Mai
 -- License      : BSD
 --
@@ -23,12 +25,27 @@
 -- >         ],
 -- >         ...
 -- >     }
+--
+-- Here's how you can define more helpers like the ones from this module:
+--
+-- > -- some function you want to transform into an infix operator
+-- > f :: a -> b -> Bool
+-- >
+-- > -- a new helper
+-- > q ***? x = fmap (\a -> f a x) q
+-- > -- or
+-- > q ***? x = fmap (`f` x) q
+--
+-- Any existing operator can be "lifted" in the same way:
+--
+-- > q ++? x = fmap (++ x) q
 
 module XMonad.Hooks.ManageHelpers (
     Side(..),
     composeOne,
-    (-?>), (/=?), (<==?), (</=?), (-->>), (-?>>),
+    (-?>), (/=?), (^?), (~?), ($?), (<==?), (</=?), (-->>), (-?>>),
     currentWs,
+    windowTag,
     isInProperty,
     isKDETrayWindow,
     isFullscreen,
@@ -39,6 +56,10 @@ module XMonad.Hooks.ManageHelpers (
     MaybeManageHook,
     transience,
     transience',
+    clientLeader,
+    sameBy,
+    shiftToSame,
+    shiftToSame',
     doRectFloat,
     doFullFloat,
     doCenterFloat,
@@ -46,16 +67,18 @@ module XMonad.Hooks.ManageHelpers (
     doFloatAt,
     doFloatDep,
     doHideIgnore,
+    doSink,
+    doLower,
+    doRaise,
+    doFocus,
     Match,
 ) where
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as W
 import XMonad.Util.WindowProperties (getProp32s)
 
-import Data.Maybe
-import Data.Monoid
-
 import System.Posix (ProcessID)
 
 -- | Denotes a side of a screen. @S@ stands for South, @NE@ for Northeast
@@ -73,29 +96,39 @@ data Match a = Match Bool a
 -- | An alternative 'ManageHook' composer. Unlike 'composeAll' it stops as soon as
 -- a candidate returns a 'Just' value, effectively running only the first match
 -- (whereas 'composeAll' continues and executes all matching rules).
-composeOne :: [MaybeManageHook] -> ManageHook
-composeOne = foldr try idHook
+composeOne :: (Monoid a, Monad m) => [m (Maybe a)] -> m a
+composeOne = foldr try (return mempty)
     where
     try q z = do
         x <- q
-        case x of
-            Just h -> return h
-            Nothing -> z
+        maybe z return x
 
 infixr 0 -?>, -->>, -?>>
 
 -- | q \/=? x. if the result of q equals x, return False
-(/=?) :: Eq a => Query a -> a -> Query Bool
+(/=?) :: (Eq a, Functor m) => m a -> a -> m Bool
 q /=? x = fmap (/= x) q
 
+-- | q ^? x. if the result of q 'isPrefixOf' x, return True
+(^?) :: (Eq a, Functor m) => m [a] -> [a] -> m Bool
+q ^? x = fmap (`isPrefixOf` x) q
+
+-- | q ~? x. if the result of q 'isSuffixOf' x, return True
+(~?) :: (Eq a, Functor m) => m [a] -> [a] -> m Bool
+q ~? x = fmap (`isInfixOf` x) q
+
+-- | q $? x. if the result of q 'isSuffixOf' x, return True
+($?) :: (Eq a, Functor m) => m [a] -> [a] -> m Bool
+q $? x = fmap (`isSuffixOf` x) q
+
 -- | q <==? x. if the result of q equals x, return True grouped with q
-(<==?) :: Eq a => Query a -> a -> Query (Match a)
+(<==?) :: (Eq a, Functor m) => m a -> a -> m (Match a)
 q <==? x = fmap (`eq` x) q
     where
     eq q' x' = Match (q' == x') q'
 
 -- | q <\/=? x. if the result of q notequals x, return True grouped with q
-(</=?) :: Eq a => Query a -> a -> Query (Match a)
+(</=?) :: (Eq a, Functor m) => m a -> a -> m (Match a)
 q </=? x = fmap (`neq` x) q
     where
     neq q' x' = Match (q' /= x') q'
@@ -103,19 +136,19 @@ q </=? x = fmap (`neq` x) q
 -- | A helper operator for use in 'composeOne'. It takes a condition and an action;
 -- if the condition fails, it returns 'Nothing' from the 'Query' so 'composeOne' will
 -- go on and try the next rule.
-(-?>) :: Query Bool -> ManageHook -> MaybeManageHook
+(-?>) :: (Functor m, Monad m) => m Bool -> m a -> m (Maybe a)
 p -?> f = do
     x <- p
     if x then fmap Just f else return Nothing
 
 -- | A helper operator for use in 'composeAll'. It takes a condition and a function taking a grouped datum to action.  If 'p' is true, it executes the resulting action.
-(-->>) :: Query (Match a) -> (a -> ManageHook) -> ManageHook
+(-->>) :: (Monoid b, Monad m) => m (Match a) -> (a -> m b) -> m b
 p -->> f = do
     Match b m <- p
-    if b then (f m) else mempty
+    if b then f m else return mempty
 
 -- | A helper operator for use in 'composeOne'.  It takes a condition and a function taking a groupdatum to action.  If 'p' is true, it executes the resulting action.  If it fails, it returns 'Nothing' from the 'Query' so 'composeOne' will go on and try the next rule.
-(-?>>) :: Query (Match a) -> (a -> ManageHook) -> MaybeManageHook
+(-?>>) :: (Functor m, Monad m) => m (Match a) -> (a -> m b) -> m (Maybe b)
 p -?>> f = do
     Match b m <- p
     if b then fmap  Just (f m) else return Nothing
@@ -124,6 +157,10 @@ p -?>> f = do
 currentWs :: Query WorkspaceId
 currentWs = liftX (withWindowSet $ return . W.currentTag)
 
+-- | Return the workspace tag of a window, if already managed
+windowTag :: Query (Maybe WorkspaceId)
+windowTag = ask >>= \w -> liftX $ withWindowSet $ return . W.findTag w
+
 -- | A predicate to check whether a window is a KDE system tray icon.
 isKDETrayWindow :: Query Bool
 isKDETrayWindow = ask >>= \w -> liftX $ do
@@ -150,12 +187,14 @@ isFullscreen = isInProperty "_NET_WM_STATE" "_NET_WM_STATE_FULLSCREEN"
 isDialog :: Query Bool
 isDialog = isInProperty "_NET_WM_WINDOW_TYPE" "_NET_WM_WINDOW_TYPE_DIALOG"
 
+-- | This function returns 'Just' the @_NET_WM_PID@ property for a
+-- particular window if set, 'Nothing' otherwise.
+--
+-- See <https://specifications.freedesktop.org/wm-spec/wm-spec-1.5.html#idm45623487788432>.
 pid :: Query (Maybe ProcessID)
-pid = ask >>= \w -> liftX $ do
-    p <- getProp32s "_NET_WM_PID" w
-    return $ case p of
-        Just [x] -> Just (fromIntegral x)
-        _        -> Nothing
+pid = ask >>= \w -> liftX $ getProp32s "_NET_WM_PID" w <&> \case
+    Just [x] -> Just (fromIntegral x)
+    _        -> Nothing
 
 -- | A predicate to check whether a window is Transient.
 -- It holds the result which might be the window it is transient to
@@ -169,19 +208,50 @@ transientTo = do
 -- | A convenience 'MaybeManageHook' that will check to see if a window
 -- is transient, and then move it to its parent.
 transience :: MaybeManageHook
-transience = transientTo </=? Nothing -?>> move
-    where
-    move mw = maybe idHook (doF . move') mw
-    move' w s = maybe s (`W.shift` s) (W.findTag w s)
+transience = transientTo </=? Nothing -?>> maybe idHook doShiftTo
 
 -- | 'transience' set to a 'ManageHook'
 transience' :: ManageHook
 transience' = maybeToDefinite transience
 
+-- | This function returns 'Just' the @WM_CLIENT_LEADER@ property for a
+-- particular window if set, 'Nothing' otherwise. Note that, generally,
+-- the window ID returned from this property (by firefox, for example)
+-- corresponds to an unmapped or unmanaged dummy window. For this to be
+-- useful in most cases, it should be used together with 'sameBy'.
+--
+-- See <https://tronche.com/gui/x/icccm/sec-5.html>.
+clientLeader :: Query (Maybe Window)
+clientLeader = ask >>= \w -> liftX $ getProp32s "WM_CLIENT_LEADER" w <&> \case
+    Just [x] -> Just (fromIntegral x)
+    _        -> Nothing
+
+-- | For a given window, 'sameBy' returns all windows that have a matching
+-- property (e.g. those obtained from Queries of 'clientLeader' and 'pid').
+sameBy :: Eq prop => Query (Maybe prop) -> Query [Window]
+sameBy prop = prop >>= \case
+    Nothing -> pure []
+    propVal -> ask >>= \w -> liftX . withWindowSet $ \s ->
+        filterM (fmap (propVal ==) . runQuery prop) (W.allWindows s \\ [w])
+
+-- | 'MaybeManageHook' that moves the window to the same workspace as the
+-- first other window that has the same value of a given 'Query'. Useful
+-- Queries for this include 'clientLeader' and 'pid'.
+shiftToSame :: Eq prop => Query (Maybe prop) -> MaybeManageHook
+shiftToSame prop = sameBy prop </=? [] -?>> maybe idHook doShiftTo . listToMaybe
+
+-- | 'shiftToSame' set to a 'ManageHook'
+shiftToSame' :: Eq prop => Query (Maybe prop) -> ManageHook
+shiftToSame' = maybeToDefinite . shiftToSame
+
 -- | converts 'MaybeManageHook's to 'ManageHook's
-maybeToDefinite :: MaybeManageHook -> ManageHook
+maybeToDefinite :: (Monoid a, Functor m) => m (Maybe a) -> m a
 maybeToDefinite = fmap (fromMaybe mempty)
 
+-- | Move the window to the same workspace as another window.
+doShiftTo :: Window -> ManageHook
+doShiftTo target = doF . shiftTo =<< ask
+  where shiftTo w s = maybe s (\t -> W.shiftWin t w s) (W.findTag target s)
 
 -- | Floats the new window in the given rectangle.
 doRectFloat :: W.RationalRect  -- ^ The rectangle to float the window in. 0 to 1; x, y, w, h.
@@ -212,12 +282,14 @@ doSideFloat :: Side -> ManageHook
 doSideFloat side = doFloatDep move
   where
     move (W.RationalRect _ _ w h) = W.RationalRect cx cy w h
-      where cx =      if side `elem` [SC,C ,NC] then (1-w)/2
-                 else if side `elem` [SW,CW,NW] then 0
-                 else {- side `elem` [SE,CE,NE] -}   1-w
-            cy =      if side `elem` [CE,C ,CW] then (1-h)/2
-                 else if side `elem` [NE,NC,NW] then 0
-                 else {- side `elem` [SE,SC,SW] -}   1-h
+      where cx
+              | side `elem` [SC,C ,NC] = (1-w)/2
+              | side `elem` [SW,CW,NW] = 0
+              | otherwise = {- side `elem` [SE,CE,NE] -} 1-w
+            cy
+              | side `elem` [CE,C ,CW] = (1-h)/2
+              | side `elem` [NE,NC,NW] = 0
+              | otherwise = {- side `elem` [SE,SC,SW] -} 1-h
 
 -- | Floats a new window with its original size, but centered.
 doCenterFloat :: ManageHook
@@ -226,3 +298,21 @@ doCenterFloat = doSideFloat C
 -- | Hides window and ignores it.
 doHideIgnore :: ManageHook
 doHideIgnore = ask >>= \w -> liftX (hide w) >> doF (W.delete w)
+
+-- | Sinks a window
+doSink :: ManageHook
+doSink = doF . W.sink =<< ask
+
+-- | Lower an unmanaged window. Useful together with 'doIgnore' to lower
+-- special windows that for some reason don't do it themselves.
+doLower :: ManageHook
+doLower = ask >>= \w -> liftX $ withDisplay $ \dpy -> io (lowerWindow dpy w) >> mempty
+
+-- | Raise an unmanaged window. Useful together with 'doIgnore' to raise
+-- special windows that for some reason don't do it themselves.
+doRaise :: ManageHook
+doRaise = ask >>= \w -> liftX $ withDisplay $ \dpy -> io (raiseWindow dpy w) >> mempty
+
+-- | Focus a window (useful in 'XMonad.Hooks.EwmhDesktops.setActivateHook').
+doFocus :: ManageHook
+doFocus = doF . W.focusWindow =<< ask
