@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.Place
+-- Description :  Automatic placement of floating windows.
 -- Copyright   :  Quentin Moser <moserq@gmail.com>
 -- License     :  BSD-style (see LICENSE)
 --
@@ -34,18 +35,14 @@ module XMonad.Hooks.Place   ( -- * Usage
 
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as S
 
 import XMonad.Layout.WindowArranger
 import XMonad.Actions.FloatKeys
-import XMonad.Util.XUtils
 
 import qualified Data.Map as M
 import Data.Ratio ((%))
-import Data.List (sortBy, minimumBy, partition)
-import Data.Maybe (fromMaybe, catMaybes)
-import Data.Monoid (Endo(..))
-import Control.Monad (guard, join)
 import Control.Monad.Trans (lift)
 
 -- $usage
@@ -166,16 +163,16 @@ placeFocused p = withFocused $ \window -> do
 
                      -- use X.A.FloatKeys if the window is floating, send
                      -- a WindowArranger message otherwise.
-                   case elem window floats of
-                     True -> keysMoveWindowTo (x', y') (0, 0) window
-                     False -> sendMessage $ SetGeometry r'
+                   if window `elem` floats
+                     then keysMoveWindowTo (x', y') (0, 0) window
+                     else sendMessage $ SetGeometry r'
 
 
 -- | Hook to automatically place windows when they are created.
 placeHook :: Placement -> ManageHook
 placeHook p = do window <- ask
                  r <- Query $ lift $ getWindowRectangle window
-                 allRs <- Query $ lift $ getAllRectangles
+                 allRs <- Query $ lift getAllRectangles
                  pointer <- Query $ lift $ getPointer window
 
                  return $ Endo $ \theWS -> fromMaybe theWS $
@@ -190,13 +187,13 @@ placeHook p = do window <- ask
                         -- workspace's screen.
                       let infos = filter ((window `elem`) . stackContents . S.stack . fst)
                                      $ [screenInfo $ S.current theWS]
-                                        ++ (map screenInfo $ S.visible theWS)
+                                        ++ map screenInfo (S.visible theWS)
                                         ++ zip (S.hidden theWS) (repeat currentRect)
 
                       guard(not $ null infos)
 
                       let (workspace, screen) = head infos
-                          rs = catMaybes $ map (flip M.lookup allRs)
+                          rs = mapMaybe (`M.lookup` allRs)
                                $ organizeClients workspace window floats
                           r' = purePlaceWindow p screen rs pointer r
                           newRect = r2rr screen r'
@@ -225,7 +222,7 @@ purePlaceWindow :: Placement -- ^ The placement strategy
                 -> Rectangle -- ^ The window to be placed
                 -> Rectangle
 purePlaceWindow (Bounds (t,r,b,l) p') (Rectangle sx sy sw sh) rs p w
-  = let s' = (Rectangle (sx + fi l) (sy + fi t) (sw - l - r) (sh - t - b))
+  = let s' = Rectangle (sx + fi l) (sy + fi t) (sw - l - r) (sh - t - b)
     in checkBounds s' $ purePlaceWindow p' s' rs p w
 
 purePlaceWindow (Fixed ratios) s _ _ w = placeRatio ratios s w
@@ -279,7 +276,7 @@ stackContents :: Maybe (S.Stack w) -> [w]
 stackContents = maybe [] S.integrate
 
 screenInfo :: S.Screen i l a sid ScreenDetail -> (S.Workspace i l a, Rectangle)
-screenInfo (S.Screen { S.workspace = ws, S.screenDetail = (SD s)}) = (ws, s)
+screenInfo S.Screen{ S.workspace = ws, S.screenDetail = (SD s)} = (ws, s)
 
 getWindowRectangle :: Window -> X Rectangle
 getWindowRectangle window
@@ -329,8 +326,7 @@ getNecessaryData :: Window
 getNecessaryData window ws floats
   = do r <- getWindowRectangle window
 
-       rs <- return (organizeClients ws window floats)
-             >>= mapM getWindowRectangle
+       rs <- mapM getWindowRectangle (organizeClients ws window floats)
 
        pointer <- getPointer window
 
