@@ -1,6 +1,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.PositionStoreHooks
+-- Description :  Hooks for XMonad.Util.PositionStore.
 -- Copyright   :  (c) Jan Vornberger 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -34,16 +35,13 @@ module XMonad.Hooks.PositionStoreHooks (
     ) where
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as W
 import XMonad.Util.PositionStore
 import XMonad.Hooks.ManageDocks
 import XMonad.Layout.Decoration
 
 import System.Random(randomRIO)
-import Control.Applicative((<$>))
-import Control.Monad(when)
-import Data.Maybe
-import Data.Monoid
 import qualified Data.Set as S
 
 -- $usage
@@ -95,12 +93,12 @@ positionStoreInit mDecoTheme w  = withDisplay $ \d -> do
                                         (Rectangle (fi $ wa_x wa) (fi (wa_y wa) - fi decoH)
                                             (fi $ wa_width wa) (decoH + fi (wa_height wa))) sr' )
     where
-        randomIntOffset :: X (Int)
+        randomIntOffset :: X Int
         randomIntOffset = io $ randomRIO (42, 242)
 
 positionStoreEventHook :: Event -> X All
-positionStoreEventHook (DestroyWindowEvent {ev_window = w, ev_event_type = et}) = do
-    when (et == destroyNotify) $ do
-        modifyPosStore (\ps -> posStoreRemove ps w)
+positionStoreEventHook DestroyWindowEvent{ev_window = w, ev_event_type = et} = do
+    when (et == destroyNotify) $
+        modifyPosStore (`posStoreRemove` w)
     return (All True)
 positionStoreEventHook _ = return (All True)
