@@ -1,9 +1,9 @@
-{-# LANGUAGE FlexibleContexts, MultiParamTypeClasses, TypeSynonymInstances, PatternGuards, DeriveDataTypeable,
-  FlexibleInstances #-}
+{-# LANGUAGE FlexibleContexts, MultiParamTypeClasses, FlexibleInstances #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.UrgencyHook
+-- Description :  Configure an action to occur when a window demands your attention.
 -- Copyright   :  Devin Mullins <me@twifkak.com>
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -30,9 +30,6 @@ module XMonad.Hooks.UrgencyHook (
                                  -- ** Useful keybinding
                                  -- $keybinding
 
-                                 -- ** Note
-                                 -- $note
-
                                  -- * Troubleshooting
                                  -- $troubleshooting
 
@@ -61,10 +58,11 @@ module XMonad.Hooks.UrgencyHook (
                                  NoUrgencyHook(..),
                                  BorderUrgencyHook(..),
                                  FocusHook(..),
-                                 filterUrgencyHook,
+                                 filterUrgencyHook, filterUrgencyHook',
                                  minutes, seconds,
+                                 askUrgent, doAskUrgent,
                                  -- * Stuff for developers:
-                                 readUrgents, withUrgents,
+                                 readUrgents, withUrgents, clearUrgents',
                                  StdoutUrgencyHook(..),
                                  SpawnUrgencyHook(..),
                                  UrgencyHook(urgencyHook),
@@ -73,19 +71,17 @@ module XMonad.Hooks.UrgencyHook (
                                  ) where
 
 import XMonad
+import XMonad.Prelude (fi, delete, fromMaybe, listToMaybe, maybeToList, when, (\\))
 import qualified XMonad.StackSet as W
 
+import XMonad.Hooks.ManageHelpers (windowTag)
 import XMonad.Util.Dzen (dzenWithArgs, seconds)
 import qualified XMonad.Util.ExtensibleState as XS
 import XMonad.Util.NamedWindows (getName)
 import XMonad.Util.Timer (TimerId, startTimer, handleTimer)
 import XMonad.Util.WindowProperties (getProp32)
 
-import Control.Applicative ((<$>))
-import Control.Monad (when)
 import Data.Bits (testBit)
-import Data.List (delete, (\\))
-import Data.Maybe (listToMaybe, maybeToList, fromMaybe)
 import qualified Data.Set as S
 import System.IO (hPutStrLn, stderr)
 import Foreign.C.Types (CLong)
@@ -121,21 +117,16 @@ import Foreign.C.Types (CLong)
 -- > main = xmonad $ withUrgencyHook NoUrgencyHook
 -- >               $ def
 --
--- Now, your "XMonad.Hooks.DynamicLog" must be set up to display the urgent
--- windows. If you're using the 'dzen' or 'dzenPP' functions from that module,
--- then you should be good. Otherwise, you want to figure out how to set
--- 'ppUrgent'.
+-- Now, your "XMonad.Hooks.StatusBar.PP" must be set up to display the urgent
+-- windows. If you're using the 'dzen' (from "XMonad.Hooks.DynamicLog") or
+-- 'dzenPP' functions from that module, then you should be good. Otherwise,
+-- you want to figure out how to set 'ppUrgent'.
 
 -- $keybinding
 --
 -- You can set up a keybinding to jump to the window that was recently marked
 -- urgent. See an example at 'focusUrgent'.
 
--- $note
--- Note: UrgencyHook installs itself as a LayoutModifier, so if you modify your
--- urgency hook and restart xmonad, you may need to rejigger your layout by
--- hitting mod-shift-space.
-
 -- $troubleshooting
 --
 -- There are three steps to get right:
@@ -206,7 +197,7 @@ import Foreign.C.Types (CLong)
 -- instead.
 withUrgencyHook :: (LayoutClass l Window, UrgencyHook h) =>
                    h -> XConfig l -> XConfig l
-withUrgencyHook hook conf = withUrgencyHookC hook urgencyConfig conf
+withUrgencyHook hook = withUrgencyHookC hook urgencyConfig
 
 -- | This lets you modify the defaults set in 'urgencyConfig'. An example:
 --
@@ -217,10 +208,11 @@ withUrgencyHookC :: (LayoutClass l Window, UrgencyHook h) =>
                     h -> UrgencyConfig -> XConfig l -> XConfig l
 withUrgencyHookC hook urgConf conf = conf {
         handleEventHook = \e -> handleEvent (WithUrgencyHook hook urgConf) e >> handleEventHook conf e,
-        logHook = cleanupUrgents (suppressWhen urgConf) >> logHook conf
+        logHook = cleanupUrgents (suppressWhen urgConf) >> logHook conf,
+        startupHook = cleanupStaleUrgents >> startupHook conf
     }
 
-data Urgents = Urgents { fromUrgents :: [Window] } deriving (Read,Show,Typeable)
+newtype Urgents = Urgents { fromUrgents :: [Window] } deriving (Read,Show)
 
 onUrgents :: ([Window] -> [Window]) -> Urgents -> Urgents
 onUrgents f = Urgents . f . fromUrgents
@@ -276,7 +268,7 @@ focusUrgent = withUrgents $ flip whenJust (windows . W.focusWindow) . listToMayb
 --
 -- > , ((modm .|. shiftMask, xK_BackSpace), clearUrgents)
 clearUrgents :: X ()
-clearUrgents = adjustUrgents (const []) >> adjustReminders (const [])
+clearUrgents = withUrgents clearUrgents'
 
 -- | X action that returns a list of currently urgent windows. You might use
 -- it, or 'withUrgents', in your custom logHook, to display the workspaces that
@@ -288,6 +280,12 @@ readUrgents = XS.gets fromUrgents
 withUrgents :: ([Window] -> X a) -> X a
 withUrgents f = readUrgents >>= f
 
+-- | Cleanup urgency and reminders for windows that no longer exist.
+cleanupStaleUrgents :: X ()
+cleanupStaleUrgents = withWindowSet $ \ws -> do
+    adjustUrgents (filter (`W.member` ws))
+    adjustReminders (filter ((`W.member` ws) . window))
+
 adjustUrgents :: ([Window] -> [Window]) -> X ()
 adjustUrgents = XS.modify . onUrgents
 
@@ -299,7 +297,7 @@ data Reminder = Reminder { timer     :: TimerId
                          , window    :: Window
                          , interval  :: Interval
                          , remaining :: Maybe Int
-                         } deriving (Show,Read,Eq,Typeable)
+                         } deriving (Show,Read,Eq)
 
 instance ExtensionClass [Reminder] where
     initialValue = []
@@ -321,14 +319,13 @@ data WithUrgencyHook h = WithUrgencyHook h UrgencyConfig
 changeNetWMState :: Display -> Window -> ([CLong] -> [CLong]) -> X ()
 changeNetWMState dpy w f = do
    wmstate <- getAtom "_NET_WM_STATE"
-   wstate  <- fromMaybe [] `fmap` getProp32 wmstate w
-   let ptype = 4 -- atom property type for changeProperty
-   io $ changeProperty32 dpy w wmstate ptype propModeReplace (f wstate)
+   wstate  <- fromMaybe [] <$> getProp32 wmstate w
+   io $ changeProperty32 dpy w wmstate aTOM propModeReplace (f wstate)
    return ()
 
 -- | Add an atom to the _NET_WM_STATE property.
 addNetWMState :: Display -> Window -> Atom -> X ()
-addNetWMState dpy w atom = changeNetWMState dpy w $ ((fromIntegral atom):)
+addNetWMState dpy w atom = changeNetWMState dpy w (fromIntegral atom :)
 
 -- | Remove an atom from the _NET_WM_STATE property.
 removeNetWMState :: Display -> Window -> Atom -> X ()
@@ -338,7 +335,7 @@ removeNetWMState dpy w atom = changeNetWMState dpy w $ delete (fromIntegral atom
 getNetWMState :: Window -> X [CLong]
 getNetWMState w = do
     a_wmstate <- getAtom "_NET_WM_STATE"
-    fromMaybe [] `fmap` getProp32 a_wmstate w
+    fromMaybe [] <$> getProp32 a_wmstate w
 
 
 -- The Non-ICCCM Manifesto:
@@ -357,10 +354,10 @@ handleEvent :: UrgencyHook h => WithUrgencyHook h -> Event -> X ()
 handleEvent wuh event =
     case event of
      -- WM_HINTS urgency flag
-      PropertyEvent { ev_event_type = t, ev_atom = a, ev_window = w } -> do
+      PropertyEvent { ev_event_type = t, ev_atom = a, ev_window = w } ->
           when (t == propertyNotify && a == wM_HINTS) $ withDisplay $ \dpy -> do
               WMHints { wmh_flags = flags } <- io $ getWMHints dpy w
-              if (testBit flags urgencyHintBit) then markUrgent w else markNotUrgent w
+              if testBit flags urgencyHintBit then markUrgent w else markNotUrgent w
       -- Window destroyed
       DestroyWindowEvent {ev_window = w} ->
           markNotUrgent w
@@ -384,7 +381,7 @@ handleEvent wuh event =
           mapM_ handleReminder =<< readReminders
       where handleReminder reminder = handleTimer (timer reminder) event $ reminderHook wuh reminder
             markUrgent w = do
-                adjustUrgents (\ws -> if elem w ws then ws else w : ws)
+                adjustUrgents (\ws -> if w `elem` ws then ws else w : ws)
                 callUrgencyHook wuh w
                 userCodeDef () =<< asks (logHook . config)
             markNotUrgent w = do
@@ -421,12 +418,15 @@ shouldSuppress :: SuppressWhen -> Window -> X Bool
 shouldSuppress sw w = elem w <$> suppressibleWindows sw
 
 cleanupUrgents :: SuppressWhen -> X ()
-cleanupUrgents sw = do
-    sw' <- suppressibleWindows sw
+cleanupUrgents sw = clearUrgents' =<< suppressibleWindows sw
+
+-- | Clear urgency status of selected windows.
+clearUrgents' :: [Window] -> X ()
+clearUrgents' ws = do
     a_da <- getAtom "_NET_WM_STATE_DEMANDS_ATTENTION"
-    dpy <- withDisplay (\dpy -> return dpy)
-    mapM_ (\w -> removeNetWMState dpy w a_da) sw'
-    adjustUrgents (\\ sw') >> adjustReminders (filter $ ((`notElem` sw') . window))
+    dpy <- withDisplay return
+    mapM_ (\w -> removeNetWMState dpy w a_da) ws
+    adjustUrgents (\\ ws) >> adjustReminders (filter ((`notElem` ws) . window))
 
 suppressibleWindows :: SuppressWhen -> X [Window]
 suppressibleWindows Visible  = gets $ S.toList . mapped
@@ -492,7 +492,7 @@ instance UrgencyHook FocusHook where
 
 borderUrgencyHook :: String -> Window -> X ()
 borderUrgencyHook = urgencyHook . BorderUrgencyHook
-data BorderUrgencyHook = BorderUrgencyHook { urgencyBorderColor :: !String }
+newtype BorderUrgencyHook = BorderUrgencyHook { urgencyBorderColor :: String }
                        deriving (Read, Show)
 
 instance UrgencyHook BorderUrgencyHook where
@@ -535,11 +535,36 @@ instance UrgencyHook StdoutUrgencyHook where
 --
 -- Useful for scratchpad workspaces perhaps:
 --
--- > main = xmonad (withUrgencyHook (filterUrgencyHook ["NSP", "SP"]) defaultConfig)
+-- > main = xmonad (withUrgencyHook (filterUrgencyHook ["NSP", "SP"]) def)
 filterUrgencyHook :: [WorkspaceId] -> Window -> X ()
-filterUrgencyHook skips w = do
-    ws <- gets windowset
-    case W.findTag w ws of
-        Just tag -> when (tag `elem` skips)
-                        $ adjustUrgents (delete w)
-        _ -> return ()
+filterUrgencyHook skips = filterUrgencyHook' $ maybe False (`elem` skips) <$> windowTag
+
+-- | 'filterUrgencyHook' that takes a generic 'Query' to select which windows
+-- should never be marked urgent.
+filterUrgencyHook' :: Query Bool -> Window -> X ()
+filterUrgencyHook' q w = whenX (runQuery q w) (clearUrgents' [w])
+
+-- | Mark the given window urgent.
+--
+-- (The implementation is a bit hacky: send a _NET_WM_STATE ClientMessage to
+-- ourselves. This is so that we respect the 'SuppressWhen' of the configured
+-- urgency hooks. If this module if ever migrated to the ExtensibleConf
+-- infrastrcture, we'll then invoke markUrgent directly.)
+askUrgent :: Window -> X ()
+askUrgent w = withDisplay $ \dpy -> do
+    rw <- asks theRoot
+    a_wmstate <- getAtom "_NET_WM_STATE"
+    a_da      <- getAtom "_NET_WM_STATE_DEMANDS_ATTENTION"
+    let state_add = 1
+    let source_pager = 2
+    io $ allocaXEvent $ \e -> do
+        setEventType e clientMessage
+        setClientMessageEvent' e w a_wmstate 32 [state_add, fi a_da, 0, source_pager]
+        sendEvent dpy rw False (substructureRedirectMask .|. substructureNotifyMask) e
+
+-- | Helper for 'ManageHook' that marks the window as urgent (unless
+-- suppressed, see 'SuppressWhen'). Useful in
+-- 'XMonad.Hooks.EwmhDesktops.setEwmhActivateHook' and also in combination
+-- with "XMonad.Hooks.InsertPosition", "XMonad.Hooks.Focus".
+doAskUrgent :: ManageHook
+doAskUrgent = ask >>= \w -> liftX (askUrgent w) >> mempty
