@@ -1,6 +1,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.WorkspaceByPos
+-- Description :  Move new window to non-focused screen based on its requested geometry.
 -- Copyright   :  (c) Jan Vornberger 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -21,12 +22,10 @@ module XMonad.Hooks.WorkspaceByPos (
     ) where
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as W
-import XMonad.Util.XUtils (fi)
 
-import Data.Maybe
-import Control.Applicative((<$>))
-import Control.Monad.Error ((<=<),guard,lift,runErrorT,throwError)
+import Control.Monad.Except (lift, runExceptT, throwError)
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -44,7 +43,7 @@ needsMoving :: Window -> X (Maybe WorkspaceId)
 needsMoving w = withDisplay $ \d -> do
     -- only relocate windows with non-zero position
     wa <- io $ getWindowAttributes d w
-    fmap (const Nothing `either` Just) . runErrorT $ do
+    fmap (const Nothing `either` Just) . runExceptT $ do
         guard $ wa_x wa /= 0 || wa_y wa /= 0
         ws <- gets windowset
         sc <- lift $ fromMaybe (W.current ws)
