@@ -1,8 +1,9 @@
-{-# LANGUAGE TypeSynonymInstances, MultiParamTypeClasses, DeriveDataTypeable #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE PatternGuards, FlexibleContexts, FlexibleInstances #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.BoringWindows
+-- Description :  Mark windows as boring.
 -- Copyright   :  (c) 2008  David Roundy <droundy@darcs.net>
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -18,8 +19,10 @@ module XMonad.Layout.BoringWindows (
                                    -- * Usage
                                    -- $usage
                                    boringWindows, boringAuto,
-                                   markBoring, clearBoring,
-                                   focusUp, focusDown, focusMaster,
+                                   markBoring, markBoringEverywhere,
+                                   clearBoring, focusUp, focusDown,
+                                   focusMaster, swapUp, swapDown,
+                                   siftUp, siftDown,
 
                                    UpdateBoring(UpdateBoring),
                                    BoringMessage(Replace,Merge),
@@ -32,11 +35,10 @@ module XMonad.Layout.BoringWindows (
 
 import XMonad.Layout.LayoutModifier(ModifiedLayout(..),
                                     LayoutModifier(handleMessOrMaybeModifyIt, redoLayout))
-import XMonad(Typeable, LayoutClass, Message, X, fromMessage,
-              sendMessage, windows, withFocused, Window)
-import Control.Applicative((<$>))
-import Data.List((\\), union)
-import Data.Maybe(fromMaybe, listToMaybe, maybeToList)
+import XMonad(LayoutClass, Message, X, fromMessage,
+              broadcastMessage, sendMessage, windows, withFocused, Window)
+import XMonad.Prelude (find, fromMaybe, listToMaybe, maybeToList, union, (\\))
+import XMonad.Util.Stack (reverseS)
 import qualified Data.Map as M
 import qualified XMonad.StackSet as W
 
@@ -65,28 +67,40 @@ import qualified XMonad.StackSet as W
 data BoringMessage = FocusUp | FocusDown | FocusMaster | IsBoring Window | ClearBoring
                      | Replace String [Window]
                      | Merge String [Window]
-                     deriving ( Read, Show, Typeable )
+                     | SwapUp
+                     | SwapDown
+                     | SiftUp
+                     | SiftDown
+                     deriving ( Read, Show )
 
 instance Message BoringMessage
 
 -- | UpdateBoring is sent before attempting to view another boring window, so
 -- that layouts have a chance to mark boring windows.
 data UpdateBoring = UpdateBoring
-    deriving (Typeable)
 instance Message UpdateBoring
 
-markBoring, clearBoring, focusUp, focusDown, focusMaster :: X ()
+markBoring, clearBoring, focusUp, focusDown, focusMaster, swapUp, swapDown, siftUp, siftDown :: X ()
 markBoring = withFocused (sendMessage . IsBoring)
 clearBoring = sendMessage ClearBoring
 focusUp = sendMessage UpdateBoring >> sendMessage FocusUp
 focusDown = sendMessage UpdateBoring >> sendMessage FocusDown
 focusMaster = sendMessage UpdateBoring >> sendMessage FocusMaster
+swapUp = sendMessage UpdateBoring >> sendMessage SwapUp
+swapDown = sendMessage UpdateBoring >> sendMessage SwapDown
+siftUp = sendMessage UpdateBoring >> sendMessage SiftUp
+siftDown = sendMessage UpdateBoring >> sendMessage SiftDown
+
+-- | Mark current focused window boring for all layouts.
+-- This is useful in combination with the 'XMonad.Actions.CopyWindow' module.
+markBoringEverywhere :: X ()
+markBoringEverywhere = withFocused (broadcastMessage . IsBoring)
 
 data BoringWindows a = BoringWindows
     { namedBoring :: M.Map String [a] -- ^ store borings with a specific source
     , chosenBoring :: [a]             -- ^ user-chosen borings
     , hiddenBoring :: Maybe [a]       -- ^ maybe mark hidden windows
-    } deriving (Show,Read,Typeable)
+    } deriving (Show,Read)
 
 boringWindows :: (LayoutClass l a, Eq a) => l a -> ModifiedLayout BoringWindows l a
 boringWindows = ModifiedLayout (BoringWindows M.empty [] Nothing)
@@ -96,13 +110,13 @@ boringAuto :: (LayoutClass l a, Eq a) => l a -> ModifiedLayout BoringWindows l a
 boringAuto = ModifiedLayout (BoringWindows M.empty [] (Just []))
 
 instance LayoutModifier BoringWindows Window where
-    redoLayout (b@BoringWindows { hiddenBoring = bs }) _r mst arrs = do
+    redoLayout b@BoringWindows{ hiddenBoring = bs } _r mst arrs = do
         let bs' = W.integrate' mst \\ map fst arrs
-        return (arrs, Just $ b { hiddenBoring = const bs' <$> bs } )
+        return (arrs, Just $ b { hiddenBoring = bs' <$ bs } )
 
     handleMessOrMaybeModifyIt bst@(BoringWindows nbs cbs lbs) m
         | Just (Replace k ws) <- fromMessage m
-        , maybe True (ws/=) (M.lookup k nbs) =
+        , Just ws /= M.lookup k nbs =
             let nnb = if null ws then M.delete k nbs
                           else M.insert k ws nbs
             in rjl bst { namedBoring = nnb }
@@ -125,10 +139,27 @@ instance LayoutModifier BoringWindows Window where
                                             . skipBoring W.focusUp'   -- no boring window gets the focus
                                             . focusMaster'
                                return Nothing
-        where skipBoring f st = fromMaybe st $ listToMaybe
-                                $ filter ((`notElem` W.focus st:bs) . W.focus)
-                                $ take (length $ W.integrate st)
-                                $ iterate f st
+        | Just SwapUp <- fromMessage m =
+                            do windows $ W.modify' skipBoringSwapUp
+                               return Nothing
+        | Just SwapDown <- fromMessage m =
+                            do windows $ W.modify' (reverseS . skipBoringSwapUp . reverseS)
+                               return Nothing
+        | Just SiftUp <- fromMessage m =
+                            do windows $ W.modify' (siftUpSkipping bs)
+                               return Nothing
+        | Just SiftDown <- fromMessage m =
+                            do windows $ W.modify' (reverseS . siftUpSkipping bs . reverseS)
+                               return Nothing
+        where skipBoring = skipBoring' ((`notElem` bs) . W.focus)
+              skipBoringSwapUp = skipBoring'
+                                   (maybe True (`notElem` bs) . listToMaybe . W.down)
+                                   swapUp'
+              skipBoring' p f st = fromMaybe st
+                                   $ find p
+                                   $ drop 1
+                                   $ take (length $ W.integrate st)
+                                   $ iterate f st
               bs = concat $ cbs:maybeToList lbs ++ M.elems nbs
               rjl = return . Just . Left
     handleMessOrMaybeModifyIt _ _ = return Nothing
@@ -139,6 +170,19 @@ focusMaster' :: W.Stack a -> W.Stack a
 focusMaster' c@(W.Stack _ [] _) = c
 focusMaster' (W.Stack t ls rs) = W.Stack x [] (xs ++ t : rs) where (x:xs) = reverse ls
 
+swapUp' :: W.Stack a -> W.Stack a
+swapUp' (W.Stack t (l:ls) rs) = W.Stack t ls (l:rs)
+swapUp' (W.Stack t []     rs) = W.Stack t (reverse rs) []
+
+siftUpSkipping :: Eq a => [a] -> W.Stack a -> W.Stack a
+siftUpSkipping bs (W.Stack t ls rs)
+  | (skips, l:ls') <- spanLeft  = W.Stack t ls' (reverse skips ++ l : rs)
+  | (skips, r:rs') <- spanRight = W.Stack t (rs' ++ r : ls) (reverse skips)
+  | otherwise                   = W.Stack t ls rs
+  where
+    spanLeft  = span (`elem` bs) ls
+    spanRight = span (`elem` bs) (reverse rs)
+
 {- $simplest
 
 An alternative to 'Full' is "XMonad.Layout.Simplest".  Less windows are
