@@ -2,6 +2,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.ButtonDecoration
+-- Description :  Decoration that includes buttons, executing actions when clicked on.
 -- Copyright   :  (c) Jan Vornberger 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -48,7 +49,7 @@ buttonDeco :: (Eq a, Shrinker s) => s -> Theme
            -> l a -> ModifiedLayout (Decoration ButtonDecoration s) l a
 buttonDeco s c = decoration s c $ NFD True
 
-data ButtonDecoration a = NFD Bool deriving (Show, Read)
+newtype ButtonDecoration a = NFD Bool deriving (Show, Read)
 
 instance Eq a => DecorationStyle ButtonDecoration a where
     describeDeco _ = "ButtonDeco"
