@@ -1,7 +1,12 @@
-{-# LANGUAGE DeriveDataTypeable, FlexibleContexts, FlexibleInstances, MultiParamTypeClasses, PatternGuards, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleContexts      #-}
+{-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE MultiParamTypeClasses #-}
+{-# LANGUAGE PatternGuards         #-}
+{-# LANGUAGE CPP                   #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.Decoration
+-- Description :  A layout modifier and a class for easily creating decorated layouts.
 -- Copyright   :  (c) 2007 Andrea Rossato, 2009 Jan Vornberger
 -- License     :  BSD-style (see xmonad/LICENSE)
 --
@@ -17,7 +22,7 @@ module XMonad.Layout.Decoration
     ( -- * Usage:
       -- $usage
       decoration
-    , Theme (..), defaultTheme, def
+    , Theme (..), def
     , Decoration
     , DecorationMsg (..)
     , DecorationStyle (..)
@@ -30,12 +35,10 @@ module XMonad.Layout.Decoration
     , DecorationState, OrigWin
     ) where
 
-import Control.Monad (when)
-import Data.Maybe
-import Data.List
 import Foreign.C.Types(CInt)
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.StackSet as W
 import XMonad.Hooks.UrgencyHook
 import XMonad.Layout.LayoutModifier
@@ -89,6 +92,7 @@ data Theme =
                                                            --    Inner @[Bool]@ is a row in a icon bitmap.
           } deriving (Show, Read)
 
+-- | The default xmonad 'Theme'.
 instance Default Theme where
   def =
     Theme { activeColor         = "#999999"
@@ -103,21 +107,20 @@ instance Default Theme where
           , activeTextColor     = "#FFFFFF"
           , inactiveTextColor   = "#BFBFBF"
           , urgentTextColor     = "#FF0000"
+#ifdef XFT
+          , fontName            = "xft:monospace"
+#else
           , fontName            = "-misc-fixed-*-*-*-*-10-*-*-*-*-*-*-*"
+#endif
           , decoWidth           = 200
           , decoHeight          = 20
           , windowTitleAddons   = []
           , windowTitleIcons    = []
           }
 
-{-# DEPRECATED defaultTheme "Use def (from Data.Default, and re-exported by XMonad.Layout.Decoration) instead." #-}
--- | The default xmonad 'Theme'.
-defaultTheme :: Theme
-defaultTheme = def
-
 -- | A 'Decoration' layout modifier will handle 'SetTheme', a message
 -- to dynamically change the decoration 'Theme'.
-data DecorationMsg = SetTheme Theme deriving ( Typeable )
+newtype DecorationMsg = SetTheme Theme
 instance Message DecorationMsg
 
 -- | The 'Decoration' state component, where the list of decorated
@@ -241,7 +244,6 @@ instance (DecorationStyle ds Window, Shrinker s) => LayoutModifier (Decoration d
                                 let ndwrs = zip toAdd $ repeat (Nothing,Nothing)
                                 ndecos <- resync (ndwrs ++ del_dwrs d dwrs) wrs
                                 processState (s {decos = ndecos })
-        | otherwise        = return (wrs, Nothing)
 
         where
           ws        = map fst wrs
@@ -282,7 +284,7 @@ instance (DecorationStyle ds Window, Shrinker s) => LayoutModifier (Decoration d
                               updateDecos sh t (font s) ndwrs
                               return (dwrs_to_wrs ndwrs, Just (Decoration (I (Just (s {decos = ndwrs}))) sh t ds))
 
-    handleMess (Decoration (I (Just s@(DS {decos = dwrs}))) sh t ds) m
+    handleMess (Decoration (I (Just s@DS{decos = dwrs})) sh t ds) m
         | Just e <- fromMessage m                = do decorationEventHook ds s e
                                                       handleEvent sh t s e
                                                       return Nothing
@@ -301,9 +303,9 @@ instance (DecorationStyle ds Window, Shrinker s) => LayoutModifier (Decoration d
 handleEvent :: Shrinker s => s -> Theme -> DecorationState -> Event -> X ()
 handleEvent sh t (DS dwrs fs) e
     | PropertyEvent {ev_window = w} <- e
-    , Just i <- w `elemIndex`             (map (fst . fst) dwrs) = updateDeco sh t fs (dwrs !! i)
+    , Just i <- w `elemIndex` map (fst . fst) dwrs      = updateDeco sh t fs (dwrs !! i)
     | ExposeEvent   {ev_window = w} <- e
-    , Just i <- w `elemIndex` (catMaybes $ map (fst . snd) dwrs) = updateDeco sh t fs (dwrs !! i)
+    , Just i <- w `elemIndex` mapMaybe (fst . snd) dwrs = updateDeco sh t fs (dwrs !! i)
 handleEvent _ _ _ _ = return ()
 
 -- | Mouse focus and mouse drag are handled by the same function, this
@@ -319,7 +321,7 @@ handleMouseFocusDrag ds (DS dwrs _) ButtonEvent { ev_window     = ew
             distFromLeft = ex - fi dx
             distFromRight = fi dwh - (ex - fi dx)
         dealtWith <- decorationCatchClicksHook ds mainw (fi distFromLeft) (fi distFromRight)
-        when (not dealtWith) $ do
+        unless dealtWith $
             mouseDrag (\x y -> focus mainw >> decorationWhileDraggingHook ds ex ey (mainw, r) x y)
                         (decorationAfterDraggingHook ds (mainw, r) ew)
 handleMouseFocusDrag _ _ _ = return ()
@@ -374,17 +376,21 @@ createDecos t ds sc s wrs ((w,r):xs) = do
 createDecos _ _ _ _ _ [] = return []
 
 createDecoWindow :: Theme -> Rectangle -> X Window
-createDecoWindow t r = let mask = Just (exposureMask .|. buttonPressMask) in
-                       createNewWindow r mask (inactiveColor t) True
+createDecoWindow t r = do
+  let mask = Just (exposureMask .|. buttonPressMask)
+  w <- createNewWindow r mask (inactiveColor t) True
+  d <- asks display
+  io $ setClassHint d w (ClassHint "xmonad-decoration" "xmonad")
+  pure w
 
 showDecos :: [DecoWin] -> X ()
-showDecos = showWindows . catMaybes . map fst . filter (isJust . snd)
+showDecos = showWindows . mapMaybe fst . filter (isJust . snd)
 
 hideDecos :: [DecoWin] -> X ()
-hideDecos = hideWindows . catMaybes . map fst
+hideDecos = hideWindows . mapMaybe fst
 
 deleteDecos :: [DecoWin] -> X ()
-deleteDecos = deleteWindows . catMaybes . map fst
+deleteDecos = deleteWindows . mapMaybe fst
 
 updateDecos :: Shrinker s => s -> Theme -> XMonadFont -> [(OrigWin,DecoWin)] -> X ()
 updateDecos s t f = mapM_ $ updateDeco s t f
@@ -396,11 +402,11 @@ updateDeco sh t fs ((w,_),(Just dw,Just (Rectangle _ _ wh ht))) = do
   nw  <- getName w
   ur  <- readUrgents
   dpy <- asks display
-  let focusColor win ic ac uc = (maybe ic (\focusw -> case () of
-                                                       _ | focusw == win -> ac
-                                                         | win `elem` ur -> uc
-                                                         | otherwise     -> ic) . W.peek)
-                                `fmap` gets windowset
+  let focusColor win ic ac uc = maybe ic (\focusw -> case () of
+                                                      _ | focusw == win -> ac
+                                                        | win `elem` ur -> uc
+                                                        | otherwise     -> ic) . W.peek
+                                <$> gets windowset
   (bc,borderc,borderw,tc) <-
     focusColor w (inactiveColor t, inactiveBorderColor t, inactiveBorderWidth t, inactiveTextColor t)
                  (activeColor   t, activeBorderColor   t, activeBorderWidth   t, activeTextColor   t)
