@@ -1,8 +1,9 @@
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, TupleSections #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.Dishes
+-- Description :  A layout that stacks extra windows underneath the master windows.
 -- Copyright   :  (c) Jeremy Apthorp
 -- License     :  BSD-style (see LICENSE)
 --
@@ -23,7 +24,7 @@ module XMonad.Layout.Dishes (
 
 import XMonad
 import XMonad.StackSet (integrate)
-import Control.Monad (ap)
+import XMonad.Prelude (ap)
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -42,7 +43,7 @@ import Control.Monad (ap)
 data Dishes a = Dishes Int Rational deriving (Show, Read)
 instance LayoutClass Dishes a where
     doLayout (Dishes nmaster h) r =
-        return . (\x->(x,Nothing)) .
+        return . (, Nothing) .
         ap zip (dishes h r nmaster . length) . integrate
     pureMessage (Dishes nmaster h) m = fmap incmastern (fromMessage m)
         where incmastern (IncMasterN d) = Dishes (max 0 (nmaster+d)) h
@@ -52,5 +53,5 @@ dishes h s nmaster n = if n <= nmaster
                         then splitHorizontally n s
                         else ws
  where
-    (m,rest) = splitVerticallyBy (1 - (fromIntegral $ n - nmaster) * h) s
+    (m,rest) = splitVerticallyBy (1 - fromIntegral (n - nmaster) * h) s
     ws = splitHorizontally nmaster m ++ splitVertically (n - nmaster) rest
