@@ -1,7 +1,8 @@
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.FixedColumn
+-- Description :  Like Tall, but split at a fixed column (or a window's smallest resize amount).
 -- Copyright   :  (c) 2008 Justin Bogner <mail@justinbogner.com>
 -- License     :  BSD3-style (as xmonad)
 --
@@ -22,8 +23,6 @@ module XMonad.Layout.FixedColumn (
         FixedColumn(..)
 ) where
 
-import Control.Monad (msum)
-import Data.Maybe (fromMaybe)
 import Graphics.X11.Xlib (Window, rect_width)
 import Graphics.X11.Xlib.Extras ( getWMNormalHints
                                 , getWindowAttributes
@@ -31,6 +30,7 @@ import Graphics.X11.Xlib.Extras ( getWMNormalHints
                                 , sh_resize_inc
                                 , wa_border_width)
 
+import XMonad.Prelude (fromMaybe, msum, (<&>))
 import XMonad.Core (X, LayoutClass(..), fromMessage, io, withDisplay)
 import XMonad.Layout (Resize(..), IncMasterN(..), tile)
 import XMonad.StackSet as W
@@ -62,7 +62,7 @@ instance LayoutClass FixedColumn Window where
             fws <- mapM (widthCols fallback ncol) ws
             let frac = maximum (take nmaster fws) // rect_width r
                 rs   = tile frac r nmaster (length ws)
-            return $ (zip ws rs, Nothing)
+            return (zip ws rs, Nothing)
         where ws     = W.integrate s
               x // y = fromIntegral x / fromIntegral y
 
@@ -84,8 +84,8 @@ instance LayoutClass FixedColumn Window where
 widthCols :: Int -> Int -> Window -> X Int
 widthCols inc n w = withDisplay $ \d -> io $ do
     sh <- getWMNormalHints d w
-    bw <- fmap (fromIntegral . wa_border_width) $ getWindowAttributes d w
-    let widthHint f = f sh >>= return . fromIntegral . fst
+    bw <- fromIntegral . wa_border_width <$> getWindowAttributes d w
+    let widthHint f = f sh <&> fromIntegral . fst
         oneCol      = fromMaybe inc $ widthHint sh_resize_inc
         base        = fromMaybe 0 $ widthHint sh_base_size
     return $ 2 * bw + base + n * oneCol
