@@ -1,7 +1,8 @@
-{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, MultiParamTypeClasses, PatternGuards, TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, PatternGuards #-}
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.MouseResizableTile
+-- Description :  Like "XMonad.Layout.ResizableTile", but use the mouse to adjust the layout.
 -- Copyright   :  (c) Jan Vornberger 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -36,7 +37,7 @@ module XMonad.Layout.MouseResizableTile (
 import XMonad hiding (tile, splitVertically, splitHorizontallyBy)
 import qualified XMonad.StackSet as W
 import XMonad.Util.XUtils
-import Control.Applicative((<$>))
+import Graphics.X11 as X
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -80,7 +81,6 @@ data MRTMessage = SetMasterFraction Rational
                     | SetRightSlaveFraction Int Rational
                     | ShrinkSlave
                     | ExpandSlave
-                    deriving Typeable
 instance Message MRTMessage
 
 data DraggerInfo = MasterDragger Position Rational
@@ -153,7 +153,7 @@ instance LayoutClass MouseResizableTile Window where
                                                               focusPos = length l,
                                                               numWindows = length wins })
         where
-            mirrorAdjust a b = if (isMirrored st)
+            mirrorAdjust a b = if isMirrored st
                                 then b
                                 else a
 
@@ -190,15 +190,24 @@ draggerGeometry :: DraggerType -> X DraggerGeometry
 draggerGeometry (FixedDragger g d) =
     return (fromIntegral $ g `div` 2, g, fromIntegral $ d `div` 2, d)
 draggerGeometry BordersDragger = do
-    w <- asks (borderWidth . config)
+    wins <- gets windowset
+    w <- case W.peek wins of
+          Just win -> getBorderWidth win
+          _        -> asks (borderWidth . config)
     return (0, 0, fromIntegral w, 2*w)
 
+getBorderWidth :: Window -> X Dimension
+getBorderWidth win = do
+    d <- asks display
+    (_,_,_,_,_,w,_) <- io $ X.getGeometry d win
+    return w
+
 adjustForMirror :: Bool -> DraggerWithRect -> DraggerWithRect
 adjustForMirror False dragger = dragger
 adjustForMirror True (draggerRect, draggerCursor, draggerInfo) =
         (mirrorRect draggerRect, draggerCursor', draggerInfo)
     where
-        draggerCursor' = if (draggerCursor == xC_sb_h_double_arrow)
+        draggerCursor' = if draggerCursor == xC_sb_h_double_arrow
                             then xC_sb_v_double_arrow
                             else xC_sb_h_double_arrow
 
@@ -234,8 +243,8 @@ replaceAtPos d (x:xs) pos x' = x : replaceAtPos d xs (pos -1 ) x'
 
 sanitizeRectangle :: Rectangle -> Rectangle -> Rectangle
 sanitizeRectangle (Rectangle sx sy swh sht) (Rectangle x y wh ht) =
-    (Rectangle (within 0 (sx + fromIntegral swh) x) (within 0 (sy + fromIntegral sht) y)
-                (within 1 swh wh) (within 1 sht ht))
+    Rectangle (within 0 (sx + fromIntegral swh) x) (within 0 (sy + fromIntegral sht) y)
+                (within 1 swh wh) (within 1 sht ht)
 
 within :: (Ord a) => a -> a -> a -> a
 within low high a = max low $ min high a
