@@ -1,8 +1,9 @@
-{-# LANGUAGE DeriveDataTypeable, ExistentialQuantification, Rank2Types, MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, FlexibleContexts, PatternGuards #-}
+{-# LANGUAGE ExistentialQuantification, Rank2Types, FunctionalDependencies, FlexibleInstances, FlexibleContexts, PatternGuards, ScopedTypeVariables #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.MultiToggle
+-- Description :  Dynamically apply and unapply transformers to your window layout.
 -- Copyright   :  (c) Lukas Mai
 -- License     :  BSD-style (see LICENSE)
 --
@@ -25,6 +26,7 @@ module XMonad.Layout.MultiToggle (
     single,
     mkToggle,
     mkToggle1,
+    isToggleActive,
 
     HList,
     HCons,
@@ -32,12 +34,13 @@ module XMonad.Layout.MultiToggle (
 ) where
 
 import XMonad
+import XMonad.Prelude hiding (find)
 
 import XMonad.StackSet (Workspace(..))
 
 import Control.Arrow
+import Data.IORef
 import Data.Typeable
-import Data.Maybe
 
 -- $usage
 -- The basic idea is to have a base layout and a set of layout transformers,
@@ -86,11 +89,11 @@ import Data.Maybe
 -- which is an instance of the 'Transformer' class.  For example, here
 -- is the definition of @MIRROR@:
 --
--- > data MIRROR = MIRROR deriving (Read, Show, Eq, Typeable)
+-- > data MIRROR = MIRROR deriving (Read, Show, Eq)
 -- > instance Transformer MIRROR Window where
 -- >     transform _ x k = k (Mirror x) (\(Mirror x') -> x')
 --
--- Note, you need to put @{-\# LANGUAGE DeriveDataTypeable,
+-- Note, you need to put @{-\# LANGUAGE
 -- TypeSynonymInstances, MultiParamTypeClasses \#-}@ at the
 -- beginning of your file.
 
@@ -113,7 +116,6 @@ transform' t (EL l det) = transform t l (\l' det' -> EL l' (det . det'))
 
 -- | Toggle the specified layout transformer.
 data Toggle a = forall t. (Transformer t a) => Toggle t
-    deriving (Typeable)
 
 instance (Typeable a) => Message (Toggle a)
 
@@ -188,11 +190,11 @@ instance (Transformer a w, HList b w) => HList (HCons a b) w where
 geq :: (Typeable a, Eq a, Typeable b) => a -> b -> Bool
 geq a b = Just a == cast b
 
-instance (Typeable a, Show ts, HList ts a, LayoutClass l a) => LayoutClass (MultiToggle ts l) a where
+instance (Typeable a, Show ts, Typeable ts, HList ts a, LayoutClass l a) => LayoutClass (MultiToggle ts l) a where
     description mt = currLayout mt `unEL` \l -> description l
 
     runLayout (Workspace i mt s) r = case currLayout mt of
-        EL l det -> fmap (fmap . fmap $ (\x -> mt { currLayout = EL x det })) $
+        EL l det -> (fmap . fmap $ (\x -> mt { currLayout = EL x det })) <$>
             runLayout (Workspace i l s) r
 
     handleMessage mt m
@@ -200,14 +202,33 @@ instance (Typeable a, Show ts, HList ts a, LayoutClass l a) => LayoutClass (Mult
         , i@(Just _) <- find (transformers mt) t
             = case currLayout mt of
                 EL l det -> do
-                    l' <- fromMaybe l `fmap` handleMessage l (SomeMessage ReleaseResources)
+                    l' <- fromMaybe l <$> handleMessage l (SomeMessage ReleaseResources)
                     return . Just $
                         mt {
                             currLayout = (if cur then id else transform' t) (EL (det l') id),
                             currIndex = if cur then Nothing else i
                         }
-                    where cur = (i == currIndex mt)
+                    where cur = i == currIndex mt
+        | Just (MultiToggleActiveQueryMessage t ref :: MultiToggleActiveQueryMessage a) <- fromMessage m
+        , i@(Just _) <- find (transformers mt) t
+            = Nothing <$ io (writeIORef ref (Just (i == currIndex mt)))
         | otherwise
             = case currLayout mt of
-                EL l det -> fmap (fmap (\x -> mt { currLayout = EL x det })) $
+                EL l det -> fmap (\x -> mt { currLayout = EL x det }) <$>
                     handleMessage l m
+
+data MultiToggleActiveQueryMessage a = forall t. (Transformer t a) =>
+    MultiToggleActiveQueryMessage t (IORef (Maybe Bool))
+
+instance (Typeable a) => Message (MultiToggleActiveQueryMessage a)
+
+-- | Query the state of a 'Transformer' on a given workspace.
+--
+-- To query the current workspace, use something like this:
+--
+-- > withWindowSet (isToggleActive t . W.workspace . W.current)
+isToggleActive :: Transformer t Window => t -> WindowSpace -> X (Maybe Bool)
+isToggleActive t w = do
+    ref <- io $ newIORef Nothing
+    sendMessageWithNoRefresh (MultiToggleActiveQueryMessage t ref) w
+    io $ readIORef ref
