@@ -1,7 +1,8 @@
-{-# LANGUAGE PatternGuards, ParallelListComp, DeriveDataTypeable, FlexibleInstances, FlexibleContexts, MultiParamTypeClasses, TypeSynonymInstances #-}
+{-# LANGUAGE PatternGuards, ParallelListComp, FlexibleInstances, FlexibleContexts, MultiParamTypeClasses #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.SubLayouts
+-- Description :  A layout combinator that allows layouts to be nested.
 -- Copyright   :  (c) 2009 Adam Vogt
 -- License     :  BSD-style (see xmonad/LICENSE)
 --
@@ -51,19 +52,15 @@ import XMonad.Layout.WindowNavigation(Navigate(Apply))
 import XMonad.Util.Invisible(Invisible(..))
 import XMonad.Util.Types(Direction2D(..))
 import XMonad hiding (def)
-import Control.Applicative((<$>),(<*))
+import XMonad.Prelude
 import Control.Arrow(Arrow(second, (&&&)))
-import Control.Monad(MonadPlus(mplus), foldM, guard, when, join)
-import Data.Function(on)
-import Data.List(nubBy, (\\), find)
-import Data.Maybe(isNothing, fromMaybe, listToMaybe, mapMaybe)
-import Data.Traversable(sequenceA)
 
 import qualified XMonad as X
 import qualified XMonad.Layout.BoringWindows as B
 import qualified XMonad.StackSet as W
 import qualified Data.Map as M
 import Data.Map(Map)
+import qualified Data.Set as S
 
 -- $screenshots
 --
@@ -187,7 +184,7 @@ import Data.Map(Map)
 --  >          $ subLayout [0,1,2] (Simplest ||| Tall 1 0.2 0.5 ||| Circle)
 --  >          $ Tall 1 0.2 0.5 ||| Full
 subLayout :: [Int] -> subl a -> l a -> ModifiedLayout (Sublayout subl) l a
-subLayout nextLayout sl x = ModifiedLayout (Sublayout (I []) (nextLayout,sl) []) x
+subLayout nextLayout sl = ModifiedLayout (Sublayout (I []) (nextLayout,sl) [])
 
 -- | @subTabbed@ is a use of 'subLayout' with 'addTabs' to show decorations.
 subTabbed :: (Eq a, LayoutModifier (Sublayout Simplest) a, LayoutClass l a) =>
@@ -199,7 +196,7 @@ subTabbed  x = addTabs shrinkText X.def $ subLayout [] Simplest x
 -- defaults ones but to be used as a 'submap' for sending messages to the
 -- sublayout.
 defaultSublMap :: XConfig l -> Map (KeyMask, KeySym) (X ())
-defaultSublMap (XConfig { modMask = modm }) = M.fromList
+defaultSublMap XConfig{ modMask = modm } = M.fromList
          [((modm, xK_space), toSubl NextLayout),
           ((modm, xK_j), onGroup W.focusDown'),
           ((modm, xK_k), onGroup W.focusUp'),
@@ -240,6 +237,9 @@ data Sublayout l a = Sublayout
 -- This representation probably simplifies the internals of the modifier.
 type Groups a = Map a (W.Stack a)
 
+-- | Stack of stacks, a simple representation of groups for purposes of focus.
+type GroupStack a = W.Stack (W.Stack a)
+
 -- | GroupMsg take window parameters to determine which group the action should
 -- be applied to
 data GroupMsg a
@@ -256,22 +256,21 @@ data GroupMsg a
     | WithGroup (W.Stack a -> X (W.Stack a)) a
     | SubMessage SomeMessage  a
                 -- ^ the sublayout with the given window will get the message
-    deriving (Typeable)
 
 -- | merge the window that would be focused by the function when applied to the
 -- W.Stack of all windows, with the current group removed. The given window
 -- should be focused by a sublayout. Example usage: @withFocused (sendMessage .
 -- mergeDir W.focusDown')@
 mergeDir :: (W.Stack Window -> W.Stack Window) -> Window -> GroupMsg Window
-mergeDir f w = WithGroup g w
+mergeDir f = WithGroup g
  where g cs = do
         let onlyOthers = W.filter (`notElem` W.integrate cs)
-        flip whenJust (sendMessage . Merge (W.focus cs) . W.focus . f)
-            =<< fmap (onlyOthers =<<) currentStack
+        (`whenJust` sendMessage . Merge (W.focus cs) . W.focus . f)
+            . (onlyOthers =<<)
+          =<< currentStack
         return cs
 
-data Broadcast = Broadcast SomeMessage -- ^ send a message to all sublayouts
-    deriving (Typeable)
+newtype Broadcast = Broadcast SomeMessage -- ^ send a message to all sublayouts
 
 instance Message Broadcast
 instance Typeable a => Message (GroupMsg a)
@@ -288,7 +287,7 @@ pullWindow = mergeNav (\o c -> sendMessage $ Migrate o c)
 pushWindow = mergeNav (\o c -> sendMessage $ Migrate c o)
 
 mergeNav :: (Window -> Window -> X ()) -> Direction2D -> Navigate
-mergeNav f = Apply (\o -> withFocused (f o))
+mergeNav f = Apply (withFocused . f)
 
 -- | Apply a function on the stack belonging to the currently focused group. It
 -- works for rearranging windows and for changing focus.
@@ -300,21 +299,21 @@ toSubl :: (Message a) => a -> X ()
 toSubl m = withFocused (sendMessage . SubMessage (SomeMessage m))
 
 instance (Read (l Window), Show (l Window), LayoutClass l Window) => LayoutModifier (Sublayout l) Window where
-    modifyLayout (Sublayout { subls = osls }) (W.Workspace i la st) r = do
+    modifyLayout Sublayout{ subls = osls } (W.Workspace i la st) r = do
             let gs' = updateGroup st $ toGroups osls
                 st' = W.filter (`elem` M.keys gs') =<< st
             updateWs gs'
-            oldStack <- gets $ W.stack . W.workspace . W.current . windowset
+            oldStack <- currentStack
             setStack st'
             runLayout (W.Workspace i la st') r <* setStack oldStack
             -- FIXME: merge back reordering, deletions?
 
-    redoLayout (Sublayout { delayMess = I ms, def = defl, subls = osls }) _r st arrs = do
+    redoLayout Sublayout{ delayMess = I ms, def = defl, subls = osls } _r st arrs = do
         let gs' = updateGroup st $ toGroups osls
         sls <- fromGroups defl st gs' osls
 
-        let newL :: LayoutClass l Window => Rectangle -> WorkspaceId -> (l Window) -> Bool
-                    -> (Maybe (W.Stack Window)) -> X ([(Window, Rectangle)], l Window)
+        let newL :: LayoutClass l Window => Rectangle -> WorkspaceId -> l Window -> Bool
+                    -> Maybe (W.Stack Window) -> X ([(Window, Rectangle)], l Window)
             newL rect n ol isNew sst = do
                 orgStack <- currentStack
                 let handle l (y,_)
@@ -391,7 +390,7 @@ instance (Read (l Window), Show (l Window), LayoutClass l Window) => LayoutModif
             in fgs $ nxsAdd $ M.insert x zs $ M.delete yf gs
 
 
-        | otherwise = fmap join $ sequenceA $ catchLayoutMess <$> fromMessage m
+        | otherwise = join <$> sequenceA (catchLayoutMess <$> fromMessage m)
      where gs = toGroups sls
            fgs gs' = do
                 st <- currentStack
@@ -415,26 +414,8 @@ currentStack = gets (W.stack . W.workspace . W.current . windowset)
 
 -- | update Group to follow changes in the workspace
 updateGroup :: Ord a => Maybe (W.Stack a) -> Groups a -> Groups a
-updateGroup mst gs =
-        let flatten = concatMap W.integrate . M.elems
-            news = W.integrate' mst \\ flatten gs
-            deads = flatten gs \\ W.integrate' mst
-
-            uniNew = M.union (M.fromList $ map (\n -> (n,single n)) news)
-            single x = W.Stack x [] []
-
-            -- pass through a list to update/remove keys
-            remDead = M.fromList . map (\w -> (W.focus w,w))
-                        . mapMaybe (W.filter (`notElem` deads)) . M.elems
-
-            -- update the current tab group's order and focus
-            followFocus hs = fromMaybe hs $ do
-                f' <- W.focus `fmap` mst
-                xs <- find (elem f' . W.integrate) $ M.elems hs
-                xs' <- W.filter (`elem` W.integrate xs) =<< mst
-                return $ M.insert f' xs' $ M.delete (W.focus xs) hs
-
-        in remDead $ uniNew $ followFocus gs
+updateGroup Nothing _ = mempty
+updateGroup (Just st) gs = fromGroupStack (toGroupStack gs st)
 
 -- | rearrange the windowset to put the groups of tabs next to eachother, so
 -- that the stack of tabs stays put.
@@ -443,20 +424,49 @@ updateWs = windowsMaybe . updateWs'
 
 updateWs' :: Groups Window -> WindowSet -> Maybe WindowSet
 updateWs' gs ws = do
-    f <- W.peek ws
-    let w = W.index ws
-        nes = concatMap W.integrate $ mapMaybe (flip M.lookup gs) w
-        ws' = W.focusWindow f $ foldr W.insertUp (foldr W.delete' ws nes) nes
-    guard $ W.index ws' /= W.index ws
-    return ws'
+    w <- W.stack . W.workspace . W.current $ ws
+    let w' = flattenGroupStack . toGroupStack gs $ w
+    guard $ w /= w'
+    pure $ W.modify' (const w') ws
+
+-- | Flatten a stack of stacks.
+flattenGroupStack :: GroupStack a -> W.Stack a
+flattenGroupStack (W.Stack (W.Stack f lf rf) ls rs) =
+    let l = lf ++ concatMap (reverse . W.integrate) ls
+        r = rf ++ concatMap W.integrate rs
+    in W.Stack f l r
+
+-- | Extract Groups from a stack of stacks.
+fromGroupStack :: (Ord a) => GroupStack a -> Groups a
+fromGroupStack = M.fromList . map (W.focus &&& id) . W.integrate
+
+-- | Arrange a stack of windows into a stack of stacks, according to (possibly
+-- outdated) Groups.
+toGroupStack :: (Ord a) => Groups a -> W.Stack a -> GroupStack a
+toGroupStack gs st@(W.Stack f ls rs) =
+    W.Stack (let Just f' = lu f in f') (mapMaybe lu ls) (mapMaybe lu rs)
+  where
+    wset = S.fromList (W.integrate st)
+    dead = W.filter (`S.member` wset) -- drop dead windows or entire groups
+    refocus s | f `elem` W.integrate s -- sync focus/order of current group
+                                       = W.filter (`elem` W.integrate s) st
+              | otherwise = pure s
+    gs' = mapGroups (refocus <=< dead) gs
+    gset = S.fromList . concatMap W.integrate . M.elems $ gs'
+    -- after refocus, f is either the focused window of some group, or not in
+    -- gs' at all, so `lu f` is never Nothing
+    lu w | w `S.member` gset = w `M.lookup` gs'
+         | otherwise = Just (W.Stack w [] []) -- singleton groups for new wins
+
+mapGroups :: (Ord a) => (W.Stack a -> Maybe (W.Stack a)) -> Groups a -> Groups a
+mapGroups f = M.fromList . map (W.focus &&& id) . mapMaybe f . M.elems
 
 -- | focusWindow'. focus an element of a stack, is Nothing if that element is
 -- absent. See also 'W.focusWindow'
 focusWindow' :: (Eq a) => a -> W.Stack a -> Maybe (W.Stack a)
 focusWindow' w st = do
-    guard $ not $ null $ filter (w==) $ W.integrate st
-    if W.focus st == w then Just st
-        else focusWindow' w $ W.focusDown' st
+    guard $ w `elem` W.integrate st
+    return $ until ((w ==) . W.focus) W.focusDown' st
 
 -- update only when Just
 windowsMaybe :: (WindowSet -> Maybe WindowSet) -> X ()
