@@ -1,8 +1,9 @@
-{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses, TypeSynonymInstances, PatternGuards #-}
+{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, PatternGuards #-}
 
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.WindowNavigation
+-- Description :  A layout modifier to allow easy navigation of a workspace.
 -- Copyright   :  (c) 2007  David Roundy <droundy@darcs.net>
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -21,11 +22,11 @@ module XMonad.Layout.WindowNavigation (
                                    Navigate(..), Direction2D(..),
                                    MoveWindowToWindow(..),
                                    navigateColor, navigateBrightness,
-                                   noNavigateBorders, defaultWNConfig, def,
-                                   WNConfig, WindowNavigation,
+                                   noNavigateBorders, def, WNConfig,
+                                   WindowNavigation,
                                   ) where
 
-import Data.List ( nub, sortBy, (\\) )
+import XMonad.Prelude ( nub, sortBy, (\\) )
 import XMonad hiding (Point)
 import qualified XMonad.StackSet as W
 import XMonad.Layout.LayoutModifier
@@ -64,12 +65,11 @@ import XMonad.Util.XUtils
 -- "XMonad.Doc.Extending#Editing_key_bindings".
 
 
-data MoveWindowToWindow a = MoveWindowToWindow a a deriving ( Read, Show, Typeable )
+data MoveWindowToWindow a = MoveWindowToWindow a a deriving ( Read, Show)
 instance Typeable a => Message (MoveWindowToWindow a)
 
 data Navigate = Go Direction2D | Swap Direction2D | Move Direction2D
               | Apply (Window -> X()) Direction2D -- ^ Apply action with destination window
-        deriving ( Typeable )
 instance Message Navigate
 
 data WNConfig =
@@ -93,10 +93,6 @@ navigateBrightness f = def { brightness = Just $ max 0 $ min 1 f }
 
 instance Default WNConfig where def = WNC (Just 0.4) "#0000FF" "#00FFFF" "#FF0000" "#FF00FF"
 
-{-# DEPRECATED defaultWNConfig "Use def (from Data.Default, and re-exported by XMonad.Layout.WindowNavigation) instead." #-}
-defaultWNConfig :: WNConfig
-defaultWNConfig = def
-
 data NavigationState a = NS Point [(a,Rectangle)]
 
 data WindowNavigation a = WindowNavigation WNConfig (Invisible Maybe (NavigationState a)) deriving ( Read, Show )
