@@ -2,6 +2,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.WindowSwitcherDecoration
+-- Description :  Switch the position of windows by dragging them onto each other.
 -- Copyright   :  (c) Jan Vornberger 2009
 --                    Alejandro Serrano 2010
 -- License     :  BSD3-style (see LICENSE)
@@ -30,7 +31,7 @@ import XMonad.Layout.DecorationAddons
 import XMonad.Layout.ImageButtonDecoration
 import XMonad.Layout.DraggingVisualizer
 import qualified XMonad.StackSet as S
-import Control.Monad
+import XMonad.Prelude
 import Foreign.C.Types(CInt)
 
 -- $usage
@@ -75,7 +76,7 @@ windowSwitcherDecorationWithButtons :: (Eq a, Shrinker s) => s -> Theme
            -> l a -> ModifiedLayout (Decoration WindowSwitcherDecoration s) l a
 windowSwitcherDecorationWithButtons s c = decoration s c $ WSD True
 
-data WindowSwitcherDecoration a = WSD Bool deriving (Show, Read)
+newtype WindowSwitcherDecoration a = WSD Bool deriving (Show, Read)
 
 instance Eq a => DecorationStyle WindowSwitcherDecoration a where
     describeDeco _ = "WindowSwitcherDeco"
@@ -86,7 +87,7 @@ instance Eq a => DecorationStyle WindowSwitcherDecoration a where
     decorationWhileDraggingHook _ ex ey (mainw, r) x y = handleTiledDraggingInProgress ex ey (mainw, r) x y
     decorationAfterDraggingHook _ (mainw, _) decoWin = do focus mainw
                                                           hasCrossed <- handleScreenCrossing mainw decoWin
-                                                          unless hasCrossed $ do sendMessage $ DraggingStopped
+                                                          unless hasCrossed $ do sendMessage DraggingStopped
                                                                                  performWindowSwitching mainw
 
 -- Note: the image button code is duplicated from the above
@@ -96,7 +97,7 @@ windowSwitcherDecorationWithImageButtons :: (Eq a, Shrinker s) => s -> Theme
            -> l a -> ModifiedLayout (Decoration ImageWindowSwitcherDecoration s) l a
 windowSwitcherDecorationWithImageButtons s c = decoration s c $ IWSD True
 
-data ImageWindowSwitcherDecoration a = IWSD Bool deriving (Show, Read)
+newtype ImageWindowSwitcherDecoration a = IWSD Bool deriving (Show, Read)
 
 instance Eq a => DecorationStyle ImageWindowSwitcherDecoration a where
     describeDeco _ = "ImageWindowSwitcherDeco"
@@ -107,7 +108,7 @@ instance Eq a => DecorationStyle ImageWindowSwitcherDecoration a where
     decorationWhileDraggingHook _ ex ey (mainw, r) x y = handleTiledDraggingInProgress ex ey (mainw, r) x y
     decorationAfterDraggingHook _ (mainw, _) decoWin = do focus mainw
                                                           hasCrossed <- handleScreenCrossing mainw decoWin
-                                                          unless hasCrossed $ do sendMessage $ DraggingStopped
+                                                          unless hasCrossed $ do sendMessage DraggingStopped
                                                                                  performWindowSwitching mainw
 
 handleTiledDraggingInProgress :: CInt -> CInt -> (Window, Rectangle) -> Position -> Position -> X ()
@@ -126,13 +127,11 @@ performWindowSwitching win =
        ws <- gets windowset
        let allWindows = S.index ws
        -- do a little double check to be sure
-       if (win `elem` allWindows) && (selWin `elem` allWindows)
-            then do
+       when ((win `elem` allWindows) && (selWin `elem` allWindows)) $ do
                 let allWindowsSwitched = map (switchEntries win selWin) allWindows
                 let (ls, t:rs) = break (win ==) allWindowsSwitched
                 let newStack = S.Stack t (reverse ls) rs
-                windows $ S.modify' $ \_ -> newStack
-            else return ()
+                windows $ S.modify' $ const newStack
     where
         switchEntries a b x
             | x == a    = b
