@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Prompt.AppendFile
+-- Description :  A prompt for appending a single line of text to a file.
 -- Copyright   :  (c) 2007 Brent Yorgey
 -- License     :  BSD-style (see LICENSE)
 --
@@ -31,7 +32,6 @@ import XMonad.Core
 import XMonad.Prompt
 
 import System.IO
-import Control.Exception.Extensible (bracket)
 
 -- $usage
 --
@@ -60,7 +60,7 @@ import Control.Exception.Extensible (bracket)
 -- before saving into the file. Previous example with date can be rewritten as:
 --
 -- > ,  ((modm .|. controlMask, xK_n), do
--- >            date <- io $ liftM (formatTime defaultTimeLocale "[%Y-%m-%d %H:%M] ") getZonedTime
+-- >            date <- io $ fmap (formatTime defaultTimeLocale "[%Y-%m-%d %H:%M] ") getZonedTime
 -- >            appendFilePrompt' def (date ++) $ "/home/me/NOTES"
 -- >        )
 --
@@ -70,7 +70,7 @@ import Control.Exception.Extensible (bracket)
 -- For detailed instructions on editing your key bindings, see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
 
-data AppendFile = AppendFile FilePath
+newtype AppendFile = AppendFile FilePath
 
 instance XPrompt AppendFile where
     showXPrompt (AppendFile fn) = "Add to " ++ fn ++ ": "
@@ -78,7 +78,7 @@ instance XPrompt AppendFile where
 -- | Given an XPrompt configuration and a file path, prompt the user
 --   for a line of text, and append it to the given file.
 appendFilePrompt :: XPConfig -> FilePath -> X ()
-appendFilePrompt c fn = appendFilePrompt' c id fn
+appendFilePrompt c = appendFilePrompt' c id
 
 -- | Given an XPrompt configuration, string transformation function
 --   and a file path, prompt the user for a line of text, transform it
@@ -91,4 +91,4 @@ appendFilePrompt' c trans fn = mkXPrompt (AppendFile fn)
 
 -- | Append a string to a file.
 doAppend :: (String -> String) -> FilePath -> String -> X ()
-doAppend trans fn = io . bracket (openFile fn AppendMode) hClose . flip hPutStrLn . trans
+doAppend trans fn = io . withFile fn AppendMode . flip hPutStrLn . trans
