@@ -1,6 +1,7 @@
 --------------------------------------------------------------------------------
 -- |
 -- Module      : XMonad.Prompt.FuzzyMatch
+-- Description : A prompt for fuzzy completion matching in prompts akin to Emacs ido-mode.
 -- Copyright   : (C) 2015 Norbert Zeh
 -- License     : GPL
 --
@@ -18,9 +19,7 @@ module XMonad.Prompt.FuzzyMatch ( -- * Usage
                                 , fuzzySort
                                 ) where
 
-import Data.Char
-import Data.Function
-import Data.List
+import XMonad.Prelude
 
 -- $usage
 --
@@ -80,7 +79,8 @@ fuzzySort :: String -> [String] -> [String]
 fuzzySort q = map snd . sort . map (rankMatch q)
 
 rankMatch :: String -> String -> ((Int, Int), String)
-rankMatch q s = (minimum $ rankMatches q s, s)
+rankMatch q s = (if null matches then (maxBound, maxBound) else minimum matches, s)
+  where matches = rankMatches q s
 
 rankMatches :: String -> String -> [(Int, Int)]
 rankMatches [] _ = [(0, 0)]
