@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Prompt.Man
+-- Description :  A manual page prompt.
 -- Copyright   :  (c) 2007 Valery V. Vorotyntsev
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -25,6 +26,7 @@ module XMonad.Prompt.Man (
 
 
 import XMonad
+import XMonad.Prelude
 import XMonad.Prompt
 import XMonad.Util.Run
 import XMonad.Prompt.Shell (split)
@@ -33,10 +35,7 @@ import System.Directory
 import System.Process
 import System.IO
 
-import qualified Control.Exception.Extensible as E
-import Control.Monad
-import Data.List
-import Data.Maybe
+import qualified Control.Exception as E
 
 -- $usage
 -- 1. In your @~\/.xmonad\/xmonad.hs@:
@@ -60,7 +59,7 @@ instance XPrompt Man where
 manPrompt :: XPConfig -> X ()
 manPrompt c = do
   mans <- io getMans
-  mkXPrompt Man c (manCompl mans) $ runInTerm "" . (++) "man "
+  mkXPrompt Man c (manCompl c mans) $ runInTerm "" . (++) "man "
 
 getMans :: IO [String]
 getMans = do
@@ -75,17 +74,17 @@ getMans = do
   mans <- forM (nub dirs) $ \d -> do
             exists <- doesDirectoryExist d
             if exists
-              then map (stripExt . stripSuffixes [".gz", ".bz2"]) `fmap`
+              then map (stripExt . stripSuffixes [".gz", ".bz2"]) <$>
                    getDirectoryContents d
               else return []
   return $ uniqSort $ concat mans
 
-manCompl :: [String] -> String -> IO [String]
-manCompl mans s | s == "" || last s == ' ' = return []
-                | otherwise                = do
+manCompl :: XPConfig -> [String] -> String -> IO [String]
+manCompl c mans s | s == "" || last s == ' ' = return []
+                  | otherwise                = do
   -- XXX readline instead of bash's compgen?
-  f <- lines `fmap` getCommandOutput ("bash -c 'compgen -A file " ++ s ++ "'")
-  mkComplFunFromList (f ++ mans) s
+  f <- lines <$> getCommandOutput ("bash -c 'compgen -A file " ++ s ++ "'")
+  mkComplFunFromList c (f ++ mans) s
 
 -- | Run a command using shell and return its output.
 --
@@ -108,7 +107,7 @@ stripExt = reverse . drop 1 . dropWhile (/= '.') . reverse
 
 stripSuffixes :: Eq a => [[a]] -> [a] -> [a]
 stripSuffixes sufs fn =
-    head . catMaybes $ map (flip rstrip fn) sufs ++ [Just fn]
+    head . catMaybes $ map (`rstrip` fn) sufs ++ [Just fn]
 
 rstrip :: Eq a => [a] -> [a] -> Maybe [a]
 rstrip suf lst
