@@ -1,5 +1,6 @@
 {- |
 Module      :  XMonad.Prompt.Unicode
+Description :  A prompt for inputting Unicode characters.
 Copyright   :  (c) 2016 Joachim Breitner
                    2017 Nick Hu
 License     :  BSD-style (see LICENSE)
@@ -14,8 +15,6 @@ The provided @unicodePrompt@ and @typeUnicodePrompt@ use @xsel@ and @xdotool@
 respectively.
 -}
 
-{-# LANGUAGE DeriveDataTypeable #-}
-
 module XMonad.Prompt.Unicode (
  -- * Usage
  -- $usage
@@ -25,19 +24,14 @@ module XMonad.Prompt.Unicode (
  ) where
 
 import qualified Data.ByteString.Char8 as BS
-import Data.Char
-import Data.Maybe
-import Data.Ord
 import Numeric
-import System.Environment
 import System.IO
-import System.IO.Unsafe
 import System.IO.Error
-import Control.Arrow
-import Data.List
 import Text.Printf
+import Control.Arrow (second)
 
 import XMonad
+import XMonad.Prelude
 import qualified XMonad.Util.ExtensibleState as XS
 import XMonad.Util.Run
 import XMonad.Prompt
@@ -49,7 +43,7 @@ instance XPrompt Unicode where
   nextCompletion Unicode = getNextCompletion
 
 newtype UnicodeData = UnicodeData { getUnicodeData :: [(Char, BS.ByteString)] }
-  deriving (Typeable, Read, Show)
+  deriving (Read, Show)
 
 instance ExtensionClass UnicodeData where
   initialValue = UnicodeData []
@@ -85,7 +79,7 @@ populateEntries unicodeDataFilename = do
           hPutStrLn stderr "Do you have unicode-data installed?"
           return False
         Right dat -> do
-          XS.put . UnicodeData . sortBy (comparing (BS.length . snd)) $ parseUnicodeData dat
+          XS.put . UnicodeData . sortOn (BS.length . snd) $ parseUnicodeData dat
           return True
     else return True
 
@@ -96,24 +90,33 @@ parseUnicodeData = mapMaybe parseLine . BS.lines
           [(c,"")] <- return . readHex $ BS.unpack field1
           return (chr c, field2)
 
-searchUnicode :: [(Char, BS.ByteString)] -> String -> [(Char, String)]
-searchUnicode entries s = map (second BS.unpack) $ filter go entries
-  where w = map BS.pack . filter (all isAscii) . filter ((> 1) . length) . words $ map toUpper s
-        go (c,d) = all (`BS.isInfixOf` d) w
+type Predicate = String -> String -> Bool
+
+searchUnicode :: [(Char, BS.ByteString)] -> Predicate -> String -> [(Char, String)]
+searchUnicode entries p s = map (second BS.unpack) $ filter go entries
+  where w = filter (all isAscii) . filter ((> 1) . length) . words $ map toUpper s
+        go (_, d) = all (`p` BS.unpack d) w
 
 mkUnicodePrompt :: String -> [String] -> String -> XPConfig -> X ()
-mkUnicodePrompt prog args unicodeDataFilename config =
+mkUnicodePrompt prog args unicodeDataFilename xpCfg =
   whenX (populateEntries unicodeDataFilename) $ do
     entries <- fmap getUnicodeData (XS.get :: X UnicodeData)
-    mkXPrompt Unicode config (unicodeCompl entries) paste
+    mkXPrompt
+      Unicode
+      (xpCfg {sorter = sorter xpCfg . map toUpper})
+      (unicodeCompl entries $ searchPredicate xpCfg)
+      paste
   where
-    unicodeCompl _ [] = return []
-    unicodeCompl entries s = do
-      let m = searchUnicode entries s
+    unicodeCompl :: [(Char, BS.ByteString)] -> Predicate -> String -> IO [String]
+    unicodeCompl _ _ "" = return []
+    unicodeCompl entries p s = do
+      let m = searchUnicode entries p s
       return . map (\(c,d) -> printf "%s %s" [c] d) $ take 20 m
     paste [] = return ()
-    paste (c:_) = do
-      runProcessWithInput prog args [c]
+    paste (c:_) = liftIO $ do
+      handle <- spawnPipe $ unwords $ prog : args
+      hPutChar handle c
+      hClose handle
       return ()
 
 -- | Prompt the user for a Unicode character to be inserted into the paste buffer of the X server.
