@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.DebugWindow
+-- Description :  Dump window information for diagnostic\/debugging purposes.
 -- Copyright   :  (c) Brandon S Allbery KF8NH, 2014
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -8,7 +9,7 @@
 -- Stability   :  unstable
 -- Portability :  not portable
 --
--- Module to dump window information for diagnostic/debugging purposes. See 
+-- Module to dump window information for diagnostic/debugging purposes. See
 -- "XMonad.Hooks.DebugEvents" and "XMonad.Hooks.DebugStack" for practical uses.
 --
 -----------------------------------------------------------------------------
@@ -18,13 +19,10 @@ module XMonad.Util.DebugWindow (debugWindow) where
 import           Prelude
 
 import           XMonad
+import           XMonad.Prelude
 
 import           Codec.Binary.UTF8.String        (decodeString)
-import           Control.Exception.Extensible                          as E
-import           Control.Monad                   (when)
-import           Data.List                       (unfoldr
-                                                 ,intercalate
-                                                 )
+import           Control.Exception                                     as E
 import           Foreign
 import           Foreign.C.String
 import           Numeric                         (showHex)
@@ -41,7 +39,7 @@ debugWindow w =  do
   case w' of
     Nothing                                   ->
       return $ "(deleted window " ++ wx ++ ")"
-    Just (WindowAttributes
+    Just WindowAttributes
       { wa_x                 = x
       , wa_y                 = y
       , wa_width             = wid
@@ -49,7 +47,7 @@ debugWindow w =  do
       , wa_border_width      = bw
       , wa_map_state         = m
       , wa_override_redirect = o
-      }) -> do
+      } -> do
       c' <- withDisplay $ \d ->
             io (getWindowProperty8 d wM_CLASS w)
       let c = case c' of
@@ -63,9 +61,9 @@ debugWindow w =  do
                                                           then s''
                                                           else tail s''
                                           in Just (w'',s')
-      t <- catchX' (wrap `fmap` getEWMHTitle  "VISIBLE" w) $
-           catchX' (wrap `fmap` getEWMHTitle  ""        w) $
-           catchX' (wrap `fmap` getICCCMTitle           w) $
+      t <- catchX' (wrap <$> getEWMHTitle  "VISIBLE" w) $
+           catchX' (wrap <$> getEWMHTitle  ""        w) $
+           catchX' (wrap <$> getICCCMTitle           w) $
            return ""
       h' <- getMachine w
       let h = if null h' then "" else '@':h'
@@ -73,7 +71,7 @@ debugWindow w =  do
       -- NB. modern stuff often does not set WM_COMMAND since it's only ICCCM required and not some
       -- horrible gnome/freedesktop session manager thing like Wayland intended. How helpful of them.
       p' <- withDisplay $ \d -> safeGetCommand d w
-      let p = if null p' then "" else wrap $ intercalate " " p'
+      let p = if null p' then "" else wrap $ unwords p'
       nWP <- getAtom "_NET_WM_PID"
       pid' <- withDisplay $ \d -> io $ getWindowProperty32 d nWP w
       let pid = case pid' of
@@ -96,7 +94,7 @@ debugWindow w =  do
                       ,show x
                       ,',':show y
                       ,if null c then "" else ' ':c
-                      ,if null cmd then "" else ' ':cmd 
+                      ,if null cmd then "" else ' ':cmd
                       ,rb
                       ]
 
@@ -114,19 +112,19 @@ getDecodedStringProp w a =  do
   t@(TextProperty t' _ 8 _) <- withDisplay $ \d -> io $ getTextProperty d w a
   [s] <- catchX' (tryUTF8     t) $
          catchX' (tryCompound t) $
-         io ((:[]) `fmap` peekCString t')
+         io ((:[]) <$> peekCString t')
   return s
 
 tryUTF8                          :: TextProperty -> X [String]
 tryUTF8 (TextProperty s enc _ _) =  do
   uTF8_STRING <- getAtom "UTF8_STRING"
-  when (enc == uTF8_STRING) $ error "String is not UTF8_STRING"
-  (map decodeString . splitNul) `fmap` io (peekCString s)
+  when (enc /= uTF8_STRING) $ error "String is not UTF8_STRING"
+  map decodeString . splitNul <$> io (peekCString s)
 
 tryCompound                            :: TextProperty -> X [String]
 tryCompound t@(TextProperty _ enc _ _) =  do
   cOMPOUND_TEXT <- getAtom "COMPOUND_TEXT"
-  when (enc == cOMPOUND_TEXT) $ error "String is not COMPOUND_TEXT"
+  when (enc /= cOMPOUND_TEXT) $ error "String is not COMPOUND_TEXT"
   withDisplay $ \d -> io $ wcTextPropertyToTextList d t
 
 splitNul    :: String -> [String]
@@ -143,7 +141,7 @@ catchX' job errcase = do
   c <- ask
   (a, s') <- io $ runX c st job `E.catch` \e -> case fromException e of
     Just x -> throw e `const` (x `asTypeOf` ExitSuccess)
-    _ -> runX c st errcase
+    _      -> runX c st errcase
   put s'
   return a
 
@@ -161,7 +159,7 @@ safeGetWindowAttributes d w =  alloca $ \p -> do
   s <- xGetWindowAttributes d w p
   case s of
     0 -> return Nothing
-    _ -> Just `fmap` peek p
+    _ -> Just <$> peek p
 
 -- and so is getCommand
 safeGetCommand     :: Display -> Window -> X [String]
