@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.Dmenu
+-- Description :  Convenient bindings to dmenu.
 -- Copyright   :  (c) Spencer Janssen <spencerjanssen@gmail.com>
 -- License     :  BSD-style (see LICENSE)
 --
@@ -24,7 +25,6 @@ import XMonad
 import qualified XMonad.StackSet as W
 import qualified Data.Map as M
 import XMonad.Util.Run
-import Control.Monad (liftM)
 
 -- $usage
 -- You can use this module with the following in your Config.hs file:
@@ -43,27 +43,27 @@ import Control.Monad (liftM)
 dmenuXinerama :: [String] -> X String
 dmenuXinerama opts = do
     curscreen <-
-      (fromIntegral . W.screen . W.current) `fmap` gets windowset :: X Int
+      fromIntegral . W.screen . W.current <$> gets windowset :: X Int
     _ <-
       runProcessWithInput "dmenu" ["-xs", show (curscreen+1)] (unlines opts)
     menuArgs "dmenu" ["-xs", show (curscreen+1)] opts
 
 -- | Run dmenu to select an option from a list.
 dmenu :: MonadIO m => [String] -> m String
-dmenu opts = menu "dmenu" opts
+dmenu = menu "dmenu"
 
 -- | like 'dmenu' but also takes the command to run.
 menu :: MonadIO m => String -> [String] -> m String
-menu menuCmd opts = menuArgs menuCmd [] opts
+menu menuCmd = menuArgs menuCmd []
 
 -- | Like 'menu' but also takes a list of command line arguments.
 menuArgs :: MonadIO m => String -> [String] -> [String] -> m String
-menuArgs menuCmd args opts = liftM (filter (/='\n')) $
+menuArgs menuCmd args opts = filter (/='\n') <$>
   runProcessWithInput menuCmd args (unlines opts)
 
 -- | Like 'dmenuMap' but also takes the command to run.
 menuMap :: MonadIO m => String -> M.Map String a -> m (Maybe a)
-menuMap menuCmd selectionMap = menuMapArgs menuCmd [] selectionMap
+menuMap menuCmd = menuMapArgs menuCmd []
 
 -- | Like 'menuMap' but also takes a list of command line arguments.
 menuMapArgs :: MonadIO m => String -> [String] -> M.Map String a ->
@@ -76,4 +76,4 @@ menuMapArgs menuCmd args selectionMap = do
 
 -- | Run dmenu to select an entry from a map based on the key.
 dmenuMap :: MonadIO m => M.Map String a -> m (Maybe a)
-dmenuMap selectionMap = menuMap "dmenu" selectionMap
+dmenuMap = menuMap "dmenu"
