@@ -1,6 +1,7 @@
 --------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.EZConfig
+-- Description :  Configure key bindings easily in Emacs style.
 -- Copyright   :  Devin Mullins <me@twifkak.com>
 --                Brent Yorgey <byorgey@gmail.com> (key parsing)
 -- License     :  BSD3-style (see LICENSE)
@@ -29,19 +30,22 @@ module XMonad.Util.EZConfig (
                              mkKeymap, checkKeymap,
                              mkNamedKeymap,
 
-                             parseKey -- used by XMonad.Util.Paste
+                             -- * Parsers
+
+                             parseKey, -- used by XMonad.Util.Paste
+                             parseKeyCombo,
+                             parseKeySequence, readKeySequence
                             ) where
 
 import XMonad
 import XMonad.Actions.Submap
+import XMonad.Prelude hiding (many)
 
 import XMonad.Util.NamedActions
 
-import qualified Data.Map as M
-import Data.List (foldl', sortBy, groupBy, nub)
-import Data.Ord (comparing)
-import Data.Maybe
 import Control.Arrow (first, (&&&))
+import qualified Data.Map as M
+import Data.Ord (comparing)
 
 import Text.ParserCombinators.ReadP
 
@@ -82,7 +86,7 @@ import Text.ParserCombinators.ReadP
 -- whichever), or add your own @myModMask = mod1Mask@ line.
 additionalKeys :: XConfig a -> [((KeyMask, KeySym), X ())] -> XConfig a
 additionalKeys conf keyList =
-    conf { keys = \cnf -> M.union (M.fromList keyList) (keys conf cnf) }
+    conf { keys = M.union (M.fromList keyList) . keys conf }
 
 -- | Like 'additionalKeys', except using short @String@ key
 --   descriptors like @\"M-m\"@ instead of @(modMask, xK_m)@, as
@@ -105,7 +109,7 @@ additionalKeysP conf keyList =
 -- >                 `removeKeys` [(mod1Mask .|. shiftMask, n) | n <- [xK_1 .. xK_9]]
 removeKeys :: XConfig a -> [(KeyMask, KeySym)] -> XConfig a
 removeKeys conf keyList =
-    conf { keys = \cnf -> keys conf cnf `M.difference` M.fromList (zip keyList $ repeat ()) }
+    conf { keys = \cnf -> foldr M.delete (keys conf cnf) keyList }
 
 -- | Like 'removeKeys', except using short @String@ key descriptors
 --   like @\"M-m\"@ instead of @(modMask, xK_m)@, as described in the
@@ -121,13 +125,12 @@ removeKeysP conf keyList =
 -- | Like 'additionalKeys', but for mouse bindings.
 additionalMouseBindings :: XConfig a -> [((ButtonMask, Button), Window -> X ())] -> XConfig a
 additionalMouseBindings conf mouseBindingsList =
-    conf { mouseBindings = \cnf -> M.union (M.fromList mouseBindingsList) (mouseBindings conf cnf) }
+    conf { mouseBindings = M.union (M.fromList mouseBindingsList) . mouseBindings conf }
 
 -- | Like 'removeKeys', but for mouse bindings.
 removeMouseBindings :: XConfig a -> [(ButtonMask, Button)] -> XConfig a
 removeMouseBindings conf mouseBindingList =
-    conf { mouseBindings = \cnf -> mouseBindings conf cnf `M.difference`
-                                   M.fromList (zip mouseBindingList $ repeat ()) }
+    conf { mouseBindings = \cnf -> foldr M.delete (mouseBindings conf cnf) mouseBindingList }
 
 
 --------------------------------------------------------------
@@ -352,6 +355,7 @@ removeMouseBindings conf mouseBindingList =
 -- > <XF86_ClearGrab>
 -- > <XF86_Next_VMode>
 -- > <XF86_Prev_VMode>
+-- > <XF86Bluetooth>
 
 mkKeymap :: XConfig l -> [(String, X ())] -> M.Map (KeyMask, KeySym) (X ())
 mkKeymap c = M.fromList . mkSubmaps . readKeymap c
@@ -378,9 +382,6 @@ mkSubmaps' subm binds = map combine gathered
                       subm . mkSubmaps' subm $ map (first tail) ks)
         fstKey = (==) `on` (head . fst)
 
-on :: (a -> a -> b) -> (c -> a) -> c -> c -> b
-op `on` f = \x y -> f x `op` f y
-
 -- | Given a configuration record and a list of (key sequence
 --   description, action) pairs, parse the key sequences into lists of
 --   @(KeyMask,KeySym)@ pairs.  Key sequences which fail to parse will
@@ -673,7 +674,8 @@ multimediaKeys = filter ((/= noSymbol) . snd) . map (id &&& stringToKeysym) $
                  , "XF86_Ungrab"
                  , "XF86_ClearGrab"
                  , "XF86_Next_VMode"
-                 , "XF86_Prev_VMode" ]
+                 , "XF86_Prev_VMode"
+                 , "XF86Bluetooth" ]
 
 -- | Given a configuration record and a list of (key sequence
 --   description, action) pairs, check the key sequence descriptions
@@ -709,9 +711,9 @@ multimediaKeys = filter ((/= noSymbol) . snd) . map (id &&& stringToKeysym) $
 checkKeymap :: XConfig l -> [(String, a)] -> X ()
 checkKeymap conf km = warn (doKeymapCheck conf km)
   where warn ([],[])   = return ()
-        warn (bad,dup) = spawn $ "xmessage 'Warning:\n"
+        warn (bad,dup) = xmessage $ "Warning:\n"
                             ++ msg "bad" bad ++ "\n"
-                            ++ msg "duplicate" dup ++ "'"
+                            ++ msg "duplicate" dup
         msg _ [] = ""
         msg m xs = m ++ " keybindings detected: " ++ showBindings xs
         showBindings = unwords . map (("\""++) . (++"\""))
