@@ -1,7 +1,9 @@
+{-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE PatternGuards #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.ExtensibleState
+-- Description :  Module for storing custom mutable state in xmonad.
 -- Copyright   :  (c) Daniel Schoepe 2009
 -- License     :  BSD3-style (see LICENSE)
 --
@@ -22,6 +24,7 @@ module XMonad.Util.ExtensibleState (
                               , get
                               , gets
                               , modified
+                              , modifiedM
                               ) where
 
 import Data.Typeable (typeOf,cast)
@@ -29,7 +32,7 @@ import qualified Data.Map as M
 import XMonad.Core
 import XMonad.Util.PureX
 import qualified Control.Monad.State as State
-import Data.Maybe (fromMaybe)
+import XMonad.Prelude (fromMaybe)
 
 -- ---------------------------------------------------------------------
 -- $usage
@@ -38,10 +41,9 @@ import Data.Maybe (fromMaybe)
 -- and make it an instance of ExtensionClass. You can then use
 -- the functions from this module for storing and retrieving your data:
 --
--- > {-# LANGUAGE DeriveDataTypeable #-}
 -- > import qualified XMonad.Util.ExtensibleState as XS
 -- >
--- > data ListStorage = ListStorage [Integer] deriving Typeable
+-- > data ListStorage = ListStorage [Integer]
 -- > instance ExtensionClass ListStorage where
 -- >   initialValue = ListStorage []
 -- >
@@ -59,7 +61,7 @@ import Data.Maybe (fromMaybe)
 -- To make your data persistent between restarts, the data type needs to be
 -- an instance of Read and Show and the instance declaration has to be changed:
 --
--- > data ListStorage = ListStorage [Integer] deriving (Typeable,Read,Show)
+-- > data ListStorage = ListStorage [Integer] deriving (Read,Show)
 -- >
 -- > instance ExtensionClass ListStorage where
 -- >   initialValue = ListStorage []
@@ -97,7 +99,7 @@ put v = modifyStateExts . M.insert (show . typeOf $ v) . Right . extensionType $
 -- | Try to retrieve a value of the requested type, return an initial value if there is no such value.
 get :: (ExtensionClass a, XLike m) => m a
 get = getState' undefined -- `trick' to avoid needing -XScopedTypeVariables
-  where toValue val = maybe initialValue id $ cast val
+  where toValue val = fromMaybe initialValue $ cast val
         getState' :: (ExtensionClass a, XLike m) => a -> m a
         getState' k = do
           v <- State.gets $ M.lookup (show . typeOf $ k) . extensibleState
@@ -108,7 +110,7 @@ get = getState' undefined -- `trick' to avoid needing -XScopedTypeVariables
                 let val = fromMaybe initialValue $ cast =<< safeRead str `asTypeOf` Just x
                 put (val `asTypeOf` k)
                 return val
-            _ -> return $ initialValue
+            _ -> return initialValue
         safeRead str = case reads str of
                          [(x,"")] -> Just x
                          _ -> Nothing
@@ -121,8 +123,11 @@ remove :: (ExtensionClass a, XLike m) => a -> m ()
 remove wit = modifyStateExts $ M.delete (show . typeOf $ wit)
 
 modified :: (ExtensionClass a, Eq a, XLike m) => (a -> a) -> m Bool
-modified f = do
+modified = modifiedM . (pure .)
+
+modifiedM :: (ExtensionClass a, Eq a, XLike m) => (a -> m a) -> m Bool
+modifiedM f = do
     v <- get
-    case f v of
+    f v >>= \case
         v' | v' == v   -> return False
            | otherwise -> put v' >> return True
