@@ -1,7 +1,7 @@
-{-# LANGUAGE PatternGuards #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.NamedScratchpad
+-- Description :  Toggle arbitrary windows to and from the current workspace.
 -- Copyright   :  (c) Konstantin Sobolev <konstantin.sobolev@gmail.com>
 -- License     :  BSD-style (see LICENSE)
 --
@@ -17,24 +17,30 @@ module XMonad.Util.NamedScratchpad (
   -- * Usage
   -- $usage
   NamedScratchpad(..),
+  scratchpadWorkspaceTag,
   nonFloating,
   defaultFloating,
   customFloating,
   NamedScratchpads,
   namedScratchpadAction,
+  spawnHereNamedScratchpadAction,
+  customRunNamedScratchpadAction,
   allNamedScratchpadAction,
   namedScratchpadManageHook,
   namedScratchpadFilterOutWorkspace,
-  namedScratchpadFilterOutWorkspacePP
+  namedScratchpadFilterOutWorkspacePP,
+  nsHideOnFocusLoss,
   ) where
 
 import XMonad
-import XMonad.Hooks.ManageHelpers (doRectFloat)
 import XMonad.Actions.DynamicWorkspaces (addHiddenWorkspace)
-import XMonad.Hooks.DynamicLog (PP, ppSort)
+import XMonad.Actions.SpawnOn (spawnHere)
+import XMonad.Hooks.StatusBar.PP (PP, ppSort)
+import XMonad.Hooks.ManageHelpers (doRectFloat)
+import XMonad.Hooks.RefocusLast (withRecentsIn)
+import XMonad.Prelude (filterM, find, unless, when)
 
-import Control.Monad (filterM)
-import Data.Maybe (listToMaybe)
+import qualified Data.List.NonEmpty as NE
 
 import qualified XMonad.StackSet as W
 
@@ -84,6 +90,19 @@ import qualified XMonad.StackSet as W
 -- For detailed instruction on editing the key binding see
 -- "XMonad.Doc.Extending#Editing_key_bindings"
 --
+-- For some applications (like displaying your workspaces in a status bar) it
+-- is convenient to filter out the @NSP@ workspace when looking at all
+-- workspaces. For this, you can use 'XMonad.Hooks.StatusBar.PP.filterOutWsPP',
+-- or 'XMonad.Util.WorkspaceCompare.filterOutWs' together with
+-- 'XMonad.Hooks.EwmhDesktops.addEwmhWorkspaceSort' if your status bar gets
+-- the list of workspaces from EWMH.  See the documentation of these functions
+-- for examples.
+--
+-- Further, there is also a @logHook@ that you can use to hide
+-- scratchpads when they lose focus; this is functionality akin to what
+-- some dropdown terminals provide.  See the documentation of
+-- 'nsHideOnFocusLoss' for an example how to set this up.
+--
 
 -- | Single named scratchpad configuration
 data NamedScratchpad = NS { name   :: String      -- ^ Scratchpad name
@@ -109,46 +128,90 @@ type NamedScratchpads = [NamedScratchpad]
 
 -- | Finds named scratchpad configuration by name
 findByName :: NamedScratchpads -> String -> Maybe NamedScratchpad
-findByName c s = listToMaybe $ filter ((s==) . name) c
+findByName c s = find ((s ==) . name) c
 
 -- | Runs application which should appear in specified scratchpad
 runApplication :: NamedScratchpad -> X ()
 runApplication = spawn . cmd
 
+-- | Runs application which should appear in a specified scratchpad on the workspace it was launched on
+runApplicationHere :: NamedScratchpad -> X ()
+runApplicationHere = spawnHere . cmd
+
 -- | Action to pop up specified named scratchpad
 namedScratchpadAction :: NamedScratchpads -- ^ Named scratchpads configuration
                       -> String           -- ^ Scratchpad name
                       -> X ()
-namedScratchpadAction = someNamedScratchpadAction (\f ws -> f $ head ws)
+namedScratchpadAction = customRunNamedScratchpadAction runApplication
+
+-- | Action to pop up specified named scratchpad, initially starting it on the current workspace.
+spawnHereNamedScratchpadAction :: NamedScratchpads           -- ^ Named scratchpads configuration
+                               -> String                     -- ^ Scratchpad name
+                               -> X ()
+spawnHereNamedScratchpadAction = customRunNamedScratchpadAction runApplicationHere
+
+-- | Action to pop up specified named scratchpad, given a custom way to initially start the application.
+customRunNamedScratchpadAction :: (NamedScratchpad -> X ())  -- ^ Function initially running the application, given the configured @scratchpad@ cmd
+                               -> NamedScratchpads           -- ^ Named scratchpads configuration
+                               -> String                     -- ^ Scratchpad name
+                               -> X ()
+customRunNamedScratchpadAction = someNamedScratchpadAction (\f ws -> f $ NE.head ws)
 
 allNamedScratchpadAction :: NamedScratchpads
                          -> String
                          -> X ()
-allNamedScratchpadAction = someNamedScratchpadAction mapM_
+allNamedScratchpadAction = someNamedScratchpadAction mapM_ runApplication
 
-someNamedScratchpadAction :: ((Window -> X ()) -> [Window] -> X ())
+-- | A @logHook@ to hide scratchpads when they lose focus.  This can be
+-- useful for e.g. dropdown terminals.  Note that this also requires you
+-- to use the 'XMonad.Hooks.RefocusLast.refocusLastLogHook'.
+--
+-- ==== __Example__
+--
+-- > import XMonad.Hooks.RefocusLast (refocusLastLogHook)
+-- > import XMonad.Util.NamedScratchpad
+-- >
+-- > main = xmonad $ def
+-- >   { logHook = refocusLastLogHook
+-- >            >> nsHideOnFocusLoss myScratchpads
+-- >               -- enable hiding for all of @myScratchpads@
+-- >   }
+nsHideOnFocusLoss :: NamedScratchpads -> X ()
+nsHideOnFocusLoss scratches = withWindowSet $ \winSet -> do
+    let cur = W.currentTag winSet
+    withRecentsIn cur () $ \lastFocus _ ->
+        when (lastFocus `elem` W.index winSet && cur /= scratchpadWorkspaceTag) $
+            whenX (isNS lastFocus) $
+                shiftToNSP (W.workspaces winSet) ($ lastFocus)
+  where
+    isNS :: Window -> X Bool
+    isNS w = or <$> traverse ((`runQuery` w) . query) scratches
+
+-- | execute some action on a named scratchpad
+someNamedScratchpadAction :: ((Window -> X ()) -> NE.NonEmpty Window -> X ())
+                          -> (NamedScratchpad -> X ())
                           -> NamedScratchpads
                           -> String
                           -> X ()
-someNamedScratchpadAction f confs n
-    | Just conf <- findByName confs n = withWindowSet $ \s -> do
-                     filterCurrent <- filterM (runQuery (query conf))
-                                        ((maybe [] W.integrate . W.stack . W.workspace . W.current) s)
-                     filterAll <- filterM (runQuery (query conf)) (W.allWindows s)
-                     case filterCurrent of
-                       [] -> do
-                         case filterAll of
-                           [] -> runApplication conf
-                           _  -> f (windows . W.shiftWin (W.currentTag s)) filterAll
-                       _ -> do
-                         if null (filter ((== scratchpadWorkspaceTag) . W.tag) (W.workspaces s))
-                             then addHiddenWorkspace scratchpadWorkspaceTag
-                             else return ()
-                         f (windows . W.shiftWin scratchpadWorkspaceTag) filterAll
-    | otherwise = return ()
+someNamedScratchpadAction f runApp scratchpadConfig scratchpadName =
+    case findByName scratchpadConfig scratchpadName of
+        Just conf -> withWindowSet $ \winSet -> do
+            let focusedWspWindows = maybe [] W.integrate (W.stack . W.workspace . W.current $ winSet)
+                allWindows        = W.allWindows winSet
+            matchingOnCurrent <- filterM (runQuery (query conf)) focusedWspWindows
+            matchingOnAll     <- filterM (runQuery (query conf)) allWindows
 
+            case NE.nonEmpty matchingOnCurrent of
+                -- no matching window on the current workspace -> scratchpad not running or in background
+                Nothing -> case NE.nonEmpty matchingOnAll of
+                    Nothing   -> runApp conf
+                    Just wins -> f (windows . W.shiftWin (W.currentTag winSet)) wins
 
--- tag of the scratchpad workspace
+                -- matching window running on current workspace -> window should be shifted to scratchpad workspace
+                Just wins -> shiftToNSP (W.workspaces winSet) (`f` wins)
+        Nothing -> return ()
+
+-- | Tag of the scratchpad workspace
 scratchpadWorkspaceTag :: String
 scratchpadWorkspaceTag = "NSP"
 
@@ -157,10 +220,19 @@ namedScratchpadManageHook :: NamedScratchpads -- ^ Named scratchpads configurati
                           -> ManageHook
 namedScratchpadManageHook = composeAll . fmap (\c -> query c --> hook c)
 
+-- | Shift some windows to the scratchpad workspace according to the
+-- given function.  The workspace is created if necessary.
+shiftToNSP :: [WindowSpace] -> ((Window -> X ()) -> X ()) -> X ()
+shiftToNSP ws f = do
+    unless (any ((scratchpadWorkspaceTag ==) . W.tag) ws) $
+        addHiddenWorkspace scratchpadWorkspaceTag
+    f (windows . W.shiftWin scratchpadWorkspaceTag)
+
 -- | Transforms a workspace list containing the NSP workspace into one that
 -- doesn't contain it. Intended for use with logHooks.
 namedScratchpadFilterOutWorkspace :: [WindowSpace] -> [WindowSpace]
 namedScratchpadFilterOutWorkspace = filter (\(W.Workspace tag _ _) -> tag /= scratchpadWorkspaceTag)
+{-# DEPRECATED namedScratchpadFilterOutWorkspace "Use XMonad.Util.WorkspaceCompare.filterOutWs [scratchpadWorkspaceTag] instead" #-}
 
 -- | Transforms a pretty-printer into one not displaying the NSP workspace.
 --
@@ -177,5 +249,6 @@ namedScratchpadFilterOutWorkspacePP :: PP -> PP
 namedScratchpadFilterOutWorkspacePP pp = pp {
   ppSort = fmap (. namedScratchpadFilterOutWorkspace) (ppSort pp)
   }
+{-# DEPRECATED namedScratchpadFilterOutWorkspacePP "Use XMonad.Hooks.StatusBar.PP.filterOutWsPP [scratchpadWorkspaceTag] instead" #-}
 
 -- vim:ts=4:shiftwidth=4:softtabstop=4:expandtab:foldlevel=20:
