@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.Scratchpad
+-- Description :  Very handy hotkey-launched toggleable floating terminal window.
 -- Copyright   :  (c) Braden Shepherdson 2008
 -- License     :  BSD-style (as xmonad)
 --
@@ -26,6 +27,7 @@ module XMonad.Util.Scratchpad (
 import XMonad
 import qualified XMonad.StackSet as W
 import XMonad.Util.NamedScratchpad
+import XMonad.Util.WorkspaceCompare (filterOutWs)
 
 
 -- $usage
@@ -109,12 +111,12 @@ scratchpadManageHook rect = namedScratchpadManageHook [NS "" "" scratchpadQuery
 
 
 -- | Transforms a workspace list containing the SP workspace into one that
--- doesn't contain it. Intended for use with logHooks.
+-- doesn't contain it. Intended for use with 'logHook's (see
+-- 'XMonad.Hooks.StatusBar.PP.filterOutWsPP') and "XMonad.Hooks.EwmhDesktops"
+-- (see 'XMonad.Hooks.EwmhDesktops.addEwmhWorkspaceSort').
 scratchpadFilterOutWorkspace :: [WindowSpace] -> [WindowSpace]
-scratchpadFilterOutWorkspace = namedScratchpadFilterOutWorkspace
+scratchpadFilterOutWorkspace = filterOutWs [scratchpadWorkspaceTag]
 
 
 scratchpadDefaultRect :: W.RationalRect
 scratchpadDefaultRect = W.RationalRect 0.25 0.375 0.5 0.25
-
-
