@@ -1,6 +1,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Util.XUtils
+-- Description :  A module for painting on the screen.
 -- Copyright   :  (c) 2007 Andrea Rossato
 --                    2010 Alejandro Serrano
 -- License     :  BSD-style (see xmonad/LICENSE)
@@ -32,11 +33,10 @@ module XMonad.Util.XUtils
     , fi
     ) where
 
-import Data.Maybe
+import XMonad.Prelude
 import XMonad
 import XMonad.Util.Font
 import XMonad.Util.Image
-import Control.Monad
 
 -- $usage
 -- See "XMonad.Layout.Tabbed" or "XMonad.Layout.DragPane" or
@@ -128,8 +128,8 @@ paintAndWrite :: Window     -- ^ The window where to draw
               -> X ()
 paintAndWrite w fs wh ht bw bc borc ffc fbc als strs = do
     d <- asks display
-    strPositions <- forM (zip als strs) $ \(al, str) ->
-        stringPosition d fs (Rectangle 0 0 wh ht) al str
+    strPositions <- forM (zip als strs) $
+        uncurry (stringPosition d fs (Rectangle 0 0 wh ht))
     let ms = Just (fs,ffc,fbc, zip strs strPositions)
     paintWindow' w (Rectangle 0 0 wh ht) bw bc borc ms Nothing
 
@@ -151,9 +151,8 @@ paintTextAndIcons :: Window      -- ^ The window where to draw
                   -> X ()
 paintTextAndIcons w fs wh ht bw bc borc ffc fbc als strs i_als icons = do
     d <- asks display
-    strPositions <- forM (zip als strs) $ \(al, str) ->
-        stringPosition d fs (Rectangle 0 0 wh ht) al str
-    let iconPositions = map ( \(al, icon) -> iconPosition (Rectangle 0 0 wh ht) al icon ) (zip i_als icons)
+    strPositions <- forM (zip als strs) $ uncurry (stringPosition d fs (Rectangle 0 0 wh ht))
+    let iconPositions = zipWith (iconPosition (Rectangle 0 0 wh ht)) i_als icons
         ms = Just (fs,ffc,fbc, zip strs strPositions)
         is = Just (ffc, fbc, zip iconPositions icons)
     paintWindow' w (Rectangle 0 0 wh ht) bw bc borc ms is
